#ifndef _CONFIG_CPP_
#define _CONFIG_CPP_

#include "../../tinyxml/tinyxml.h"
#include "../spirits.hpp"
#include "config.hpp"
#include "path.hpp"
#include "log.hpp"

Config::Config()
{
    gameLanguage = NULL;
}

Config::~Config()
{
    if (gameLanguage!=NULL) delete gameLanguage;
}

int  Config::getNumber(int stackindex,int minvalue,int maxvalue,int def)
{
    int value=def;
    lua_State *ls = ioLua.getHandler();
    if (lua_isnumber(ls,stackindex))
    {
        value = lua_tointeger(ls,stackindex);
        if (value<minvalue) value=minvalue;
        if (value>maxvalue) value=maxvalue;
    }
    return value;
}

TMAString Config::getString(int stackindex,const char *def)
{
    TMAString str;
    lua_State *ls = ioLua.getHandler();
    const char * s = lua_tostring(ls,stackindex);
    str=(!s)?def:s;
    return str;
}

void Config::setConfigDefaults(void)
{
    fullscreen = (DFLT_FULLSCREEN!=0);
    music_vol  = DFLT_MUSIC_VOL;
    sfx_vol    = DFLT_SFX_VOL;
    surround   = (DFLT_SURROUND!=0);
    
    sfx_on = TMAGame::isSoundAvailable();
    music_on = TMAGame::isSoundAvailable();
    
    gameKeys[TSpiritsGame::K_ABORT]=SDLK_ESCAPE;
    
    #ifdef TARGET_GP2X
        gameKeys[TSpiritsGame::K_PAUSE]=SDLK_x;
    #else
        gameKeys[TSpiritsGame::K_PAUSE]=SDLK_p;
    #endif
    
    gameKeys[TSpiritsGame::K_UP]   =SDLK_UP;
    gameKeys[TSpiritsGame::K_DOWN] =SDLK_DOWN;
    gameKeys[TSpiritsGame::K_LEFT] =SDLK_LEFT;
    gameKeys[TSpiritsGame::K_RIGHT]=SDLK_RIGHT;
    
    #ifdef TARGET_GP2X
        gameKeys[TSpiritsGame::K_SHOOT]=SDLK_a;
        gameKeys[TSpiritsGame::K_TAKE] =SDLK_y;
        gameKeys[TSpiritsGame::K_NEXT] =SDLK_b;
    #else
        gameKeys[TSpiritsGame::K_SHOOT]=SDLK_SPACE;
        gameKeys[TSpiritsGame::K_TAKE] =SDLK_z;
        gameKeys[TSpiritsGame::K_NEXT] =SDLK_x;
    #endif

    gameKeys[TSpiritsGame::K_KEY_LEFT] =SDLK_LEFT;
    gameKeys[TSpiritsGame::K_KEY_RIGHT] =SDLK_RIGHT;
    gameKeys[TSpiritsGame::K_KEY_UP] =SDLK_UP;
    gameKeys[TSpiritsGame::K_KEY_DOWN] =SDLK_DOWN;
    
    gameKeys[TSpiritsGame::K_KEY_SELECT0] =SDLK_SPACE;
    gameKeys[TSpiritsGame::K_KEY_SELECT1] =SDLK_RETURN;
    gameKeys[TSpiritsGame::K_KEY_ESCAPE] =SDLK_ESCAPE;
    
    if (gameLanguage==NULL)
    {
        languageCount=1;
        languageCurrent=0;
        if (gameLanguage!=NULL) delete gameLanguage;
        gameLanguage = new strLanguage[1];
        gameLanguage[0].lang_id="es";
        gameLanguage[0].lang_name="Espa\x0c3\x0b1ol";
        language = DFLT_LANGUAGE;
    }
}

bool Config::loadConfigFile(void)
{
    int result;
    Path &path = TSpiritsGame::gamePath;
    Log  &log  = TSpiritsGame::gameLog;
    lua_State *ls;

    // Leemos la lista de lenguajes
    //
    languageCount = 0;
    gameLanguage = new strLanguage[10];

    TMAString filename = path.makePath(path.getLangPrefix(),"lang.xml");
    
    TiXmlDocument doc( *filename );
    if (doc.LoadFile())
    {
        // Obtenemos el nivel "translation"
        //
        TiXmlElement* entry = doc.FirstChildElement("translations");
        if (entry) entry = entry->FirstChildElement("entry");

        while (entry)
        {
            TiXmlElement* e_id = entry->FirstChildElement("msgid");
            TiXmlElement* e_str = entry->FirstChildElement("msgstr");
            
            gameLanguage[languageCount].lang_id = e_id->GetText();
            gameLanguage[languageCount].lang_name = e_str->GetText();
            languageCount++;
            
            entry = entry->NextSiblingElement( "entry" );
            
        }
    }

    // Ahora leemos el resto de la configuracin.
    //
    ioLua.reset(); ls = ioLua.getHandler();
    filename = path.makePath(path.getConfigPrefix(),CONFIG_FILE_NAME);
    
	TSpiritsGame::gameLog.logMsg(_("Calling loadConfigFile."));
	result=ioLua.executeFileScript(filename.text());

	// Algo ha fallado, se pasa de los valores en el fichero de configuracion
	// y se graba uno nuevo.
	//
	if (result)
	{
		log.logMsg(_("Warning: config file %s, unreadable. Setting up default config."),filename.text());
		saveConfigFile();
		return false;
	}
	
    lua_getglobal(ls,"key_next");     // Index -14
    lua_getglobal(ls,"key_take");     // Index -13
    lua_getglobal(ls,"key_shoot");    // Index -12
    lua_getglobal(ls,"key_right");    // Index -11
    lua_getglobal(ls,"key_left");     // Index -10
    lua_getglobal(ls,"key_down");     // Index -9
    lua_getglobal(ls,"key_up");       // Index -8
    lua_getglobal(ls,"key_pause");    // Index -7
    lua_getglobal(ls,"key_abort");    // Index -6
    lua_getglobal(ls,"language");     // Index -5
	lua_getglobal(ls,"surround");     // Index -4
	lua_getglobal(ls,"sfx_vol");      // Index -3
	lua_getglobal(ls,"music_vol");    // Index -2
	lua_getglobal(ls,"fullscreen");   // Index -1
	
	// Leemos los valores de las variables y los corregimos si estan
    // fuera de rango.
    //
    fullscreen = (getNumber(-1,0,1,DFLT_FULLSCREEN)==1);
    music_vol  = getNumber(-2,0,128,DFLT_MUSIC_VOL);
    music_on = (TMAGame::isSoundAvailable() && music_vol>0);
    sfx_vol  = getNumber(-3,0,128,DFLT_SFX_VOL);
    sfx_on = (TMAGame::isSoundAvailable() && sfx_vol>0);
    surround = (getNumber(-4,0,1,DFLT_SURROUND)==1);
    language=getString(-5,DFLT_LANGUAGE);
    
    gameKeys[TSpiritsGame::K_ABORT]=getNumber(-6 ,0,10000,SDLK_ESCAPE);
    gameKeys[TSpiritsGame::K_PAUSE]=getNumber(-7 ,0,10000,SDLK_p);
    gameKeys[TSpiritsGame::K_UP]   =getNumber(-8 ,0,10000,SDLK_UP);
    gameKeys[TSpiritsGame::K_DOWN] =getNumber(-9 ,0,10000,SDLK_DOWN);
    gameKeys[TSpiritsGame::K_LEFT] =getNumber(-10,0,10000,SDLK_LEFT);
    gameKeys[TSpiritsGame::K_RIGHT]=getNumber(-11,0,10000,SDLK_RIGHT);
    gameKeys[TSpiritsGame::K_SHOOT]=getNumber(-12,0,10000,SDLK_SPACE);
    gameKeys[TSpiritsGame::K_TAKE] =getNumber(-13,0,10000,SDLK_z);
    gameKeys[TSpiritsGame::K_NEXT] =getNumber(-15,0,10000,SDLK_x);

    for (int i=0;i<languageCount;i++)
    {
        // Determinamos si es el idioma por defecto
        //
        if (gameLanguage[i].lang_id.compare(language)==0) languageCurrent = i;
    }
    log.logMsg(_("Config file readed."));
    
    return true;
}

bool Config::saveConfigFile(void)
{
    Path &path = TSpiritsGame::gamePath;
    Log  &log  = TSpiritsGame::gameLog;
    FILE *f;

    f=fopen(path.makePath(path.getConfigPrefix(),CONFIG_FILE_NAME).text(),"wt");
	if (f==NULL)
	{
		log.logMsg(_("Warning: The Configuration file %s can not be saved. Using defaults."),path.makePath(path.getConfigPrefix(),CONFIG_FILE_NAME).text());
		setConfigDefaults();
		return false;
	}
	
	fprintf(f,_("-- Fullscreen: 0 -> Windowed mode, 1 -> Fullscreen mode\n"));
	fprintf(f,"fullscreen = %d\n\n",fullscreen?1:0);

    fprintf(f,_("-- Surround effect: 0 -> surround off, 1 -> surround on\n"));
	fprintf(f,"surround = %d\n\n",surround?1:0);

    fprintf(f,_("-- Music volume: 0 (min) - 512 (max)\n"));
	fprintf(f,"music_vol = %d\n\n",music_vol);
	
	fprintf(f,_("-- Sound effects volume: 0 (min) - 255 (max)\n"));
	fprintf(f,"sfx_vol = %d\n\n",sfx_vol);

	fprintf(f,_("-- Language code: en, es, fr, it, de...\n"));
	fprintf(f,"language = \"%s\"\n\n",language.text());
	
	fprintf(f,_("-- Keyboard scancode config.\n"));
	fprintf(f,"key_abort = %i\n",gameKeys[TSpiritsGame::K_ABORT]);
	fprintf(f,"key_pause = %i\n",gameKeys[TSpiritsGame::K_PAUSE]);
	fprintf(f,"key_up = %i\n",gameKeys[TSpiritsGame::K_UP]);
	fprintf(f,"key_down = %i\n",gameKeys[TSpiritsGame::K_DOWN]);
	fprintf(f,"key_left = %i\n",gameKeys[TSpiritsGame::K_LEFT]);
	fprintf(f,"key_right = %i\n",gameKeys[TSpiritsGame::K_RIGHT]);
	fprintf(f,"key_shoot = %i\n",gameKeys[TSpiritsGame::K_SHOOT]);
	fprintf(f,"key_take = %i\n",gameKeys[TSpiritsGame::K_TAKE]);
	fprintf(f,"key_next = %i\n",gameKeys[TSpiritsGame::K_NEXT]);

	fprintf(f,_("\n-- End config file.\n"));

	log.logMsg(_("The Configuration file %s has been saved."),path.makePath(path.getConfigPrefix(),CONFIG_FILE_NAME).text());
	fclose(f);
	return true;
}

void Config::setSfxVolume(int vol)
{
    if (vol<0) vol=0;
    if (vol>DFLT_SFX_VOL) vol=DFLT_SFX_VOL;
    sfx_vol = vol;
    sfx_on = (TMAGame::isSoundAvailable() && sfx_vol>0);
}

void Config::setMusicVolume(int vol)
{
    if (vol<0) vol=0;
    if (vol>DFLT_MUSIC_VOL) vol=DFLT_MUSIC_VOL;
    music_vol = vol;
    music_on = (TMAGame::isSoundAvailable() && music_vol>0);
}

void Config::setLanguageIndex(int index)
{
    if (index < languageCount )
    {
        languageCurrent = index;
    }
    else languageCurrent=0;
    
    language = gameLanguage[languageCurrent].lang_id.text();
}


#endif
