#ifndef _CONFIG_HPP_
#define _CONFIG_HPP_

#include "../../masoftware/util/tmastring.hpp"
#include "luaspirits.hpp"

#ifdef LINUX
    #define CONFIG_FILE_NAME ".spirits-cfg"
#else
    #define CONFIG_FILE_NAME "spirits.cfg"
#endif

#define DFLT_FULLSCREEN     0
#define DFLT_MUSIC_VOL      128
#define DFLT_SFX_VOL		128
#define DFLT_SURROUND       1
#define DFLT_LANGUAGE       "es"

typedef struct
{
    TMAString lang_id;
    TMAString lang_name;
} strLanguage;

class Config
{
    private:
        TMAString language;
        bool fullscreen;
        bool surround;
        bool music_on;
        bool sfx_on;
        int music_vol;
        int sfx_vol;
        int gameKeys[16];
        LuaHandler ioLua;
        
        strLanguage *gameLanguage;
        int languageCount,languageCurrent;

        int  getNumber(int stackindex,int minvalue,int maxvalue,int def);
        TMAString getString(int stackindex,const char *def);
        
	public:

        Config();
        ~Config();

		bool loadConfigFile(void);
		bool saveConfigFile(void);
		void setConfigDefaults(void);

		bool isFullScreen(void)   { return fullscreen; }
		bool isSurround(void)     { return surround; }
        bool isSfxOn(void)        { return sfx_on; }
        bool isMusicOn(void)      { return music_on; }
		int  getSfxVolume(void)   { return sfx_vol; }
		int  getMusicVolume(void) { return music_vol; }
		int  getGameKey(int pkey) { return gameKeys[pkey]; }

        int getLanguageIndex(void){ return languageCurrent; }
        int getLanguageCount(void){ return languageCount; }
		const char *getLanguageID() { return gameLanguage[languageCurrent].lang_id.text(); }
        const char *getLanguageName(void) { return gameLanguage[languageCurrent].lang_name.text(); }
		
		// --------- SET
		
		void setLanguageIndex(int index);
		void setGameKey(int pkey,int value) { gameKeys[pkey]=value; }
		void setFullScreen(bool b) { fullscreen=b; }
        void setSurround(bool b)   { surround=b; }
        void setSfxVolume(int vol);
        void setMusicVolume(int vol);
};

#endif
