#ifndef _FADERS_CPP_
#define _FADERS_CPP_

#include "faders.hpp"

// -------------------------------- Counter

REGISTER_CLASS_ID(TMACounter);

void TMACounter::initCountUp(long v,int pinc)
{
    value=0;
    endvalue=v;
    inc=pinc;
    counttype=COUNT_UP;
}

void TMACounter::initCountDown(long v,int pinc)
{
    value=v;
    endvalue=0;
    if (pinc>0) pinc=-pinc;
    inc=pinc;
    counttype=COUNT_DOWN;
}

bool TMACounter::doCount(void)
{
    switch (counttype)
    {
        case COUNT_UP:

            if (value<endvalue)
            {
                value+=inc;
                if (value>endvalue) value=endvalue;
                return true;
            }
            return false;
            
        case COUNT_DOWN:

            if (value>endvalue)
            {
                value+=inc;
                if (value<endvalue) value=endvalue;
                return true;
            }
            return false;
    }
    return false;
}
            

bool TMACounter::isFinished(void)
{
    return (value==endvalue);
}

// -------------------------------- Fader


REGISTER_CLASS_ID(TMAFader);

void TMAFader::doFade(TMABitmap& destino,TMABitmap& origen)
{
    switch (fadeType)
    {
        case FADE_IN:
            doFadeIn(destino,origen);
        case FADE_OUT:
            doFadeOut(destino,origen);
    }
}

bool TMAFader::doFade()
{
    switch (fadeType)
    {
        case FADE_IN:
            return doFadeIn();
        case FADE_OUT:
            return doFadeOut();
    }
    return false;
}

// -------------------------------- FaderLight

REGISTER_CLASS_ID(TMAFaderLight);

TMAFaderLight::TMAFaderLight():TMAFader()
{
    light=0;
    ipausefps=1;
    ipause=0;
    inclight=1;
}

void TMAFaderLight::doInitFade(TMAFader::FadeType t,int iclight,int pausefps)
{
    inclight=iclight;
    ipause=0;
    ipausefps=pausefps;
    ibFading=true;
    ox=0,oy=0,width=640,height=480,dx=0,dy=0;
    
    switch(t)
    {
        case TMAFader::FADE_IN:
            fadeType = TMAFader::FADE_IN;
            light=0;
            break;
            
        case TMAFader::FADE_OUT:
            fadeType = TMAFader::FADE_OUT;
            light=255;
            break;
    }
}

void TMAFaderLight::doInitFade1(TMAFader::FadeType t,int ax,int ay,int iclight,
                        int orix,int oriy,int destx,int desty,int pausefps)
{
    doInitFade(t,iclight,pausefps);
    ox=orix,oy=oriy,width=ax,height=ay,dx=destx,dy=desty;
}

void TMAFaderLight::doFadeIn(TMABitmap& destino,TMABitmap& origen)
{
    origen.setAlpha(light);
    destino.draw(&origen,ox,oy,width,height,dx,dy);
}

bool TMAFaderLight::doFadeIn()
{
    if (light<255)
    {
        ipause++;
        if (ipause==ipausefps)
        {
            ipause=0;
            light+=inclight;
            if (light>255) light=255;
        }
    }
    ibFading=(light<255);
    return ibFading;
}

void TMAFaderLight::doFadeOut(TMABitmap& destino,TMABitmap& origen)
{
    doFadeIn(destino,origen);
}

bool TMAFaderLight::doFadeOut()
{
    if (light>0)
    {
        ipause++;
        if (ipause==ipausefps)
        {
            ipause=0;
            light-=inclight;
            if (light<0) light=0;
        }
    }
    ibFading=(light>0);
    return ibFading;
}

// -------------------------------- FaderSpectrum

REGISTER_CLASS_ID(TMAFaderSpectrum);

TMAFaderSpectrum::TMAFaderSpectrum():TMAFader()
{
    spec_j = spec_y = 0;
}

void TMAFaderSpectrum::doInitFade()
{
    ibFading=true;
    spec_j = spec_y = 0;
    fadeType = TMAFader::FADE_IN;
}

void TMAFaderSpectrum::doFadeIn(TMABitmap& destino,TMABitmap& origen)
{
    destino.draw(&origen,0,spec_j+spec_y,640,2,0,spec_j+spec_y);
}

bool TMAFaderSpectrum::doFadeIn()
{
    spec_j+=20;
    if (spec_j>480)
    {
        if (spec_y<20) spec_y+=2;
        spec_j=0;

        // Se ha completado el visionado del grafico.
        //
        if (spec_y>=20)
        {
            ibFading = false;
        }
    }
    return ibFading;
}

void TMAFaderSpectrum::finish()
{
    spec_j=481;
    spec_y=21;
    ibFading=false;
}

// -------------------------------- FaderBox

REGISTER_CLASS_ID(TMAFaderBox);

TMAFaderBox::TMAFaderBox():TMAFader()
{
    cx=cy=0;
    boxes=NULL;
}

TMAFaderBox::~TMAFaderBox()
{
    if (boxes!=NULL)
    {
        delete boxes;
        boxes=NULL;
    }
}

void TMAFaderBox::doInitFade(TMAFader::FadeType t,int ax,int ay,int tx,int ty,int iclight,
                             int orix,int oriy,int destx,int desty)
{
    ibFading=true;
    inclight=iclight;
    cx = ax/tx;
    cy = ay/ty;
    fadeType=t;
    dest_x = destx; dest_y = desty;
    ori_x = orix; ori_y = oriy;

    // Si previamente estaba inicializado, liberamos estructura.
    //
    if (boxes!=NULL)
    {
        delete boxes;
        boxes=NULL;
    }

    // Creamos estructura
    //
    boxes = new box[cx*cy];
    
    // Rellenamos estructura
    //
    for (int i=0;i<cx;i++)
    for (int j=0;j<cy;j++)
    {
        int index=cx*j+i;
        
        boxes[index].x=i*tx;
        boxes[index].y=j*ty;
        boxes[index].ax=tx;
        boxes[index].ay=ty;
        #ifndef TARGET_GP2X
            boxes[index].wait=j*4+i;
        #else
            boxes[index].wait=j+i+1;
        #endif 
        
        if (fadeType==TMAFader::FADE_IN)
        {
            boxes[index].fade=0;
        }
        else
        {
            boxes[index].fade=255;
        }
    }
}

void TMAFaderBox::doFadeIn(TMABitmap& destino,TMABitmap& origen)
{    
    for (int x=0;x<cx;x++)
    for (int y=0;y<cy;y++)
    {
        int index = y*cx+x;
        
        origen.setAlpha(boxes[index].fade);
        destino.draw(&origen,boxes[index].x+ori_x,boxes[index].y+ori_y,
                            boxes[index].ax,boxes[index].ay,
                            boxes[index].x+dest_x,boxes[index].y+dest_y);
    }
}

bool TMAFaderBox::doFadeIn()
{
    // Suponemos que se ha terminado la iteracin de FADE
    //
    ibFading=false;
    
    for (int x=0;x<cx;x++)
    for (int y=0;y<cy;y++)
    {
        int index = y*cx+x;
        
        if (boxes[index].wait>0)
        {
            boxes[index].wait--;
            
            // No se ha terminado la iteracin
            ibFading=true;
        }
        else
        {
            if (boxes[index].fade<255)
            {
                boxes[index].fade+=inclight;
                ibFading=true;
                if (boxes[index].fade>255)
                {
                    boxes[index].fade=255;
                }
            }
        }
    }
    
    return ibFading;
}

void TMAFaderBox::doFadeOut(TMABitmap& destino,TMABitmap& origen)
{
    for (int x=0;x<cx;x++)
    for (int y=0;y<cy;y++)
    {
        int index = y*cx+x;
        
        origen.setAlpha(boxes[index].fade);
        destino.draw(&origen,boxes[index].x+ori_x,boxes[index].y+ori_y,
                          boxes[index].ax,boxes[index].ay,
                          boxes[index].x+dest_x,boxes[index].y+dest_y);
    }
}

bool TMAFaderBox::doFadeOut()
{
    // Suponemos que se ha terminado la iteracin de FADE
    //
    ibFading=false;

    for (int x=0;x<cx;x++)
    for (int y=0;y<cy;y++)
    {
        int index = y*cx+x;

        if (boxes[index].wait>0)
        {
            boxes[index].wait--;

            // No se ha terminado la iteracin
            ibFading=true;
        }
        else
        {
            if (boxes[index].fade>0)
            {
                boxes[index].fade-=inclight;
                ibFading=true;
                if (boxes[index].fade<0)
                {
                    boxes[index].fade=0;
                }
            }
        }
    }

    return ibFading;
}


#endif

