#ifndef _FADERS_HPP_
#define _FADERS_HPP_

#include "../../masoftware/sdl/ma_classes.hpp"

// Clase Counter
//
class TMACounter:public TMAObject
{
    REGISTER_CLASS protected:

        enum CountType { COUNT_UP=1,COUNT_DOWN=2 };

        int counttype,inc;
        long value,endvalue;
        
    public:
        PUBLIC_CLASS_ID;

        TMACounter() { initCountDown(0,0); }
        void initCountUp(long v,int pinc=1);
        void initCountDown(long v,int pinc=1);
        bool doCount(void);
        void finish(void) { value = endvalue; }
        bool isFinished(void);
        long getValue(void) { return value; };
};

// SuperClase FADER
//
class TMAFader:public TMAObject
{
    REGISTER_CLASS protected:

        bool ibFading;
        int  fadeType;

        virtual void doFadeIn(TMABitmap& destino,TMABitmap& origen) = 0;
        virtual bool doFadeIn()=0;

        virtual void doFadeOut(TMABitmap& destino,TMABitmap& origen) = 0;
        virtual bool doFadeOut()=0;

    public:
        PUBLIC_CLASS_ID;
        
        enum FadeType { FADE_IN=1,FADE_OUT=2 };
        TMAFader() { ibFading=false; fadeType=0; }
        bool isFading(void) { return ibFading; }
        
        // Realiza el efecto de FADE configurado, y devuelve FALSE cuando
        // el efecto ha terminado de realizarse
        //
        void doFade(TMABitmap& destino,TMABitmap& origen);
        bool doFade();
        virtual void finish() { ibFading = false; }
        
};

// Un simple fundido de color
//
class TMAFaderLight:public TMAFader
{
    REGISTER_CLASS protected:

        int light,inclight;
        int ipausefps,ipause;
        int ox,oy,width,height,dx,dy;
        
        void doFadeIn(TMABitmap& destino,TMABitmap& origen);
        bool doFadeIn();
        
        void doFadeOut(TMABitmap& destino,TMABitmap& origen);
        bool doFadeOut();

    public:
        PUBLIC_CLASS_ID;

        TMAFaderLight();
        void doInitFade(TMAFader::FadeType,int iclight=1,int pausefps=1);
        void doInitFade1(TMAFader::FadeType,int ax=640,int ay=480,int iclight=1,
                        int orix=0,int oriy=0,int destx=0,int desty=0,int pausefps=1);
                        
};

// Simula la carga del spectrum
//
class TMAFaderSpectrum:public TMAFader
{
    REGISTER_CLASS protected:

        int spec_j,spec_y;

        void doFadeIn(TMABitmap& destino,TMABitmap& origen);
        bool doFadeIn();
        
        void doFadeOut(TMABitmap& destino,TMABitmap& origen) { }
        bool doFadeOut() { return false; }

    public:
        PUBLIC_CLASS_ID;

        TMAFaderSpectrum();
        void doInitFade();
        void finish();
};

// Aparicion en Cuadros
//
class TMAFaderBox:public TMAFader
{
    REGISTER_CLASS protected:

        typedef struct
        {
            int x,y;
            int ax,ay;
            int fade;
            int wait;
        } box;
        
        box *boxes;
        int cx,cy,inclight;
        int ori_x,ori_y,dest_x,dest_y;

        void doFadeIn(TMABitmap& destino,TMABitmap& origen);
        bool doFadeIn();
        
        void doFadeOut(TMABitmap& destino,TMABitmap& origen);
        bool doFadeOut();

    public:
        PUBLIC_CLASS_ID;

        TMAFaderBox();
        ~TMAFaderBox();

        void doInitFade(TMAFader::FadeType,int ax=640,int ay=480,int tx=16,int ty=16,int iclight=1,
                        int orix=0,int oriy=0,int destx=0,int desty=0);
};

#endif
