#ifndef _PATH_CPP_
#define _PATH_CPP_

#include "path.hpp"

void Path::init(int argc,char **argv)
{
    int len;
    
    // Obtenemos la ruta completa con el nombre del ejecutable
    // y luego separamos el path del nombre. Al final nos quedamos
    // unicamente con el PATH del ejecutable
    //
    current_directory = argv[0];

    // Normalizamos las barras de directorios
    //
    current_directory.replaceChar('\\','/');

    len = current_directory.length()-1;
    while (current_directory.text()[len]!='/') len--;
    current_directory = current_directory.substring(0,len);

    // Establecemos el directorio a nivel de sistema
    //
    chdir(*current_directory);

    // Calculamos las rutas de los otros directorios
    //
    data_prefix = (current_directory + "/") + DFLT_DATA_DIR;
    scripts_prefix = (current_directory + "/") + DFLT_SCRIPTS_DIR;
    
	#ifdef LINUX
        config_prefix=getenv("HOME");
        config_prefix=config_prefix+"/";
	#else
		config_prefix=(current_directory + "/") + DFLT_DATA_DIR;
	#endif

    lang_prefix = (current_directory + "/") + DFLT_LANG_DIR;
}

const TMAString& Path::getCurrentDirectory()
{
	return current_directory;
}

const TMAString& Path::getDataPrefix()
{
	return data_prefix;
}

const TMAString& Path::getLangPrefix()
{
	return lang_prefix;
}

const TMAString& Path::getScriptsPrefix()
{
	return scripts_prefix;
}

const TMAString& Path::getConfigPrefix()
{
	return config_prefix;
}

TMAString Path::makePath(const char *path, const char *file)
{
    TMAString s = path;
    s = (s+"/")+file;
	return s;
}

TMAString Path::makePath(const TMAString& path, const char *file)
{
    TMAString s = path;
    char last = s.text()[s.length()-1];
    if (last!='/' && last!='\\') s = s+"/";
    s = s + file;
	return s;
}

bool Path::checkFile(Path::FileType ft, const char *name)
{
    TMAString file="";
	switch(ft)
	{
		case FT_DATA:
			file = makePath(*data_prefix,name);
			break;

		case FT_CONFIG:
            file = makePath(*config_prefix,name);
			break;

		case FT_SCRIPTS:
            file = makePath(*scripts_prefix,name);
			break;

		case FT_LANG:
			file = makePath(*lang_prefix,name);
			break;

		case FT_CURRENT:
            file = makePath(*current_directory,name);
			break;
	}
	
	// Comprobamos que exista el archivo especificado
	//
    FILE *f = fopen(file.text(),"rb");
    if (!f) return false;
    fclose(f);
	return true;
}

#endif
