#ifndef _SCENES_H_
#define _SCENES_H_

#include <string.h>
#include "../spirits.hpp"
#include "../misc/faders.hpp"
#include "../../masoftware/maps/mapas.hpp"

#ifdef TARGET_GP2X
    #define VEL_FADE_LIGHT 20
    #define VEL_FADE_BOX   40
    #define TILESIZE 16
#else
    #define VEL_FADE_LIGHT 10
    #define VEL_FADE_BOX   6
    #define TILESIZE 16
#endif

class TSpriteSpirits;
class TSpriteAnim;
class TSpriteBrujo;
class TSpriteHandle;
class TSpriteEnemigoAleatorio;
class TSpriteObject;
class TSpriteDeambulante;
class TSpriteEnemigo;

// Escenario de Splash/Logo
//
class tsc_logo:public TMAScene
{
    REGISTER_CLASS protected:

        PSpiritsGame myGame;
        PMABitmap io_logo,io_logo1;
        TMAOgg ogg_sound;
        
        TMAFaderLight fLight;
        TMAFaderSpectrum fSpectrum;
        TMAFaderBox fBoxes;
        TMACounter fadeMusic;
        TMACounter waitScreen;
        
        enum LogoEstado { IN=0,OUT=1,IN1,OUT1,SPECTRUM0,SPECTRUM1,SPECTRUM2,
                          SPIRITS1,SPIRITS2,SPIRITS3,GOTO_MENU};
                          
    public:
        PUBLIC_CLASS_ID;
        tsc_logo();
        ~tsc_logo();
        
        bool doInit(const PMAObject params);
        bool doRun(void);
        void doPaint(TMABitmap&);
};

// Escenario de Menu Principal
//

#define MAX_SCROLL_TEXT 150

class tsc_menu:public TMAScene
{
    REGISTER_CLASS protected:

        enum MenuState { SHOW_SCREEN=0, HIDE_MENU, SHOW_MENU,SHOW_MENU_OPTIONS,SHOW_MENU_KEYS,
                         SHOW_INSTRUCTIONS,SHOW_CREDITS,SHOW_MENU_CONFIRM,GOTO_GAME};

        PSpiritsGame myGame;
        PMABitmap io_screen;
        PMABitmap io_scrollbox;
        PMABitmap io_selector;
        PMABitmap io_volumegauge;
        PMABitmap io_volumeindicator;
        TMAFaderLight fLight;
        TMAFaderBox fBoxes;
        TMASound snd_menuchange;
        TMASound snd_menuselect;
        TMASound snd_abracadabra;
        bool ib_prepare_graph;
        int MenuWidth[10][2],MenuItems,MenuIndex;
        MenuState ii_NextState;
        
        struct
        {
            int offsety;
            int first_index;
            int last_index;
            char *txt[MAX_SCROLL_TEXT];
        } StrScroll;
        
        int movex, dirx, fgrama;

        void freeScrollText(void);
        void prepareHistory(void);
        void prepareCredits(void);
        void changeVolume(bool music);
        void gotoState(tsc_menu::MenuState e,tsc_menu::MenuState next=tsc_menu::SHOW_SCREEN);
        
    public:
        PUBLIC_CLASS_ID;
        tsc_menu();
        ~tsc_menu();

        bool doInit(const PMAObject params);
        bool doRun(void);
        void doPaint(TMABitmap&);
        void doPaintMenu(TMABitmap&);
        void doPrepareMenu(bool fade=true);
        void doPaintKeys(TMABitmap&);
        void doScroll(TMABitmap&);
        void doScroll();

        int drawBicolorFont(TMABitmap&,const char *,int x,int y);
};

// Escenario de Juego
//
class tsc_game:public TMAScene
{
    REGISTER_CLASS protected:

        enum State { S_GAME, S_PAUSE, S_ABORT, S_END , S_ENTERCODE};

        typedef struct
        {
            PMABitmap scrBuffer;
            PMABitmap scrImage;
            PMAScreenMap screen;
        } scrCanvas;

        static const objectInfo objects[16];
        static const int MAX_ROOM = 88;

        PSpiritsGame myGame;
        TMAGameMap map;
        roomInfo rooms[MAX_ROOM];
        
        PMABitmap infoPanel,border_up,border_down;
        PMABitmap border_left,border_right;
        PMABitmap interference;
        PMABitmap imgLive,imgDie,imgEnergy;
        PMAImageBank bankShadow;
        scrCanvas screenUp,screenDown;

        TMAVector<TSpriteEnemigo> iv_Enemy;         // Vector de Enemigos
        int lastScreenDown,objectsLocation,objetivo,percentdone;
        bool st_abort,ib_enemy_stopped;
        int waitfps;
        int alpha,alpha_dir;
        
        char codebuff[100];
        int codeindex;

        // ---- Sprites
        //
        TSpriteBrujo *sprBrujo;
        TSpriteDoor *openingdoor;
        TSpriteEnemigoAleatorio *sprCat;
        TSpriteEnemigoAleatorio *sprArchier;
        TSpriteObject *sprSphere;
        TSpriteObject *sprBook;
        TSpriteObject *sprMagic;
        TSpriteDeambulante *sprCasandra;
        TSpriteDeambulante *sprWorf;
        TSpriteDeambulante *sprEagle;
        TSpriteSpirits *objetivos[6];

        // ---- Sonidos
        //
        int ii_music;
        gameSound sounds;

        void loadMusic(void);
        void prepareScreen(scrCanvas&,int num);
        void initEnemy(void);
        void getAllRooms(void);
        void makeShadows(void);
        void handlePushed(int room);
        void showOpeningDoor(TSpriteDoor*);
        void putObjects(void);
        void setState(State e);
        void gotoMenu(void);
        void gotoFinish(void);

        void dispatchMessages(void);
        void detectTrainer(void);
        void enterCode(void);

    public:
        PUBLIC_CLASS_ID;
        tsc_game();
        ~tsc_game();

        bool doInit(const PMAObject params);

        void doSounds(void);
        bool doRun(void);
        void doAnimateRoom(scrCanvas&);
        
        void doPaint(TMABitmap&);
        void doPaintRoom(scrCanvas&);
        void doPaintInfoPanel(TMABitmap&);
        
        int doPaintText(TMABitmap&,const char *txt,int x,int y);
        int doPaintTextBig(TMABitmap& buff,const char *txt,int x,int y);
};

// Escenario de Fin del Juego
//
class tsc_end:public TMAScene
{
    REGISTER_CLASS protected:

        enum State { E_FADEIN,E_TEXT1,E_TEXT2,E_TEXT3,
                     E_ANIM1,E_ANIM2,E_ANIM3,E_ANIM3_A,
                     E_ANIM4,E_ANIM4_A,E_ANIM5,E_ANIM6,E_ANIM7,
                     E_ANIM8,E_ANIM9,E_ANIM10,E_ANIM11,E_ANIM12,E_FADEOUT};

        static int ia_AnimMago[4];
        static int ia_AnimRana[8];
        static int ia_AnimSaltoRana[1];
        static int ia_AnimHumo[14];
        static int ia_AnimExp[8];
        static int ia_AnimShot[4];
        static int ia_AnimHeart[4];
        static int ia_AnimFeo[10];
        static int ia_AnimRisa[4];
        static int ia_AnimSorpresa[4];

        PSpiritsGame myGame;
        TMAGameMap map;
        TMAFaderLight fLight;
        TMACounter wait;
        PMABitmap io_screen;
        TSpriteAnim *sprBrujo;
        TSpriteAnim *sprMago;
        TSpriteAnim *sprCasandra;
        TSpriteAnim *sprAux;
        int aux1,aux2;
        
        TMASound snd_Frog;
        TMASound snd_Shoot;
        TMASound snd_Abra;
        TMASound snd_Die;
        TMASound snd_Energy;
        TMASound snd_Jump;
        TMASound snd_Laugh;

        void setState(State s);
        int drawBicolorFont(TMABitmap& buff,const char *txt,int x,int y);
        void gotoMenu();

    public:
        PUBLIC_CLASS_ID;
        tsc_end();
        ~tsc_end();

        bool doInit(const PMAObject params);
        bool doRun(void);
        void doPaint(TMABitmap&);
};

#endif
