#ifndef _tsc_end_
#define _tsc_end_

#include "scenes.hpp"
#include "../sprites/sprites.hpp"
REGISTER_CLASS_ID(tsc_end);

int tsc_end::ia_AnimMago[4] = {92,92,93,93};
int tsc_end::ia_AnimRana[8] = {90,90,90,90,91,91,90,90};
int tsc_end::ia_AnimSaltoRana[1] = {114};
int tsc_end::ia_AnimHumo[14] = {109,110,111,112,111,110,111,112,111,110,111,112,111,113 };
int tsc_end::ia_AnimExp[8] = {101,102,103,104,105,106,107,108};
int tsc_end::ia_AnimShot[4] = {98,99,100,99};
int tsc_end::ia_AnimHeart[4] = {78,79,80,79};
int tsc_end::ia_AnimFeo[10] = {81,82,83,84,85,86,87,87,87,87};
int tsc_end::ia_AnimRisa[4] = {88,88,89,89};
int tsc_end::ia_AnimSorpresa[4] = {76,75,76,76};

tsc_end::tsc_end():TMAScene()
{
    myGame = (PSpiritsGame)TMAGame::getCurrentGame();
    sprMago=sprBrujo=sprCasandra=sprAux=NULL;
}

tsc_end::~tsc_end()
{
    if (sprMago) delete sprMago;
    if (sprBrujo) delete sprBrujo;
    if (sprCasandra) delete sprCasandra;
    if (sprAux) delete sprAux;
}

bool tsc_end::doInit(const PMAObject params)
{
    // Cargamos el archivo de Mapa
    //
    TMAString data_path = myGame->gamePath.getDataPrefix();
    if (!map.setMapFile("end.mpb",data_path))
    {
        myGame->gameLog.logMsg(_("No es posible abrir el archivo de mapa '%s%s'."),
            data_path.text(),"end.mpb");
        return false;
    }

    // Indicamos los bancos graficos del mapa que usan informacin alpha
    //
    map.getGraphBank(0)->setTransparentMode(TMAGraphBank::T_ALPHA); // Cielo
    io_screen = map.getGraphBank(2)->getImage(0);

    // Cargamos los sonidos a utilizar
    //
    snd_Frog.load(myGame->gamePath.makePath(data_path,"sound/frog.wav").text());
    snd_Abra.load(myGame->gamePath.makePath(data_path,"sound/abracadabra.wav").text());
    snd_Shoot.load(myGame->gamePath.makePath(data_path,"sound/shoot.wav").text());
    snd_Die.load(myGame->gamePath.makePath(data_path,"sound/die.wav").text());
    snd_Energy.load(myGame->gamePath.makePath(data_path,"sound/energywall.wav").text());
    snd_Jump.load(myGame->gamePath.makePath(data_path,"sound/jump.wav").text());
    snd_Laugh.load(myGame->gamePath.makePath(data_path,"sound/laugh.wav").text());

    // Creamos los actores.
    //
    PMAImageBank imgb = map.getGraphBank(1)->getImageBank();
    sprMago = new TSpriteAnim(imgb,TSpriteAnim::T_OTHER);
    sprBrujo = new TSpriteAnim(imgb,TSpriteAnim::T_OTHER);
    sprCasandra = new TSpriteAnim(imgb,TSpriteAnim::T_OTHER);
    sprAux = new TSpriteAnim(imgb,TSpriteAnim::T_OTHER);

    myGame->setGameFPS(32);
    setState(E_FADEIN);
    return true;
}

void tsc_end::setState(State s)
{
    switch ((int)s)
    {
        case E_FADEIN:
            fLight.doInitFade(TMAFader::FADE_IN,4);
            break;

        case E_FADEOUT:
            fLight.doInitFade(TMAFader::FADE_OUT,4);
            break;

        case E_TEXT1:
        case E_TEXT2:
            wait.initCountDown(200);
            break;
            
        case E_ANIM1:   // Aparece aguila y brujo
            sprMago->setAnimSequence(TSpriteDeambulante::ia_AnimEagle,4);
            sprMago->putAt(-60,0);
            sprBrujo->setAnimSequence(TSpriteBrujo::ia_AnimWalk,6);
            sprBrujo->putAt(-60,380);
            break;
            
        case E_ANIM2:   // El aguila se vuelve al brujo
            sprMago->setFlipX(true);
            sprBrujo->setAnimSequence(TSpriteBrujo::ia_AnimDying,1);
            break;
            
        case E_ANIM3:   // El aguila se transforma en mago
            sprMago->setFlipX(false);
            sprMago->setAnimSequence(ia_AnimHumo,14);
            myGame->playSfx(snd_Energy);
            wait.initCountDown(30);
            break;
            
        case E_ANIM3_A: // Ahora ya es un mago.
            sprMago->setAnimSequence(ia_AnimMago,4);
            wait.initCountDown(200);
            break;

        case E_ANIM4:   // El brujo le dispara al mago
            sprBrujo->setAnimSequence(TSpriteBrujo::ia_AnimShoot,3);
            sprAux->setAnimSequence(ia_AnimShot,4);
            sprAux->position.x = sprBrujo->position.x+10;
            sprAux->position.y = sprBrujo->position.y+20;
            myGame->playSfx(snd_Abra);
            break;
            
        case E_ANIM4_A: // Explosion Mgica
            sprBrujo->setAnimSequence(TSpriteBrujo::ia_AnimDying,1);
            sprMago->setAnimSequence(ia_AnimExp,8);
            myGame->playSfx(snd_Die);
            wait.initCountDown(100);
            break;
            
        case E_ANIM5:   // El mago es transformado en rana
            sprBrujo->setAnimSequence(TSpriteBrujo::ia_AnimDying,1);
            sprMago->setAnimSequence(ia_AnimRana,8);
            sprMago->position.y += 50;
            sprMago->position.x += 10;
            myGame->playSfx(snd_Frog);
            wait.initCountDown(100);
            break;
            
        case E_ANIM6:   // La rana se pega la vuelta y se larga
            sprMago->setFlipX(true);
            sprMago->setAnimSequence(ia_AnimSaltoRana,1);
            myGame->playSfx(snd_Frog);
            aux1 = sprMago->position.y;
            aux2 = -4;
            myGame->playSfx(snd_Jump);
            break;
            
        case E_ANIM7:   // Aparece Casandra
            sprCasandra->setFlipX(true);
            sprCasandra->position.x = sprBrujo->position.x + 250;
            sprCasandra->position.y = sprBrujo->position.y;
            sprCasandra->setAnimSequence(ia_AnimHumo,14);
            myGame->playSfx(snd_Energy);
            wait.initCountDown(30);
            break;
            
        case E_ANIM8:   // El brujo se acerca un poco a ella 
            sprBrujo->setAnimSequence(TSpriteBrujo::ia_AnimWalk,6);
            sprCasandra->setAnimSequence(TSpriteDeambulante::ia_AnimCasandra,4);
            sprCasandra->setSpeed(4);
            break;
            
        case E_ANIM9:   // Sale el corazoncito
            sprBrujo->setAnimSequence(TSpriteBrujo::ia_AnimDying,1);
            sprAux->position.x = sprCasandra->position.x - 50;
            sprAux->position.y = sprCasandra->position.y - 10;
            sprAux->setAnimSequence(ia_AnimHeart,4);
            sprAux->setSpeed(4);
            wait.initCountDown(150);
            break;
            
        case E_ANIM10:  // Se quita la capucha
            sprBrujo->setAnimSequence(ia_AnimFeo,10);
            sprBrujo->setSpeed(10);
            wait.initCountDown(100);
            break;

        case E_ANIM11:  // Brujo se rie y casandra sorprendida, corazon cae
            sprBrujo->setAnimSequence(ia_AnimRisa,4);
            sprBrujo->setSpeed(4);
            sprCasandra->setAnimSequence(ia_AnimSorpresa,4);
            sprCasandra->setSpeed(8);
            myGame->playSfx(snd_Laugh);
            wait.initCountDown(200);
            break;
            
        case E_ANIM12:  // Casandra se esfuma.
            sprCasandra->setAnimSequence(ia_AnimHumo,14);
            myGame->playSfx(snd_Energy);
            wait.initCountDown(30);
            break;


    }
    ii_Estado=s;
}

bool tsc_end::doRun(void)
{
    switch (ii_Estado)
    {
        case E_FADEIN:
            if (!fLight.doFade()) setState(E_TEXT1);
            break;

        case E_FADEOUT:
            if (!fLight.doFade()) gotoMenu();
            break;

        case E_TEXT1:
            if (!wait.doCount()) setState(E_TEXT2);
            break;

        case E_TEXT2:
            if (!wait.doCount()) setState(E_ANIM1);
            break;

        case E_ANIM1:
            sprMago->animate();
            sprBrujo->animate();
            sprMago->position.x += 5;
            sprMago->position.y ++;
            
            if (sprMago->position.x>50)
            {
                sprBrujo->position.x += 3;
            }
            if (sprBrujo->position.x > 150) setState(E_ANIM2);
            break;
            
        case E_ANIM2:
            sprMago->animate();
            sprBrujo->animate();
            sprMago->position.x -= 1;
            sprMago->position.y += 4;
            if (sprMago->position.y > sprBrujo->position.y - 10) setState(E_ANIM3);
            break;
            
        case E_ANIM3:
            sprMago->animate();
            sprBrujo->animate();
            if (!wait.doCount()) setState(E_ANIM3_A);
            break;

        case E_ANIM3_A:
            sprMago->animate();
            sprBrujo->animate();
            if (!wait.doCount()) setState(E_ANIM4);
            break;

        case E_ANIM4:
            sprMago->animate();
            sprBrujo->animate();
            if (!snd_Abra.isPlaying())
            {
                sprAux->animate();
                sprAux->position.x += 8;
            }
            if (sprAux->position.x >= sprMago->position.x + 20) setState(E_ANIM4_A);
            break;

        case E_ANIM4_A:
            sprMago->animate();
            sprBrujo->animate();
            if (!wait.doCount()) setState(E_ANIM5);
            break;

        case E_ANIM5:
            sprMago->animate();
            sprBrujo->animate();
            if (!wait.doCount()) setState(E_ANIM6);
            break;

        case E_ANIM6:
            sprMago->animate();
            sprBrujo->animate();
            sprMago->position.x += 2;
            sprMago->position.y += aux2;
            if (sprMago->position.y < aux1-60)
            {
                aux2 = -aux2;
                if (aux2<0) sprMago->setAnimSequence(ia_AnimSaltoRana,1);
                else sprMago->setAnimSequence(ia_AnimRana,8);
                sprMago->animate();
            }
            if (sprMago->position.y > aux1)
            {
                sprMago->position.y = aux1;
                aux2 = -aux2;
                myGame->playSfx(snd_Jump);
                if (aux2<0) sprMago->setAnimSequence(ia_AnimSaltoRana,1);
                else sprMago->setAnimSequence(ia_AnimRana,8);
                sprMago->animate();
            }
            if (sprMago->position.x > 650) setState(E_ANIM7);
            break;

        case E_ANIM7:
            sprMago->animate();
            sprCasandra->animate();
            if (!wait.doCount()) setState(E_ANIM8);
            break;
            
        case E_ANIM8:
            sprBrujo->animate();
            sprCasandra->animate();
            sprBrujo->position.x += 2;
            if (sprBrujo->position.x > sprCasandra->position.x - 120) setState(E_ANIM9);
            break;
            
        case E_ANIM9:
            sprBrujo->animate();
            sprCasandra->animate();
            sprAux->animate();
            if (!wait.doCount()) setState(E_ANIM10);
            break;

        case E_ANIM10:
            sprCasandra->animate();
            sprBrujo->animate();
            sprAux->animate();
            if (!wait.doCount()) setState(E_ANIM11);
            break;

        case E_ANIM11:
            sprCasandra->animate();
            sprBrujo->animate();
            sprAux->position.y += 8;
            if (!wait.doCount()) setState(E_ANIM12);
            break;

        case E_ANIM12:
            sprCasandra->animate();
            sprBrujo->animate();
            if (!wait.doCount()) setState(E_FADEOUT);
            break;
            
    }
    return true;
}

void tsc_end::doPaint(TMABitmap& buff)
{
    buff.clear(0);
    
    switch (ii_Estado)
    {
        case E_FADEIN:
            fLight.doFade(buff,*io_screen);
            break;

        case E_FADEOUT:
            fLight.doFade(buff,*io_screen);
            break;

        case E_TEXT1:
            buff.draw(io_screen,0,0);
            drawBicolorFont(buff,_("Aquel que no debe nombrarse"),-1,100);
            drawBicolorFont(buff,_("al verse amenzadado intenta"),-1,140);
            drawBicolorFont(buff,_("escapar del castillo."),-1,180);
            break;

        case E_TEXT2:
            buff.draw(io_screen,0,0);
            drawBicolorFont(buff,_("Pero tu, siempre tenaz"),-1,100);
            drawBicolorFont(buff,_("le impides la huida."),-1,140);
            break;
            
        case E_ANIM1:
        case E_ANIM2:
        case E_ANIM3:
        case E_ANIM4_A:
        case E_ANIM5:
        case E_ANIM6:
            buff.draw(io_screen,0,0);
            sprMago->draw(buff);
            sprBrujo->draw(buff);
            break;

        case E_ANIM3_A:
            buff.draw(io_screen,0,0);
            sprMago->draw(buff);
            sprBrujo->draw(buff);
            drawBicolorFont(buff,_("Asquerosa piltrafa, no te"),-1,100);
            drawBicolorFont(buff,_("atreveras a enfrentarte a mi!!"),-1,140);
            break;

        case E_ANIM4:
            buff.draw(io_screen,0,0);
            sprMago->draw(buff);
            if (!snd_Abra.isPlaying()) sprAux->draw(buff);
            sprBrujo->draw(buff);
            break;

        case E_ANIM7:
        case E_ANIM8:
        case E_ANIM12:
            buff.draw(io_screen,0,0);
            sprBrujo->draw(buff);
            sprCasandra->draw(buff);
            break;
            
        case E_ANIM9:
            buff.draw(io_screen,0,0);
            sprBrujo->draw(buff);
            sprCasandra->draw(buff);
            sprAux->draw(buff);            
            drawBicolorFont(buff,_("ERES MI HEROE!!"),-1,100);
            break;

        case E_ANIM10:
        case E_ANIM11:
            buff.draw(io_screen,0,0);
            sprBrujo->draw(buff);
            sprCasandra->draw(buff);
            sprAux->draw(buff);
            break;

    }
}

int tsc_end::drawBicolorFont(TMABitmap& buff,const char *txt,int x,int y)
{
    int width = myGame->fontMenu.getTextWidth(txt);
    #ifdef SMALL_GAME
        int ax = (x==-1)?(buff.getWidth()*2 - width)/2:x;
    #else
        int ax = (x==-1)?(buff.getWidth() - width)/2:x;
    #endif
    int ay = myGame->fontMenu.getFontHeight();

    // Sombras
    myGame->fontMenu.drawText(buff,txt,ax+6,y+6,MAKERGBCOLOR(0,0,0),true);
    myGame->fontMenu.drawText(buff,txt,ax+2,y+2,MAKERGBCOLOR(20,10,0),true);

    // Mitad Inferior
    myGame->fontMenu.drawText(buff,txt,ax,y,MAKERGBCOLOR(200,220,250),true);

    // Mitad Superior
    int rx,ry,rx1,ry1;
    buff.getClipRect(rx,ry,rx1,ry1);    
    #ifdef SMALL_GAME
        buff.setClipRect(rx,y/2,rx1,ay/4+1);
    #else
        buff.setClipRect(rx,y,rx1,ay/2+2);
    #endif
    myGame->fontMenu.drawText(buff,txt,ax,y,MAKERGBCOLOR(250,200,100),true);
    buff.setClipRect(rx,ry,rx1,ry1);

    return width;
}

void tsc_end::gotoMenu(void)
{
    myGame->setScene(new tsc_menu());
    if (myGame->gameConfig.isMusicOn())
    {
        myGame->ogg_music.load(myGame->gamePath.makePath(myGame->gamePath.getDataPrefix(),"music/spirits.ogg").text());
        myGame->playMusic(myGame->ogg_music,true,128);
    }
}
#endif
