#ifndef _tsc_game_
#define _tsc_game_

#include <ctype.h>
#include <string.h>
#include "scenes.hpp"
#include "../sprites/sprites.hpp"

#define GETKEY(X)   myGame->inputController.isPressDown(TSpiritsGame::X)

REGISTER_CLASS_ID(tsc_game);

const objectInfo tsc_game::objects[16] =
{
    {6  ,36 ,65},
	{17 ,48 ,25},
	{25 ,68	,57},
	{33	,60	,46},
	{36	,65	,17},
	{46	,75	,87},
	{47	,33	,60},
	{48	,25	,68},
	{49	,79	,80},
	{57	,47	,33},
	{60	,46	,75},
	{65	,17	,48},
	{68	,57	,47},
	{75	,87	,80},
	{80	,49	,79},
	{87	,80	,49}
};

tsc_game::tsc_game():TMAScene()
{
    myGame = (PSpiritsGame)TMAGame::getCurrentGame();
    memset(rooms,0,sizeof(rooms));
    memset(codebuff,0,sizeof(codebuff));

    // Inicializamos las estructuras de las 2 pantallas.
    //
    screenUp.scrBuffer=NULL;
    screenUp.scrImage=NULL;
    screenUp.screen=NULL;
    screenDown.scrBuffer=NULL;
    screenDown.scrImage=NULL;
    screenDown.screen=NULL;
    border_up=NULL;
    border_down=NULL;
    border_left=NULL;
    border_right=NULL;
    interference=NULL;
    imgLive=NULL;
    imgDie=NULL;
    imgEnergy=NULL;
    bankShadow=NULL;
    
    // Enemigos aleatorios
    //
    sprCat=NULL;
    sprArchier=NULL;
    sprSphere=NULL;
    sprBook=NULL;
    sprMagic=NULL;
    sprCasandra=NULL;
    sprWorf=NULL;
    sprEagle=NULL;
    
    // Control de Apertura de Puertas
    //
    openingdoor=NULL;
    lastScreenDown=0;
    
    // Localizacin inicial de los objetos
    //
    objectsLocation = TMARandom::nextInt(16);
}

tsc_game::~tsc_game()
{
    // Limpiamos las estructuras de las 2 pantallas.
    //
    if (screenUp.scrBuffer!=NULL) delete screenUp.scrBuffer;
    if (screenUp.scrImage!=NULL)  delete screenUp.scrImage;
    if (screenDown.scrBuffer!=NULL) delete screenDown.scrBuffer;
    if (screenDown.scrImage!=NULL)  delete screenDown.scrImage;
    if (border_up!=NULL) delete border_up;
    if (border_down!=NULL) delete border_down;
    if (border_left!=NULL) delete border_left;
    if (border_right!=NULL) delete border_right;
    if (interference!=NULL) delete interference;

    if (imgLive!=NULL) delete imgLive;
    if (imgDie!=NULL) delete imgDie;
    if (imgEnergy!=NULL) delete imgEnergy;

    if (sprBrujo!=NULL) delete sprBrujo;
    if (bankShadow!=NULL) delete bankShadow;
    
    if (sprCat!=NULL) delete sprCat;
    if (sprArchier!=NULL) delete sprArchier;

    if (sprSphere!=NULL) delete sprSphere;
    if (sprBook!=NULL) delete sprBook;
    if (sprMagic!=NULL) delete sprMagic;
    if (sprCasandra!=NULL) delete sprCasandra;
    if (sprWorf!=NULL) delete sprWorf;
    if (sprEagle!=NULL) delete sprEagle;

    // Destruimos informacion de las pantallas
    //
    for (int i=0;i<MAX_ROOM;i++)
    {
        if (rooms[i].scr!=NULL) delete rooms[i].scr;
        if (rooms[i].lights!=NULL) delete rooms[i].lights;
        if (rooms[i].handles!=NULL) delete rooms[i].handles;
        if (rooms[i].lamp!=NULL) delete rooms[i].lamp;
        if (rooms[i].door!=NULL) delete rooms[i].door;
    }
}

void tsc_game::setState(State e)
{
    int i,count=0;
    ii_Estado = (int)e;
    
    switch (ii_Estado)
    {
        case S_END:

            // Ponemos a 0 los contadores.
            //
            sprBrujo->die();

            // Calculamos el porcentaje de consecucin
            //
            for (i=0;i<MAX_ROOM;i++)
            {
                if (rooms[i].visited) count++;
            }
            percentdone = ((count*70) / MAX_ROOM);

            if (sprSphere->isGot()) percentdone+=5;
            if (sprBook->isGot()) percentdone+=5;
            if (sprMagic->isGot()) percentdone+=5;  // 70+15 = 85
            
            if (sprCasandra->isGot()) percentdone+=5;
            if (sprWorf->isGot()) percentdone+=5;
            if (sprEagle->isGot()) percentdone+=5;  // 85 + 15 = 100%

            myGame->inputController.clearStatus();
            break;
            
        case S_PAUSE:
            myGame->releaseKeys();
            break;
            
        case S_ABORT:
            st_abort=true;
            myGame->releaseKeys();
            break;
    }
}

bool tsc_game::doInit(const PMAObject params)
{
    // Cargamos el archivo de Mapa
    //
    TMAString data_path = myGame->gamePath.getDataPrefix();
    if (!map.setMapFile("spirits.mpb",data_path))
    {
        myGame->gameLog.logMsg(_("No es posible abrir el archivo de mapa '%s%s'."),
            data_path.text(),"spirits.mpb");
        return false;
    }
    
    // Indicamos los bancos graficos del mapa que usan informacin alpha
    //
    bankShadow = map.getGraphBank(1)->getImageBank()->clone();
    makeShadows();

    // Inicializamos algunos valores globales de los Sprites
    //
    TSpriteSpirits::setRoomsInfo(rooms,bankShadow);
    TSpriteSpirits::setSounds(&sounds);

    // Cargamos todas las habitaciones
    //
    getAllRooms();

    // Cargamos la imagen del panel informativo
    //
    infoPanel = TMABitmap::loadFromFile(myGame->gamePath.makePath(data_path,"pics/infopanel.png").text());

    // Ponemos los textos del InfoPanel, en el idioma correcto.
    //
    doPaintText(*infoPanel,_("Vidas"),-1,0);
    doPaintText(*infoPanel,_("Energia"),-1,127);
    doPaintText(*infoPanel,_("Objetos"),-1,208);

    // Cargamos los sonidos si los hay
    //
    if (TSpiritsGame::gameConfig.isSfxOn())
    {
        // Sonidos de enemigos
        //
        sounds.snd_wolf.load(myGame->gamePath.makePath(data_path,"sound/wolf.wav").text());
        sounds.snd_ghost.load(myGame->gamePath.makePath(data_path,"sound/ghost.wav").text());
        sounds.snd_hump.load(myGame->gamePath.makePath(data_path,"sound/hump.wav").text());
        sounds.snd_cat.load(myGame->gamePath.makePath(data_path,"sound/cat.wav").text());
        sounds.snd_archier.load(myGame->gamePath.makePath(data_path,"sound/archier.wav").text());
        sounds.snd_arrow.load(myGame->gamePath.makePath(data_path,"sound/arrow.wav").text());

        // Sonidos de ambiente
        //
        sounds.snd_handle.load(myGame->gamePath.makePath(data_path,"sound/pushhandle.wav").text());
        sounds.snd_cricket.load(myGame->gamePath.makePath(data_path,"sound/cricket.wav").text());
        sounds.snd_kettle.load(myGame->gamePath.makePath(data_path,"sound/kettle.wav").text());
        sounds.snd_door.load(myGame->gamePath.makePath(data_path,"sound/door.wav").text());
        sounds.snd_fire.load(myGame->gamePath.makePath(data_path,"sound/fire.wav").text());
        sounds.snd_rockdown.load(myGame->gamePath.makePath(data_path,"sound/rockdown.wav").text());
        sounds.snd_lampdown.load(myGame->gamePath.makePath(data_path,"sound/lampdown.wav").text());
        sounds.snd_tictac.load(myGame->gamePath.makePath(data_path,"sound/tictac.wav").text());
        sounds.snd_take.load(myGame->gamePath.makePath(data_path,"sound/take.wav").text());

        // Sonidos del Personaje
        //
        sounds.snd_shoot.load(myGame->gamePath.makePath(data_path,"sound/shoot.wav").text());
        sounds.snd_jump.load(myGame->gamePath.makePath(data_path,"sound/jump.wav").text());
        sounds.snd_step.load(myGame->gamePath.makePath(data_path,"sound/step.wav").text());
        sounds.snd_energy.load(myGame->gamePath.makePath(data_path,"sound/energywall.wav").text());
        sounds.snd_die.load(myGame->gamePath.makePath(data_path,"sound/die.wav").text());
        sounds.snd_stairs.load(myGame->gamePath.makePath(data_path,"sound/stairs.wav").text());
        sounds.snd_hurt.load(myGame->gamePath.makePath(data_path,"sound/hurt.wav").text());
        sounds.snd_abra.load(myGame->gamePath.makePath(data_path,"sound/abracadabra.wav").text());
    }

    // Definimos el Alpha de Animacion Transparente para la pantalla inferior.
    //
    alpha=80;
    alpha_dir=2;

    // Cargamos los bordes de adorno a las pantallas
    //
    border_up = TMABitmap::loadFromFile(myGame->gamePath.makePath(data_path,"pics/border_up.png").text());
    border_down = TMABitmap::loadFromFile(myGame->gamePath.makePath(data_path,"pics/border_down.png").text());
    border_left = TMABitmap::loadFromFile(myGame->gamePath.makePath(data_path,"pics/border_left.png").text());
    border_right = TMABitmap::loadFromFile(myGame->gamePath.makePath(data_path,"pics/border_right.png").text());
    interference = TMABitmap::loadFromFile(myGame->gamePath.makePath(data_path,"pics/interference.png").text());
    imgLive = TMABitmap::loadFromFile(myGame->gamePath.makePath(data_path,"pics/live.png").text());
    imgDie = TMABitmap::loadFromFile(myGame->gamePath.makePath(data_path,"pics/skeleton.png").text());
    imgEnergy = TMABitmap::loadFromFile(myGame->gamePath.makePath(data_path,"pics/energyindicator.png").text());

    /*
    #ifdef SMALL_GAME
        border_up->setTransparentColor(MAKERGBCOLOR(255,0,255));
        border_down->setTransparentColor(MAKERGBCOLOR(255,0,255));
        border_left->setTransparentColor(MAKERGBCOLOR(255,0,255));
        border_right->setTransparentColor(MAKERGBCOLOR(255,0,255));
        interference->setTransparentColor(MAKERGBCOLOR(255,0,255));
        imgLive->setTransparentColor(MAKERGBCOLOR(255,0,255));
        imgDie->setTransparentColor(MAKERGBCOLOR(255,0,255));
    #endif
    */
    
    
    // Creacion del Sprite Brujo
    //
    sprBrujo = new TSpriteBrujo(map.getGraphBank(1)->getImageBank(),map.getGraphBank(3)->getImageBank());
    sprBrujo->putAt(3,9);
    sprBrujo->setScreenID(1);

    //sprBrujo->putAt(3,9);
    //sprBrujo->setScreenID(44);

    // Establecemos el objetivo de los enemigos
    //
    TSpriteSpirits::setTarget(sprBrujo);

    // Creacin de enemigos aleatorios
    //
    sprCat = new TSpriteEnemigoAleatorio(map.getGraphBank(1)->getImageBank(),TSpriteEnemigoAleatorio::T_CAT);
    sprCat->init(sprBrujo->getScreenID());
    sprArchier = new TSpriteEnemigoAleatorio(map.getGraphBank(1)->getImageBank(),TSpriteEnemigoAleatorio::T_ARC);
    sprArchier->init(sprBrujo->getScreenID());

    // Positionamos los objetos en el mapa
    //
    putObjects();

    // Inicializamos los enemigos.
    //
    initEnemy();
    
    // Rellenamos el array de objetivos
    //
    objetivo=0;
    objetivos[0]=sprSphere;
    objetivos[1]=sprBook;
    objetivos[2]=sprMagic;
    objetivos[3]=sprCasandra;
    objetivos[4]=sprWorf;
    objetivos[5]=sprEagle;

    /*
    // ###DEBUG
    sprSphere->setGot();
    sprBook->validate();
    sprBook->setGot();
    sprBrujo->getBook();
    sprMagic->validate();
    sprMagic->setGot();
    sprBrujo->getMagic();    
    sprCasandra->validate();
    sprWorf->validate();
    sprEagle->validate();
    // ######
    */
    
    // Creamos el buffer de dibujo en las estructuras de las 2 pantallas
    // Sern bitmaps, compartidos con el back buffer del juego.
    //    
    prepareScreen(screenUp,sprBrujo->getScreenID());
    prepareScreen(screenDown,objetivos[objetivo]->getScreenID());

    // Cargamos la musica
    //
    ii_music=0;
    loadMusic();

    // Establecemos la velocidad de refresco para el juego
    //
    myGame->setGameFPS(32);
    
    // Limpiamos la cola de mensajes, por si acaso.
    //
    TSpiritsGame::msgQueue.clearQueue();
    
    // Iniciamos el estado de juego
    //
    setState(S_GAME);
    
    return true;
}

void tsc_game::loadMusic(void)
{
    if (ii_music!=1)
    {
        myGame->ogg_music.load(myGame->gamePath.makePath(myGame->gamePath.getDataPrefix(),"music/castle1.ogg").text());
        ii_music=1;
    }
    else
    {
        myGame->ogg_music.load(myGame->gamePath.makePath(myGame->gamePath.getDataPrefix(),"music/castle2.ogg").text());
        ii_music=2;
    }
    myGame->playMusic(myGame->ogg_music,true,128);
}

void tsc_game::putObjects(void)
{
    int px=0,py=0;
    
    // Creamos los Sprites de la Esfera, Barita y Libro
    //
    sprSphere = new TSpriteObject(map.getGraphBank(1)->getImageBank(),TSpriteObject::T_SPHERE);
    sprSphere->setScreenID(objects[objectsLocation].sphere);

    sprBook = new TSpriteObject(map.getGraphBank(1)->getImageBank(),TSpriteObject::T_BOOK);
    sprBook->setScreenID(objects[objectsLocation].book);
    sprBook->invalidate();

    sprMagic = new TSpriteObject(map.getGraphBank(1)->getImageBank(),TSpriteObject::T_MAGIC);
    sprMagic->setScreenID(objects[objectsLocation].magical);
    sprMagic->invalidate();

    PSpriteObject obj[3] = {sprSphere, sprBook, sprMagic };

    for (int i=0;i<3;i++)
    {
        // Coordenadas para los objetos
        //
        switch (obj[i]->getScreenID())
        {
            case 6:
                px=16,py=9;
                break;

            case 25:
            case 33:
            case 36:
            case 65:
                px=18,py=9;
                break;

            case 80:
                px=20,py=9;
                break;

            case 17:
            case 46:
            case 47:
            case 57:
            case 60:
            case 87:
                px=10,py=9;
                break;

            case 48:
            case 68:
            case 75:
            case 79:
                px=4,py=9;
                break;

            case 49:
                px=2,py=9;
                break;
            default:
                myGame->showAlert("OBJETO NO COLOCADO"); // %i.",obj[i]->getScreenID());
        }
        obj[i]->putAt(px,py);
    }
    
    // Creamos a Casandra, Worf y el Aguila
    //
    sprCasandra = new TSpriteDeambulante(map.getGraphBank(1)->getImageBank(),TSpriteDeambulante::T_CASANDRA);
    sprWorf     = new TSpriteDeambulante(map.getGraphBank(1)->getImageBank(),TSpriteDeambulante::T_WORF);
    sprEagle    = new TSpriteDeambulante(map.getGraphBank(1)->getImageBank(),TSpriteDeambulante::T_EAGLE);
}

void tsc_game::makeShadows(void)
{
    Uint32 trans = MAKERGBCOLOR(255,0,255);
    for (int i=0;i<bankShadow->size();i++)
    {
        PMABitmap bmp = bankShadow->getBitmap(i);
        if (bmp)
        {
            for (int x=0;x<bmp->getWidth();x++)
            for (int y=0;y<bmp->getHeight();y++)
            {
                if (bmp->getPixel(x,y)!=trans)
                {
                    bmp->setPixel(x,y,MAKERGBCOLOR(0,0,0));
                }
            }
            bmp->setAlpha(150);
        }
    }
}

void tsc_game::getAllRooms(void)
{
    for (int i=0;i<MAX_ROOM;i++)
    {
        // Inicialmente, indicamos que todos los muros de todas las pantallas
        // estan cerrados.
        //
        rooms[i].wall_open=false;
        rooms[i].visited=false;
        rooms[i].visited_eagle=false;
        rooms[i].visited_worf=false;
        rooms[i].visited_casandra=false;
        rooms[i].sound_torch=false;

        // Si la pantalla solicitada no se encuentra, devolvera NULL
        //
        rooms[i].scr = map.getScreen(i);

        if (rooms[i].scr)
        {
            rooms[i].lights=NULL;
            rooms[i].handles=NULL;
            
            // Obtenemos los destellos de luces de esta habitacin si los hay
            // Recorremos la lista de animaciones frontales, buscando determinamos
            // graficos, que sern sustituidos por objetos.
            //
            TMAVector<TMAGraphOperation>* v = &(rooms[i].scr->getGraphicsOperationsForType(TMAGraphOperation::OP_ANIM_FRONT));
            for (int j=0;j<v->size();)
            {
                bool vertical_door;
                PMAGraphOperation op = (PMAGraphOperation)v->elementAt(j);
                PSpriteAnim light = NULL;
                PSpriteHandle shandle = NULL;
                
                switch (op->getBankID())
                {
                    case 1: // Busqueda de Sprites

                        // Caso que sea una lampara
                        //
                        if (op->getTileID()==62)
                        {
                            rooms[i].lamp = new TSpriteLamp(map.getGraphBank(1)->getImageBank());
                            rooms[i].lamp->putAt(op->getPosX(),op->getPosY());
                            rooms[i].lamp->setInit(100,4);
                            rooms[i].lamp->setScreenID(i);
                            v->removeElementAt(j);
                        }
                        else j++;
                        break;

                    case 2: // Busqueda de los destellos de luces

                        // Si el vector de luces para esta habitacin aun no esta creado
                        // lo creamos ahora.
                        //
                        if (rooms[i].lights==NULL) rooms[i].lights = new TMAVector<TSpriteAnim>();

                        // Crear Sprite con los datos de la luz
                        //
                        light = new TSpriteAnim(map.getGraphBank(2)->getImageBank(),TSpriteAnim::T_LIGHT);
                        light->putAt(op->getPosX(),op->getPosY());

                        // Incluimos este nuevo sprite en la lista de luces
                        //
                        rooms[i].lights->addElement(light);

                        // Eliminamos esta operacion grafica de la lista
                        //
                        v->removeElementAt(j);
                        break;

                    case 3: // Busqueda de compuertas que se abren y palancas

                        switch (op->getTileID())
                        {
                            case 1: // Columnas
                            case 3: // El muro de piedra
                            case 4: // Reja de hierro
                            case 21: // Piedra horizontal tipo 1
                            case 22: // Piedra horizontal tipo 2

                                vertical_door = !(op->getTileID()==21 || op->getTileID()==22);
                                rooms[i].door = new TSpriteDoor(map.getGraphBank(3)->getImageBank(),op->getTileID(),vertical_door);
                                rooms[i].door->setScreenID(i);
                                
                                if (op->isGridAligned())
                                    rooms[i].door->putAt(op->getPosX()*20,op->getPosY()*20);
                                else
                                    rooms[i].door->putAt(op->getPosX(),op->getPosY());
                                    
                                v->removeElementAt(j);
                                break;
                                
                            case 18: // Palanca
                            
                                if (rooms[i].handles==NULL) rooms[i].handles = new TMAVector<TSpriteHandle>();
                                shandle = new TSpriteHandle(map.getGraphBank(3)->getImageBank());

                                if (op->isGridAligned())
                                    shandle->putAt(op->getPosX()*20,op->getPosY()*20);
                                else
                                    shandle->putAt(op->getPosX(),op->getPosY());
                                    
                                rooms[i].handles->addElement(shandle);
                                v->removeElementAt(j);
                                break;
                                
                            default:
                                j++;
                        }
                        break;
                        
                    default:
                        j++;
                }
            }

            // Otra pasada ahora sobre las animaciones de fondo para detectar antorchas
            //
            v = &(rooms[i].scr->getGraphicsOperationsForType(TMAGraphOperation::OP_ANIM_NORMAL));
            for (int j=0;j<v->size();j++)
            {
                PMAGraphOperation op = (PMAGraphOperation)v->elementAt(j);

                switch (op->getBankID())
                {
                    case 3: // Busqueda de fuegos de antorcha
                        switch (op->getTileID())
                        {
                            case 6:
                            case 7:
                            case 8:
                            case 9:
                                rooms[i].sound_torch=true;
                                break;
                        }
                        break;
                }
            }

        } // El de: if (rooms[i].scr)
    }
}

void tsc_game::prepareScreen(scrCanvas& canvas ,int num)
{
    char buff[100];
    TMAString aux;

    if (canvas.scrImage!=NULL) delete canvas.scrImage;
    
    // Establecemos la pantalla del mapa
    //
    canvas.screen = rooms[num].scr;

    // Cargamos la imagen.
    //
    sprintf(buff,"pics/rooms/scr_%02i.png",num);
    aux = myGame->gamePath.makePath(myGame->gamePath.getDataPrefix(),buff);
    canvas.scrImage = TMABitmap::loadFromFile(*aux);
    canvas.scrImage->setTransparentColor(MAKERGBCOLOR(255,0,255));
    
    // Si la pantalla es la de arriba y no hay luces, quitamos el sonido de fuego
    //
    if (&canvas == &screenUp)
    {
        // Si en la pantalla indicada, hay una lampara, restablecemos su posicin
        //
        if (rooms[num].lamp) rooms[num].lamp->reset();

        // Contador de espera a 0
        //
        waitfps=0;
        
        // Apagamos el sonido de luces, si no las hay
        //
        if (!rooms[num].lights) sounds.snd_fire.stop();
        
        // Apagamos el sonido del caldero.
        //
        sounds.snd_kettle.stop();
        
        // Apagamos el sonido de grillos, si no estamos en las pantallas adecuadas
        //
        if (num!=26 && num!=27 && num!=28) sounds.snd_cricket.stop();
    }
    
    // Ponemos la musica en funcin de la zona del castillo
    //
    if ( (num==28 || num==52) && ii_music==1) loadMusic();
    if ( (num==26 || num==54) && ii_music==2) loadMusic();
}

// Inicializa las posiciones iniciales de los sprites enemigos
//
void tsc_game::initEnemy(void)
{
    TSpriteEnemigo* enemy;
    PMAImageBank ib = map.getGraphBank(1)->getImageBank();
    PMAImageBank ib1 = map.getGraphBank(3)->getImageBank();

    // Jorobado (Pantallas 1 a 4)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_HUMP,ib,ib);
    enemy->setRoute(1,6,4,8,9,4);
    iv_Enemy.addElement(enemy);
    
    // Fantasma (Pantallas 12 a 6)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(12,2,6,9,7,4);
    iv_Enemy.addElement(enemy);

    // Fantasma (Pantallas 13 a 23)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(13,4,23,4,7,4);
    iv_Enemy.addElement(enemy);

    // Fantasma (Pantallas 17 a 25)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(17,10,25,10,7,4);
    iv_Enemy.addElement(enemy);

    // Jorobado (Pantallas 20 a 22)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_HUMP1,ib,ib);
    enemy->setRoute(20,3,22,12,9,4);
    iv_Enemy.addElement(enemy);

    // Fantasma (Pantallas 79 a 65)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(79,12,65,10,7,4);
    iv_Enemy.addElement(enemy);

    // Lobo (Pantallas 55 a 63)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_WOLF,ib,ib);
    enemy->setRoute(55,15,63,17,9,4);
    iv_Enemy.addElement(enemy);

    // Jorobado (Pantallas 57 a 59)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_HUMP,ib,ib);
    enemy->setRoute(57,12,59,5,10,4);
    iv_Enemy.addElement(enemy);

    // Lobo (Pantallas 26 a 28)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_WOLF,ib,ib);
    enemy->setRoute(26,2,28,22,10,4);
    iv_Enemy.addElement(enemy);

    // Fantasma (Pantallas 69 a 71)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(69,10,71,9,8,4);
    iv_Enemy.addElement(enemy);

    // Jorobado (Pantallas 71 a 76)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_HUMP1,ib,ib);
    enemy->setRoute(71,16,76,17,9,4);
    iv_Enemy.addElement(enemy);

    // Fantasma (Pantallas 68 a 74)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(68,18,74,5,7,4);
    iv_Enemy.addElement(enemy);

    // Jorobado (Pantallas 50 a 46)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_HUMP,ib,ib);
    enemy->setRoute(50,2,46,22,9,4);
    iv_Enemy.addElement(enemy);

    // Fantasma (Pantallas 65 a 53)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(65,17,53,14,7,4);
    iv_Enemy.addElement(enemy);

    // Fantasma (Pantallas 38 a 41)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(38,21,41,14,7,4);
    iv_Enemy.addElement(enemy);

    // Fantasma (Pantallas 29 a 32)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(29,6,32,12,7,4);
    iv_Enemy.addElement(enemy);

    // Jorobado (Pantallas 31 a 33)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_HUMP1,ib,ib);
    enemy->setRoute(31,15,33,11,9,4);
    iv_Enemy.addElement(enemy);
    
    // Fantasma (Pantallas 86 a 36)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(86,3,36,6,7,4);
    iv_Enemy.addElement(enemy);

    // Lobo (Pantallas 36 a 34)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_GHOST,ib1,ib);
    enemy->setRoute(36,19,34,12,7,4);
    iv_Enemy.addElement(enemy);

    // Lobo (Pantallas 83 a 81)
    //
    enemy = new TSpriteEnemigo(TSpriteEnemigo::T_WOLF,ib,ib);
    enemy->setRoute(83,15,81,6,9,4);
    iv_Enemy.addElement(enemy);
}

void tsc_game::dispatchMessages(void)
{
    while (TSpiritsGame::msgQueue.hasMessage())
    {
        TMAMsg& msg = TSpiritsGame::msgQueue.getMessage();

        switch (msg.getID())
        {
            case MSG_SPHERE_GOT:

                // Se ha cogido la bola magica
                //
                sprBook->validate();
                sprMagic->validate();
                sprCasandra->validate();
                sprWorf->validate();
                sprEagle->validate();
                break;
                
            case MSG_BOOK_GOT:
                sprBrujo->getBook();
                break;
                
            case MSG_MAGIC_GOT:
                sprBrujo->getMagic();
                break;
                
            case MSG_CASANDRA_GOT:
                sprBrujo->getCasandra();
                break;
                
            case MSG_WORF_GOT:
                sprBrujo->getWorf();
                break;
                
            case MSG_BAD_GOT:
                sprBrujo->getBad();
                gotoFinish();
                break;
                
            case MSG_PUSH_HANDLE:

                // Se ha pulsado una palanca
                //
                handlePushed((long)msg.getParam());
                break;
        }
        TSpiritsGame::msgQueue.dispatchMessage();
    }
}

bool tsc_game::doRun(void)
{
    // Animacion Alpha Pantalla Inferior
    //
    alpha += alpha_dir;
    if (alpha<80 || alpha>120) alpha_dir=-alpha_dir;    
    
    // Activamos los sonidos de las pantallas
    //
    doSounds();

    switch (ii_Estado)
    {
        case S_GAME:

            // Despachamos los mensajes pendientes si los hay
            //
            dispatchMessages();
            detectTrainer();

            // Localizar siguiente objetivo
            //
            if (GETKEY(K_NEXT) && sprSphere->isGot())
            {
                objetivo++;
                if (objetivo>5) objetivo=0;
            }
            else if (GETKEY(K_PAUSE))
            {
                
                setState(S_PAUSE);
                return true;
            }
            else if (GETKEY(K_ABORT))
            {
                setState(S_ABORT);
                return true;
            }

            if (sprBrujo->getScreenID()!=screenUp.screen->getID())
            {
                prepareScreen(screenUp,sprBrujo->getScreenID());

                // Ahora inicializamos los enemigos aleatorios.
                //
                sprCat->init(sprBrujo->getScreenID());
                sprArchier->init(sprBrujo->getScreenID());
                return true;
            }

            // Si se estaba abriendo una puerta, y ya termino de hacerse
            // volvemos a mostrar la pantalla que haba anteriormente abajo
            //
            if (openingdoor!=NULL)
            {
                if (openingdoor->isOpen())
                {
                    sounds.snd_door.stop();

                    // Caso que se haya abierto la columna, de la pantalla 39
                    // luego debe abrirse la columna de la 40.
                    //
                    if (openingdoor->getScreenID()==39)
                    {
                        showOpeningDoor(rooms[40].door);
                    }
                    else
                    {
                        openingdoor = NULL;
                        if (lastScreenDown!=0)
                        {
                            prepareScreen(screenDown,lastScreenDown);
                            lastScreenDown=0;
                        }
                    }
                }
                else
                {
                    // Si la puerta que se esta abriendo, no se ve en ninguna pantalla
                    // la animamos, para que se acabe de abrir.
                    //
                    if (openingdoor->getScreenID()!=screenDown.screen->getID() &&
                        openingdoor->getScreenID()!=screenUp.screen->getID()) openingdoor->animate();
                }
            }
            else
            {
                if (objetivos[objetivo]->getScreenID()!=screenDown.screen->getID())
                {
                    prepareScreen(screenDown,objetivos[objetivo]->getScreenID());
                }
            }


            // Animamos los Sprites
            //
            sprBrujo->animate();
            sprCat->animate();
            sprArchier->animate();
            sprSphere->animate();
            if (sprBook->isValid()) sprBook->animate();
            if (sprMagic->isValid()) sprMagic->animate();
            
            if (sprCasandra->isValid()) sprCasandra->animate();
            if (sprWorf->isValid()) sprWorf->animate();
            if (sprEagle->isValid()) sprEagle->animate();

            ib_enemy_stopped = false;
            
            for (int i=0;i<iv_Enemy.size();i++)
            {
                TSpriteEnemigo *enemy = (TSpriteEnemigo*)iv_Enemy.elementAt(i);
                enemy->animate();
                ib_enemy_stopped |= (enemy->getScreenID()==sprBrujo->getScreenID() && enemy->isStopped());
            }
            
            // Animamos los graficos de las pantallas.
            //
            doAnimateRoom(screenUp);

            // Si la pantalla de abajo, es distinta a la de arriba
            // entonces animamos los graficos, sino no, puesto que ya
            // se ha hecho en la llamada anterior.
            //
            if (screenUp.screen!=screenDown.screen) doAnimateRoom(screenDown);

            // Descontamos vida, si precisa.
            //
            if (sprBrujo->getLives()>=0 && sprBrujo->getEnerty()==0)
            {
                myGame->playSfx(sounds.snd_die);
                sprBrujo->setLiveDown();
            }
            if (sprBrujo->getLives()<0)
            {
                setState(S_END);
                return true;
            }

            // Si ha caido una roca, final de la partida.
            //
            if (rooms[sprBrujo->getScreenID()].lamp)
            {
                if (rooms[sprBrujo->getScreenID()].lamp->isRock() && rooms[sprBrujo->getScreenID()].lamp->isOnFloor())
                {
                    if (sprBrujo->getScreenID()==67)
                    {
                        setState(S_END);
                    }
                    else
                    {
                        delete rooms[sprBrujo->getScreenID()].lamp;
                        rooms[sprBrujo->getScreenID()].lamp=NULL;
                        myGame->playSfx(sounds.snd_die);
                        sprBrujo->setLiveDown();
                    }
                    return true;
                }
            }
            
            // Si se ha entrado en la pantalla de los ojos, quitamos una vida
            //
            if (sprBrujo->getScreenID()==72)
            {
                waitfps++;
                if (waitfps>100)
                {
                    myGame->playSfx(sounds.snd_die);
                    sprBrujo->setLiveDown();
                    sprBrujo->setScreenID(70);
                    sprBrujo->putAt(8,10);
                }
            }
            
            // Si estamos en la pantalla 69 en la zona de fuego, quitamos vida.
            //
            if (sprBrujo->getScreenID()==69)
            {
                if (sprBrujo->getHotspotMapX()<7 && sprBrujo->getHotspotMapY()>8 && sprBrujo->getFlashTime()<=0)
                    sprBrujo->setEnergyDown(2);
            }
            break;
            
        case S_PAUSE:

            if (myGame->inputController.isAnyHoldDown())
            {
                myGame->inputController.clearStatus();
                setState(S_GAME);
            }
            break;

        case S_ABORT:

            if (GETKEY(K_KEY_LEFT) || GETKEY(K_KEY_RIGHT) || GETKEY(K_KEY_UP) || GETKEY(K_KEY_DOWN) )
            {
                st_abort = !st_abort;
            }
            else if (GETKEY(K_KEY_SELECT0)||GETKEY(K_KEY_SELECT1))
            {
                if (st_abort) gotoMenu();
                else setState(S_GAME);
            }
            else if (GETKEY(K_KEY_ESCAPE))
            {
                setState(S_GAME);
            }
            break;
            
        case S_END:

            if (GETKEY(K_KEY_SELECT0)||GETKEY(K_KEY_SELECT1))
            {
                gotoMenu();
                return true;
            }
            break;
            
        case S_ENTERCODE:
            
            enterCode();
            break;
    }
    
    // Devolvemos false, para ejecutar el comportamiento por defecto.
    //
    return true;
}

void tsc_game::showOpeningDoor(PSpriteDoor d)
{
    d->open();
    openingdoor = d;

    if (!sounds.snd_door.isPlaying()) myGame->playSfx(sounds.snd_door,true,80);
    
    if (d->getScreenID()!=sprBrujo->getScreenID())
    {
        if (lastScreenDown==0)
        {
            lastScreenDown = screenDown.screen->getID();
        }
        prepareScreen(screenDown,d->getScreenID());
    }
}

void tsc_game::handlePushed(int room)
{
    // Primero determinamos cual ha sido la palanca de la pantalla
    // con la que el brujo ha chocado.
    //
    PSpriteHandle hnd=NULL;
    PSpriteDoor   door=NULL;
    int hpx=0;
    
    for (int i=0;(i<rooms[room].handles->size()) && (hnd==NULL);i++)
    {
        PSpriteHandle h=(PSpriteHandle)rooms[room].handles->elementAt(i);
        if (h->isPushedBy(sprBrujo))
        {
            hnd=h;
        }
    }
    
    if (hnd)
    {
        // Si la palanca ya esta pulsada, no hacemos nada
        //
        if (hnd->isPushed()) return;
        
        // Pulsamos la palanca
        //
        hnd->push(); 
        myGame->playSfx(sounds.snd_handle);
        
        switch (room)
        {
            case 5: // Palanca de la pantalla 5 -> Muro 5
                    door = rooms[5].door;
                    break;
                    
            case 23: // Palanca de la pantalla 23 -> Muro 8
                    door = rooms[8].door;
                    break;

            case 22: // Palanca de la pantalla 22 -> Muro 22
                    door = rooms[22].door;
                    break;
                    
            case 55: // Palanca de la pantalla 55 -> Muro 80
                    door = rooms[80].door;
                    break;

            case 38: // Palanca de la pantalla 38 -> Muro 39 y 40
                    door = rooms[39].door;
                    break;

            case 44: // Hay 3 Palancas

                    hpx = hnd->position.x / 20;
                    
                    switch (hpx)
                    {
                        case 4: // Izquierda
                            door = rooms[44].door;
                            break;
                            
                        case 6: // Centro
                            if (rooms[44].lamp) delete rooms[44].lamp;
                            rooms[44].lamp = new TSpriteLamp(map.getGraphBank(3)->getImageBank(),TSpriteLamp::T_ROCK);
                            rooms[44].lamp->putAt(5,-3);
                            rooms[44].lamp->setInit(0,8);
                            rooms[44].lamp->setScreenID(44);
                            break;
                            
                        case 8: // Derecha
                            door = rooms[45].door;
                            break;
                    }
                    break;

            case 65: // Palanca de la pantalla 65 -> Muro 62 (Evitar quedar bloqueado)
                    door = rooms[62].door;
                    break;

            case 67: // Hay 2 Palancas

                    hpx = hnd->position.x / 20;

                    switch (hpx)
                    {
                        case 3: // Izquierda
                            door = rooms[67].door;
                            break;
                            
                        case 5: // Derecha
                            if (rooms[67].lamp) delete rooms[67].lamp;
                            rooms[67].lamp = new TSpriteLamp(map.getGraphBank(3)->getImageBank(),TSpriteLamp::T_ROCK);
                            rooms[67].lamp->putAt(4,-3);
                            rooms[67].lamp->setInit(0,8);
                            rooms[67].lamp->setScreenID(67);
                            break;
                    }
                    break;

             case 75: // Palanca de la pantalla 75 -> Muro 75
                    door = rooms[75].door;
                    break;

             case 76: // Palanca de la pantalla 76 -> Muro 62
                    door = rooms[62].door;
                    break;
        }
        
        // Caso que se haya localizado una puerta que abrir..
        //
        if (door)
        {
            // Si la puerta a abrir, esta en una pantalla que no es la misma
            // donde se encuentra la palanca, entonces la mostramos momentaneamente
            // en la pantalla inferior del juego, hasta que se haya abierto dicha puerta
            //
            showOpeningDoor(door);
        }
    }
}

void tsc_game::doSounds(void)
{
    int scrid = sprBrujo->getScreenID();

    // Control del sonido de las antorchas.
    //
    if (rooms[scrid].sound_torch)
    {
        if (!sounds.snd_fire.isPlaying()) myGame->playSfx(sounds.snd_fire,false,40);
    }
    
    // Control del sonido de los grillos
    //
    if (scrid==26 || scrid==27 || scrid==28)
    {
        if (!sounds.snd_cricket.isPlaying()) myGame->playSfx(sounds.snd_cricket,false,40);
    }
    
    // Sonido del reloj
    //
    if (scrid==14 || scrid==37)
    {
        if (!sounds.snd_tictac.isPlaying()) myGame->playSfx(sounds.snd_tictac);
    }
    
    // Sonido del Caldero
    //
    if (scrid==60 || scrid==71)
    {
        if (!sounds.snd_kettle.isPlaying()) myGame->playSfx(sounds.snd_kettle);
    }

    // Si en la pantalla donde esta el brujo, hay enemigos paralizados
    // ponemos el sonido de campo de fuerza.
    //
    if (ib_enemy_stopped)
    {
        if (!sounds.snd_energy.isPlaying()) myGame->playSfx(sounds.snd_energy,false,100);
    }
    else
    {
        if (sounds.snd_energy.isPlaying()) sounds.snd_energy.stop();
    }
}

void tsc_game::doAnimateRoom(scrCanvas& scr)
{
    if (scr.screen==NULL) return;

    int scrid = scr.screen->getID();
    map.animateGraphOperations(scr.screen,TMAGraphOperation::OP_ANIM_NORMAL);
    map.animateGraphOperations(scr.screen,TMAGraphOperation::OP_ANIM_FRONT);
    
    // Si hay palancas, las animamos
    //
    if (rooms[scrid].handles)
    {
        for (int i=0;i<rooms[scrid].handles->size();i++)
        {
            PSpriteHandle spr = (PSpriteHandle)rooms[scrid].handles->elementAt(i);
            spr->animate();
        }
    }
    // Animamos las luces si las hay
    //
    if (rooms[scrid].lights)
    {
        for (int i=0;i<rooms[scrid].lights->size();i++)
        {
            PSpriteAnim spr = (PSpriteAnim)rooms[scrid].lights->elementAt(i);
            spr->animate();
        }
    }
    
    // Animamos la lampara si la hay
    //
    if (rooms[scrid].lamp) rooms[scrid].lamp->animate();
    
    // Animamos la puerta, si la hay
    //
    if (rooms[scrid].door) rooms[scrid].door->animate();

}

void tsc_game::doPaint(TMABitmap& buff)
{
    char txt[100];
    
    if (screenUp.scrBuffer!=NULL) 
    {
        delete screenUp.scrBuffer;
        screenUp.scrBuffer=NULL;
    }

    if (screenDown.scrBuffer!=NULL) 
    {
        delete screenDown.scrBuffer;
        screenDown.scrBuffer=NULL;
    }
    
    #ifdef SMALL_GAME
        screenUp.scrBuffer = TMABitmap::create(&buff,0,0,240,120);
        screenDown.scrBuffer = TMABitmap::create(&buff,0,120,240,120);
    #else
        screenUp.scrBuffer = TMABitmap::create(&buff,0,0,480,240);
        screenDown.scrBuffer = TMABitmap::create(&buff,0,240,480,240);
    #endif
    
    // Pintamos la habitacin de arriba
    //
    doPaintRoom(screenUp);
    
    // Pintamos la habitacin de abajo
    //
    doPaintRoom(screenDown);
    
    // Dibujamos el panel informativo
    //
    doPaintInfoPanel(buff);
    
    switch (ii_Estado)
    {
        case S_PAUSE:
            doPaintTextBig(*screenUp.scrBuffer,_("PAUSA"),-1,100);
            break;
            
        case S_END:
            sprintf(txt,_("HAS COMPLETADO EL %i%%"),percentdone);
            doPaintTextBig(*screenUp.scrBuffer,txt,-1,90);
            doPaintTextBig(*screenUp.scrBuffer,_("DE LA AVENTURA"),-1,130);
            break;
            
        case S_ABORT:
            doPaintTextBig(*screenUp.scrBuffer,_("Seguro que desea abandonar?"),-1,90);
            if (st_abort)
                doPaintTextBig(*screenUp.scrBuffer,_("Si, gracias"),-1,130);
            else
                doPaintTextBig(*screenUp.scrBuffer,_("No, por favor"),-1,130);
            break;
            
        case S_ENTERCODE:
            // Pintamos el bicho
            //
            PMAImageBank bank = map.getImageBank(0);
            buff.draw(bank->getBitmap(7),200,50);
            doPaintTextBig(buff,_("Trainer Mode"),-1,250);
            doPaintTextBig(buff,_("Try Code"),-1,300);
            doPaintTextBig(buff,codebuff,-1,360);
            break;

    }
}

void tsc_game::doPaintRoom(scrCanvas& scr)
{
    if (scr.screen==NULL) return;
    
    // Obtenemos el ID de la pantalla
    //
    int scrid = scr.screen->getID();

    scr.scrBuffer->clear(0);
    map.drawScreenGraphOperations(*scr.scrBuffer,scr.screen,TMAGraphOperation::OP_NORMAL);
    scr.scrBuffer->draw(scr.scrImage,0,0);
    map.drawScreenGraphOperations(*scr.scrBuffer,scr.screen,TMAGraphOperation::OP_ANIM_NORMAL);

    // Si hay puerta horizontal, la pintamos antes que los sprites
    //
    if (rooms[scrid].door && !rooms[scrid].door->isVertical()) rooms[scrid].door->draw(*scr.scrBuffer);
    
    // Dibujamos aqui las Sombras y los Sprites...
    //
    if (sprSphere->getScreenID()==scrid) sprSphere->drawShadow(*scr.scrBuffer,rooms[scrid].lights);
    if (sprBook->isValid() && sprBook->getScreenID()==scrid) sprBook->drawShadow(*scr.scrBuffer,rooms[scrid].lights);
    if (sprMagic->isValid() && sprMagic->getScreenID()==scrid) sprMagic->drawShadow(*scr.scrBuffer,rooms[scrid].lights);

    if (sprCasandra->isValid() && sprCasandra->getScreenID()==scrid) sprCasandra->drawShadow(*scr.scrBuffer,rooms[scrid].lights);
    if (sprWorf->isValid() && sprWorf->getScreenID()==scrid) sprWorf->drawShadow(*scr.scrBuffer,rooms[scrid].lights);
    if (sprEagle->isValid() && sprEagle->getScreenID()==scrid) sprEagle->drawShadow(*scr.scrBuffer,rooms[scrid].lights);

    if (sprBrujo->getScreenID()==scrid)
    {
        sprBrujo->drawShadow(*scr.scrBuffer,rooms[scrid].lights);
        if (sprCat->isValid()) sprCat->drawShadow(*scr.scrBuffer,rooms[scrid].lights);
        if (sprArchier->isValid()) sprArchier->drawShadow(*scr.scrBuffer,rooms[scrid].lights);
    }
    for (int i=0;i<iv_Enemy.size();i++)
    {
        TSpriteEnemigo *enemy = (TSpriteEnemigo*)iv_Enemy.elementAt(i);
        if (enemy->getScreenID()==scrid) enemy->drawShadow(*scr.scrBuffer,rooms[scrid].lights);
    }

    // Si hay lampara, pintamos su sombra
    //
    if (rooms[scrid].lamp) rooms[scrid].lamp->drawShadow(*scr.scrBuffer,rooms[scrid].lights);

    if (sprSphere->getScreenID()==scrid) sprSphere->draw(*scr.scrBuffer);
    if (sprBook->isValid() && sprBook->getScreenID()==scrid) sprBook->draw(*scr.scrBuffer);
    if (sprMagic->isValid() && sprMagic->getScreenID()==scrid) sprMagic->draw(*scr.scrBuffer);

    if (sprCasandra->isValid() && sprCasandra->getScreenID()==scrid) sprCasandra->draw(*scr.scrBuffer);
    if (sprWorf->isValid() && sprWorf->getScreenID()==scrid) sprWorf->draw(*scr.scrBuffer);
    if (sprEagle->isValid() && sprEagle->getScreenID()==scrid) sprEagle->draw(*scr.scrBuffer);

    if (sprBrujo->getScreenID()==scrid)
    {
        sprBrujo->draw(*scr.scrBuffer);
        sprCat->draw(*scr.scrBuffer);
        sprArchier->draw(*scr.scrBuffer);
    }

    for (int i=0;i<iv_Enemy.size();i++)
    {
        TSpriteEnemigo *enemy = (TSpriteEnemigo*)iv_Enemy.elementAt(i);
        if (enemy->getScreenID()==scrid) enemy->draw(*scr.scrBuffer);
    }

    // Dibujamos el resto del decorado
    //
    map.drawScreenGraphOperations(*scr.scrBuffer,scr.screen,TMAGraphOperation::OP_FRONT);
    map.drawScreenGraphOperations(*scr.scrBuffer,scr.screen,TMAGraphOperation::OP_ANIM_FRONT);

    // Si hay puerta vertical, la pintamos tras los sprites
    //
    if (rooms[scrid].door && rooms[scrid].door->isVertical()) rooms[scrid].door->draw(*scr.scrBuffer);

    // Si hay palancas, las pintamos
    //
    if (rooms[scrid].handles)
    {
        for (int i=0;i<rooms[scrid].handles->size();i++)
        {
            PSpriteHandle spr = (PSpriteHandle)rooms[scrid].handles->elementAt(i);
            spr->draw(*scr.scrBuffer);
        }
    }

    // Si hay lampara, la pintamos
    //
    if (rooms[scrid].lamp) rooms[scrid].lamp->draw(*scr.scrBuffer);

    // Si hay luces, las pintamos
    //
    if (rooms[scrid].lights)
    {
        for (int i=0;i<rooms[scrid].lights->size();i++)
        {
            PSpriteAnim spr = (PSpriteAnim)rooms[scrid].lights->elementAt(i);
            spr->draw(*scr.scrBuffer);
        }
    }

    // Dibujamos la interferencia (si es la pantalla inferior)
    //
    if (&scr == &screenDown)
    {
        if (objetivos[objetivo]->getScreenID()==scrid)
        {
            switch(objetivo)
            {
            case 0:
                if (sprSphere->isGot()) doPaintTextBig(*scr.scrBuffer,_("Bola Conseguida"),-1,100);
                break;
            case 1:
                if (sprBook->isGot()) doPaintTextBig(*scr.scrBuffer,_("Libro Conseguido"),-1,100);
                break;
            case 2:
                if (sprMagic->isGot()) doPaintTextBig(*scr.scrBuffer,_("Varita Conseguida"),-1,100);
                break;
            case 3:
                if (sprCasandra->isGot()) doPaintTextBig(*scr.scrBuffer,_("Casandra Liberada"),-1,100);
                break;
            case 4:
                if (sprWorf->isGot()) doPaintTextBig(*scr.scrBuffer,_("Worf Liberado"),-1,100);
                break;
            case 5:
                if (sprEagle->isGot()) doPaintTextBig(*scr.scrBuffer,_("Maligno Vencido"),-1,100);
                break;
            }
        }
        
        #ifndef TARGET_GP2X
            interference->setAlpha(alpha);
            scr.scrBuffer->draw(interference,0,0);
        #else
            scr.scrBuffer->drawRectFilled(0,0,480,240,MAKERGBACOLOR(0,100,200,60));
        #endif
    }


    // Dibujamos el reborde de la pantalla.
    //
    scr.scrBuffer->draw(border_up,0,0);
    scr.scrBuffer->draw(border_right,448,32);
    scr.scrBuffer->draw(border_down,0,208);
    scr.scrBuffer->draw(border_left,0,32);

}

void tsc_game::doPaintInfoPanel(TMABitmap& buff)
{
    buff.draw(infoPanel,480,0);
    
    // Pintamos las vidas o muertes
    //
    int lives = sprBrujo->getLives();

    for (int i=0;i<3;i++)
    {
        buff.draw( (i<lives)?imgLive:imgDie,508+i*35,43);
    }
    
    // Pintamos la energia
    //
    buff.draw(imgEnergy,0,0,(sprBrujo->getEnerty()*106)/100,30,508,164);
    
    // Pintamos los objetos cogidos
    //
    if (sprSphere->isGot()) sprSphere->drawOnPanel(buff,512,250);
    if (sprBook->isGot()) sprBook->drawOnPanel(buff,544,290);
    if (sprMagic->isGot()) sprMagic->drawOnPanel(buff,530,357);
}

int tsc_game::doPaintText(TMABitmap& buff,const char *txt,int x,int y)
{
    int width = myGame->fontGame.getTextWidth(txt);
    #ifdef SMALL_GAME
        int ax = (x==-1)?(buff.getWidth()*2 - width)/2:x;
    #else
        int ax = (x==-1)?(buff.getWidth() - width)/2:x;
    #endif    
    int ay = myGame->fontGame.getFontHeight();

    // Sombras
    int col = MAKERGBCOLOR(0,0,0);
    myGame->fontGame.drawText(buff,txt,ax-1,y-1,col,true);
    myGame->fontGame.drawText(buff,txt,ax  ,y-1,col,true);
    myGame->fontGame.drawText(buff,txt,ax+1,y-1,col,true);
    myGame->fontGame.drawText(buff,txt,ax-1,y+1,col,true);
    myGame->fontGame.drawText(buff,txt,ax  ,y+1,col,true);
    myGame->fontGame.drawText(buff,txt,ax+1,y+1,col,true);
    myGame->fontGame.drawText(buff,txt,ax+3,y+3,col,true);

    // Mitad Inferior
    myGame->fontGame.drawText(buff,txt,ax,y,MAKERGBCOLOR(230,220,100),true);

    // Mitad Superior
    int rx,ry,rx1,ry1;
    buff.getClipRect(rx,ry,rx1,ry1);
    #ifdef SMALL_GAME
        buff.setClipRect(rx,y/2,rx1,ay/4);
    #else
        buff.setClipRect(rx,y,rx1,ay/2);
    #endif
    myGame->fontGame.drawText(buff,txt,ax,y,MAKERGBCOLOR(240,140,40),true);
    buff.setClipRect(rx,ry,rx1,ry1);

    return width;
}

int tsc_game::doPaintTextBig(TMABitmap& buff,const char *txt,int x,int y)
{
    int width = myGame->fontMenu.getTextWidth(txt);
    #ifdef SMALL_GAME
        int ax = (x==-1)?(buff.getWidth()*2 - width)/2:x;
    #else
        int ax = (x==-1)?(buff.getWidth() - width)/2:x;
    #endif
    int ay = myGame->fontMenu.getFontHeight();
    
    // Sombras
    myGame->fontMenu.drawText(buff,txt,ax+6,y+6,MAKERGBCOLOR(0,0,0),true);
    myGame->fontMenu.drawText(buff,txt,ax+2,y+2,MAKERGBCOLOR(20,10,0),true);

    // Mitad Inferior
    myGame->fontMenu.drawText(buff,txt,ax,y,MAKERGBCOLOR(200,220,250),true);

    // Mitad Superior
    int rx,ry,rx1,ry1;
    buff.getClipRect(rx,ry,rx1,ry1);
    #ifdef SMALL_GAME
        buff.setClipRect(rx,y/2,rx1,ay/4+1);
    #else
        buff.setClipRect(rx,y,rx1,ay/2+2);
    #endif
    myGame->fontMenu.drawText(buff,txt,ax,y,
        (ii_Estado==S_PAUSE)?MAKERGBCOLOR(230,220,100):MAKERGBCOLOR(100,220,250),true);
    buff.setClipRect(rx,ry,rx1,ry1);

    return width;
}


void tsc_game::gotoMenu(void)
{
    myGame->setScene(new tsc_menu());
    if (myGame->gameConfig.isMusicOn())
    {
        myGame->ogg_music.load(myGame->gamePath.makePath(myGame->gamePath.getDataPrefix(),"music/spirits.ogg").text());
        myGame->playMusic(myGame->ogg_music,true,128);
    }
}

void tsc_game::gotoFinish(void)
{
    myGame->setScene(new tsc_end());
    if (myGame->gameConfig.isMusicOn())
    {
        myGame->ogg_music.load(myGame->gamePath.makePath(myGame->gamePath.getDataPrefix(),"music/spirits.ogg").text());
        myGame->playMusic(myGame->ogg_music,true,128);
    }
}

void tsc_game::detectTrainer(void)
{
    // ---- TRUCOS DEL JUEGO
    //    
    #if !defined(TARGET_GP2X) && !defined(TARGET_WII)
        
        if (TMAGame::KeyState[SDLK_F1] && TMAGame::KeyState[SDLK_m] && TMAGame::KeyState[SDLK_a] && TMAGame::KeyState[SDLK_F10])
        {
            setState(S_ENTERCODE);
            codeindex=0;
            memset(codebuff,0,sizeof(codebuff));
            myGame->playSfx(sounds.snd_abra);
        }
        
    #else
        static TSpiritsGame::KEY_VALUE keysequence[6] = { 
                                      TSpiritsGame::K_KEY_SELECT0,
                                      TSpiritsGame::K_KEY_UP,
                                      TSpiritsGame::K_KEY_DOWN,
                                      TSpiritsGame::K_KEY_UP,
                                      TSpiritsGame::K_KEY_DOWN,
                                      TSpiritsGame::K_KEY_SELECT0 };
                                          
        for (int i=0;i<=sizeof(keysequence);i++)
        {                
            if (i==6)
            {
                setState(S_ENTERCODE);
                codeindex=0;
                memset(codebuff,0,sizeof(codebuff));
                myGame->playSfx(sounds.snd_abra);
                return;
            }

            if (!codebuff[i])
            {
                if (i>0 && myGame->inputController.isHoldDown(keysequence[i-1])) return;
				if (myGame->inputController.isPressDown(keysequence[i]))
				{
                    codebuff[i]=true;
                    return;
                }
                else break;
            }
        }
        
        // Si hay una tecla pulsada, pero no es ninguna de las esperadas, 
        // empezamos de nuevo.
        //
		if (myGame->inputController.isAnyHoldDown()>0)
        {
            for (int li_i=0;li_i<sizeof(keysequence);li_i++) codebuff[li_i]=false;
        }
        
    #endif    
}

void tsc_game::enterCode(void)
{
    bool lb_salir = false;
    
    #if !defined(TARGET_GP2X) && !defined(TARGET_WII)
    
        if (myGame->getLastAsciiKeyPressed())
        {
            char lread = myGame->getLastAsciiKeyPressed();
            myGame->releaseLastAsciiKeyPressed();
            if (lread==24 || lread==13)
            {
                lb_salir=true;
            }
            else if (lread==8)
            {
                if (codeindex>0)
                {
                    codeindex--;
                    codebuff[codeindex]=0;
                }
            }   
            else
            {
                codebuff[codeindex++]=lread;
                codebuff[codeindex]=0;
                if (codeindex>sizeof(codebuff)-1) lb_salir=true;
            }
        }     
    
        if (lb_salir)
        {
            if (strcmp(codebuff,"morelives")==0)
            {
                sprBrujo->moreLives();
            }
            else if (strcmp(codebuff,"gototarget")==0)
            {
                sprBrujo->setScreenID(objetivos[objetivo]->getScreenID());
            }
            else if (strcmp(codebuff,"gotofinish")==0)
            {
                gotoFinish();
            }
            setState(S_GAME);
        }
        
    #else

            if (GETKEY(K_KEY_LEFT))
            {
                sprBrujo->moreLives();
                lb_salir=true;
            }
            else if (GETKEY(K_KEY_UP))
            {
                sprBrujo->setScreenID(objetivos[objetivo]->getScreenID());
                lb_salir=true;
            }
            else if (GETKEY(K_KEY_RIGHT))
            {
                gotoFinish();
                lb_salir=true;
            }
            
            if (lb_salir) setState(S_GAME);
    
    #endif
}

#undef GETKEY

#endif
