#ifndef _tsc_logo_
#define _tsc_logo_

#include "scenes.hpp"

REGISTER_CLASS_ID(tsc_logo);

tsc_logo::tsc_logo():TMAScene()
{
    myGame = (PSpiritsGame)TMAGame::getCurrentGame();
    io_logo = io_logo1 = NULL;
}

tsc_logo::~tsc_logo()
{
    if (io_logo) delete io_logo;
    if (io_logo1) delete io_logo1;
}

bool tsc_logo::doInit(const PMAObject params)
{
    Path& path = myGame->gamePath;
    ii_Estado=IN;
    fLight.doInitFade(TMAFader::FADE_IN,VEL_FADE_LIGHT);
    fBoxes.doInitFade(TMAFader::FADE_IN,640,480,TILESIZE,TILESIZE,VEL_FADE_BOX);

    // Tiempo de espera de la primera pantalla de LOGO
    waitScreen.initCountDown(100);

    io_logo = TMABitmap::loadFromFile(path.makePath(path.getDataPrefix(),"pics/logo.png").text());
    io_logo1 = TMABitmap::loadFromFile(path.makePath(path.getDataPrefix(),"pics/screen.png").text());

    if (myGame->gameConfig.isMusicOn())
    {
        ogg_sound.load(path.makePath(path.getDataPrefix(),"sound/logo.ogg").text());
        myGame->playMusic(ogg_sound,false);
    }
    myGame->setGameFPS(32);
    return true;
}        

bool tsc_logo::doRun(void)
{
    Path& path = myGame->gamePath;
        
    // Si se pulsa alguna tecla, pasamos al siguiente estado de logo
    //
    if (myGame->inputController.isAnyHoldDown())
    {
        // Si la tecla es ESCAPE, directamente a menu de juego
        //
        if (myGame->inputController.isPressDown(myGame->K_KEY_ESCAPE))
        {
            ii_Estado=GOTO_MENU;
            if (myGame->gameConfig.isMusicOn())
            {
                ogg_sound.stop();
                ogg_sound.close();
                myGame->ogg_music.load(path.makePath(path.getDataPrefix(),"music/spirits.ogg").text());
                myGame->playMusic(myGame->ogg_music,true,128);
            }
            return true;
        }        
        
        myGame->inputController.clearStatus();
        
        switch (ii_Estado)
        {
            case IN:
                fLight.doInitFade(TMAFader::FADE_OUT,VEL_FADE_LIGHT);
                ii_Estado=OUT;
                break;

            case IN1:
                fLight.doInitFade(TMAFader::FADE_OUT,VEL_FADE_LIGHT);
                ii_Estado=OUT1;
                break;
            
            case SPECTRUM0:
            case SPECTRUM1:
                fSpectrum.finish();
                ii_Estado=SPECTRUM1;
                break;
                                
            case SPIRITS1:
                ii_Estado=SPIRITS2;
                break;
                
            case SPIRITS2:
                waitScreen.finish();
                break;                         
        }
    }

    switch (ii_Estado)
    {
        case IN:
        case IN1:
                
            if (!fLight.doFade())
            {
                if (!waitScreen.doCount())
                {
                    fLight.doInitFade(TMAFader::FADE_OUT,VEL_FADE_LIGHT);
                    switch (ii_Estado)
                    {
                        case IN:
                            ii_Estado=OUT;
                            waitScreen.initCountDown(100);
                            break;
                        case IN1:
                            ii_Estado=OUT1;
                            break;
                    }
                }
            }
            break;
                
        case OUT:
            
            // Desapareciendo LOGO
            //
            fLight.doFade();
            if (!fLight.isFading())
            {
                // Eliminamos el BITMAP de LOGO y cargamos la imagen
                // de patrocinadores
                //
                delete io_logo;
                io_logo = TMABitmap::loadFromFile(path.makePath(path.getDataPrefix(),"pics/retrocompo.png").text());
                fLight.doInitFade(TMAFader::FADE_IN,VEL_FADE_LIGHT);

                // Imitamos la carga de spectrum.
                //
                ii_Estado=IN1;
            }
            break;
            
        case OUT1:

            fLight.doFade();
            if (!fLight.isFading())
            {
                // Eliminamos el BITMAP de LOGO y cargamos la imagen
                // del juego original
                //
                delete io_logo;
                io_logo = TMABitmap::loadFromFile(path.makePath(path.getDataPrefix(),"pics/screenold.png").text());

                // Si hay sonido, establecemos el de carga de spectrum
                //
                if (myGame->gameConfig.isMusicOn())
                {
                    ogg_sound.stop();
                    ogg_sound.load(path.makePath(path.getDataPrefix(),"sound/spectrum.ogg").text());
                    myGame->playMusic(ogg_sound,false);
                    fadeMusic.initCountUp(128,4);
                }
        
                // Imitamos la carga de spectrum.
                //
                fSpectrum.doInitFade();
                ii_Estado=SPECTRUM0;
            }
            break;
                                    
        case SPECTRUM1:
                
            // Simulando carga de spectrum
            //
            if (!fSpectrum.doFade())
            {
                // Comienzo de fundido con aparicion de presentacion moderna
                //
                if (myGame->gameConfig.isMusicOn())
                {
                    ogg_sound.stop();
                    ogg_sound.close();
                    myGame->ogg_music.load(path.makePath(path.getDataPrefix(),"music/spirits.ogg").text());
                    myGame->playMusic(myGame->ogg_music,true,0);
                    fadeMusic.initCountUp(128,2);
                }
                ii_Estado=SPECTRUM2;
            }
            break;
            
        case SPECTRUM2:
            
            fBoxes.doInitFade(TMAFader::FADE_IN,640,480,TILESIZE,TILESIZE,VEL_FADE_BOX);
            ii_Estado=SPIRITS1;
            break;
                                        
        case SPIRITS1:
                
            // Mostrando presentacion moderna
            //
            if (!fBoxes.doFade())
            {
                waitScreen.initCountDown(300);
                ii_Estado=SPIRITS2;
            }
            break;

        case SPIRITS2:

            if (!waitScreen.doCount())
            {
                fBoxes.doInitFade(TMAFader::FADE_OUT,640,480,TILESIZE,TILESIZE,VEL_FADE_BOX);
                ii_Estado = SPIRITS3;
            }
            break;

        case SPIRITS3:

            if (!fBoxes.doFade())
            {
                ii_Estado=GOTO_MENU;
            }
            break;
    }

    // Gestin del Sonido
    //
    if (myGame->gameConfig.isMusicOn())
    {
        switch (ii_Estado)
        {
            case SPIRITS1:
            case SPIRITS2:
            case SPIRITS3:

                if (fadeMusic.doCount())
                {
                    myGame->setMusicVolume(myGame->ogg_music,fadeMusic.getValue());
                }
                break;
        }
    }
    
    // Vamos al Menu Principal
    //
    if (ii_Estado==GOTO_MENU)
    {
        myGame->setScene(new tsc_menu());
    }
    return true;
} 

void tsc_logo::doPaint(TMABitmap& buff)
{    
    if (!io_logo) return;
    
    switch (ii_Estado)
    {
        case IN:
        case IN1:
        case OUT:
        case OUT1:
                
            // Apareciendo y Desapareciendo LOGO
            //
            buff.clear(0);
            fLight.doFade(buff,*io_logo);
            break;
            
        case SPECTRUM0:
                
            // Inicio de simulacion carga de spectrum
            //
            buff.clear(0);
            ii_Estado=SPECTRUM1;
            break;  
            
        case SPECTRUM1:
                
            // Simulando carga de spectrum
            //
            fSpectrum.doFade(buff,*io_logo);
            break;
                
        case SPECTRUM2:
                
            // Dibuja totalmente la presentacion antigua
            //
            buff.draw(io_logo,0,0);
            break; 
                
        case SPIRITS1:
                
            // Mostrando presentacion moderna
            //
            buff.draw(io_logo,0,0);
            fBoxes.doFade(buff,*io_logo1);
            break;

        case SPIRITS2:

            // Manteniendo un poco la presentacion moderna
            //
            io_logo1->disableAlpha();
            buff.draw(io_logo1,0,0);
            break;

        case SPIRITS3:
            
            // Desapareciendo presentacion moderna
            //
            buff.clear(0);
            fBoxes.doFade(buff,*io_logo1);
            break;
              
    }
}


#endif
