#ifndef _tsc_menu_
#define _tsc_menu_

#include "scenes.hpp"

REGISTER_CLASS_ID(tsc_menu);

tsc_menu::tsc_menu():TMAScene()
{
    myGame = (PSpiritsGame)TMAGame::getCurrentGame();
    io_screen=NULL;
    io_scrollbox=NULL;
    io_selector=NULL;
    memset(&StrScroll,0,sizeof(StrScroll));
}

tsc_menu::~tsc_menu()
{
    freeScrollText();
    if (io_screen) delete io_screen;
    if (io_scrollbox) delete io_scrollbox;
    if (io_selector) delete io_selector;
    if (io_volumegauge) delete io_volumegauge;
    if (io_volumeindicator) delete io_volumeindicator;
}

bool tsc_menu::doInit(const PMAObject params)
{
    Path& path = myGame->gamePath;
    movex=0;
    dirx=1;
    fgrama=0;
    ib_prepare_graph=true;
    ii_Estado=SHOW_SCREEN;
    io_screen = TMABitmap::loadFromFile(path.makePath(path.getDataPrefix(),"pics/menu.png").text());
    io_selector = TMABitmap::loadFromFile(path.makePath(path.getDataPrefix(),"pics/selector.png").text());
    io_volumegauge = TMABitmap::loadFromFile(path.makePath(path.getDataPrefix(),"pics/volumegauge.png").text());
    io_volumeindicator = TMABitmap::loadFromFile(path.makePath(path.getDataPrefix(),"pics/volumeindicator.png").text());
    
    #ifdef SMALL_GAME
        io_scrollbox = TMABitmap::create(320,125,false);
    #else
        io_scrollbox = TMABitmap::create(640,250,false);
    #endif
    
    fBoxes.doInitFade(TMAFader::FADE_IN,640,480,16,16,VEL_FADE_BOX);
    
    // La musica debe estar sonando ya
    //
    if (myGame->gameConfig.isMusicOn())
    {
        myGame->setMusicVolume(myGame->ogg_music,128);
    }
    
    if (myGame->gameConfig.isSfxOn())
    {
        snd_menuchange.load(path.makePath(path.getDataPrefix(),"sound/menu_chg.wav").text());
        snd_menuselect.load(path.makePath(path.getDataPrefix(),"sound/menu_sel.wav").text());
        snd_abracadabra.load(path.makePath(path.getDataPrefix(),"sound/abracadabra.wav").text());
    }
    myGame->setGameFPS(32);
    return true;
}

void tsc_menu::gotoState(tsc_menu::MenuState e,tsc_menu::MenuState next)
{
    switch ((int)e)
    {
        case HIDE_MENU:

            switch (ii_Estado)
            {
                case SHOW_MENU:
                    fLight.doInitFade1(TMAFader::FADE_OUT,640,210,VEL_FADE_LIGHT,0,20,0,170);
                    break;
            
                case SHOW_MENU_OPTIONS:
                    fLight.doInitFade1(TMAFader::FADE_OUT,640,250,VEL_FADE_LIGHT,0,0,0,150);
                    break;

                case SHOW_MENU_CONFIRM:
                    fLight.doInitFade1(TMAFader::FADE_OUT,640,130,VEL_FADE_LIGHT,0,40,0,190);
                    break;
            }
            break;

        case SHOW_MENU:
        case SHOW_MENU_OPTIONS:
        case SHOW_MENU_CONFIRM:
            ib_prepare_graph=true;
            break;

        case SHOW_MENU_KEYS:
            myGame->releaseKeys();
            MenuIndex=0;
            break;
            
        case SHOW_INSTRUCTIONS:
            prepareHistory();
            break;

        case SHOW_CREDITS:
            prepareCredits();
            break;
            
        case GOTO_GAME:
            if (myGame->gameConfig.isMusicOn())
            {
                myGame->ogg_music.stop();
            }
            myGame->setScene(new tsc_game());
            break;
    }
    ii_Estado = e;
    ii_NextState = next;
}

bool tsc_menu::doRun(void)
{
    int rkey;
    
    // Movimiento del Cursor de seleccion
    movex+=dirx;
    fgrama++;
    if (fgrama>17) fgrama=0;
    if (movex>10 || movex<0) dirx=-dirx;
    // --- FIN
    
    switch (ii_Estado)
    {
        case HIDE_MENU:
            fLight.doFade();
            break;
            
        case SHOW_SCREEN:
            
            if (!fBoxes.doFade())
            {
                gotoState(SHOW_MENU);
            }
            
            // Si se pulsa una tecla durante el fundido, pasamos del fundido.
            //
            if (myGame->inputController.isAnyHoldDown())
            {
                myGame->inputController.clearStatus();
                gotoState(SHOW_MENU);
            }
            
            break;
            
        case SHOW_INSTRUCTIONS:
        case SHOW_CREDITS:

            doScroll();

            // Si se pulsa una tecla durante el fundido, pasamos del fundido.
            //
            if (myGame->inputController.isAnyHoldDown())
            {
                myGame->inputController.clearStatus();
                gotoState(SHOW_MENU);
            }
            break;
            
        case SHOW_MENU_KEYS:

            // Obtenemos la configuracion de teclado
            //
            if (myGame->isAnyKeyPressed())
            {
                rkey = myGame->isAnyKeyPressed();
                
                // Miramos que la tecla pulsada, no esta ya utilizada
                //
                for (int i=0;i<MenuIndex;i++)
                {
                    if (myGame->gameConfig.getGameKey(i) == rkey) rkey=0;
                }

                if (rkey)
                {
                    myGame->gameConfig.setGameKey(MenuIndex++,rkey);
                    myGame->playSfx(snd_menuchange);
                }
                
                if (MenuIndex==9)
                {
                    gotoState(SHOW_MENU_OPTIONS);
                    MenuIndex=1;
                }
                
                myGame->releaseKeys();
            }            
            break;
            
        case SHOW_MENU:
        case SHOW_MENU_OPTIONS:
        case SHOW_MENU_CONFIRM:

            if (fLight.isFading()) fLight.doFade();

            if (myGame->inputController.isAnyHoldDown())
            {
                if (!fLight.isFading())
                {
                    if (myGame->inputController.isPressDown(myGame->K_KEY_UP))
                    {
                        MenuIndex--;
                        if (MenuIndex<0) MenuIndex = MenuItems-1;
                        myGame->playSfx(snd_menuchange);
                    }

                    if (myGame->inputController.isPressDown(myGame->K_KEY_DOWN))
                    {
                        MenuIndex++;
                        if (MenuIndex>MenuItems-1) MenuIndex=0;
                        myGame->playSfx(snd_menuchange);
                    }

                    if (myGame->inputController.isPressDown(myGame->K_KEY_SELECT0)||myGame->inputController.isPressDown(myGame->K_KEY_SELECT1))
                    {
                        myGame->playSfx(snd_menuselect);
                        switch (ii_Estado)
                        {
                            case SHOW_MENU:
                                switch (MenuIndex)
                                {
                                    case 0: // Jugar
                                        gotoState(HIDE_MENU,GOTO_GAME);
                                        break;
                                    case 1: // Instrucciones
                                        gotoState(HIDE_MENU,SHOW_INSTRUCTIONS);
                                        break;
                                    case 2: // Opciones
                                        gotoState(HIDE_MENU,SHOW_MENU_OPTIONS);
                                        break;
                                    case 3: // Creditos
                                        gotoState(HIDE_MENU,SHOW_CREDITS);
                                        break;
                                    case 4: // Salir
                                        gotoState(HIDE_MENU,SHOW_MENU_CONFIRM);
                                        break;
                                }
                                break;

                            case SHOW_MENU_CONFIRM:
                                switch (MenuIndex)
                                {
                                    case 0: // Salir
                                        myGame->endGame();
                                        break;
                                        
                                    case 1:
                                        gotoState(HIDE_MENU,SHOW_MENU);
                                        break;
                                }
                                break;

                            case SHOW_MENU_OPTIONS:
                                switch (MenuIndex)
                                {
                                    case 0: // Idioma
                                        myGame->gameConfig.setLanguageIndex(myGame->gameConfig.getLanguageIndex()+1);
                                        myGame->setupLanguage("spirits",myGame->gameConfig.getLanguageID());
                                        myGame->gameConfig.saveConfigFile();
                                        ib_prepare_graph=true;
                                        doPrepareMenu(false);
                                        break;
                                    case 1: // Redefinir Teclado
                                        #if !defined(TARGET_GP2X) && !defined(TARGET_WII)
                                            gotoState(HIDE_MENU,SHOW_MENU_KEYS);
                                        #endif
                                        break;
                                    case 2: // Ventana/pantalla
                                        #if !defined(TARGET_GP2X) && !defined(TARGET_WII)
                                            myGame->setFullScreen(!myGame->isFullScreen());
                                            myGame->gameConfig.saveConfigFile();
                                            ib_prepare_graph=true;
                                            doPrepareMenu(false); // Para que se repinte el menu
                                        #endif
                                        MenuIndex=2;
                                        break;
                                    case 5: // Volver
                                        myGame->gameConfig.saveConfigFile();
                                        gotoState(HIDE_MENU,SHOW_MENU);
                                        break;
                                }
                                break;
                        }
                    }
                        
                    // Subir y Bajar volumen
                    //
                    if (ii_Estado == SHOW_MENU_OPTIONS && (
                        myGame->inputController.isHoldDown(myGame->K_KEY_LEFT) || myGame->inputController.isHoldDown(myGame->K_KEY_RIGHT) ) )
                    {
                        switch (MenuIndex)
                        {
                            case 3: // Musica
                                changeVolume(true);
                                break;
                            case 4: // Sonido
                                changeVolume(false);
                                break;
                        }
                    }
                }
            }
    }
    
    // Devolvemos true, para que no se ejecute el comportamiento por defecto
    //
    return true;
}

void tsc_menu::changeVolume(bool music)
{
    bool done=false;
    int vol = (music)?myGame->gameConfig.getMusicVolume():myGame->gameConfig.getSfxVolume();
    int maxvol = (music)?DFLT_MUSIC_VOL:DFLT_SFX_VOL;
    
    if (myGame->inputController.isPressDown(myGame->K_KEY_LEFT))
    {
        if (vol>0)
        {
            vol-=10;
            if (vol<0) vol=0;
            done=true;
        }
    }
    else if (myGame->inputController.isPressDown(myGame->K_KEY_RIGHT))
    {
        if (vol<maxvol)
        {
            vol+=10;
            if (vol>maxvol) vol=maxvol;
            done=true;
        }
    }
        
    // Si se ha modificado el volumen... hacemos magia..
    //
    if (done)
    {
        if (music)
        {
            myGame->gameConfig.setMusicVolume(vol);
            myGame->setMusicVolume(myGame->ogg_music,maxvol);
        }
        else
        {
            myGame->gameConfig.setSfxVolume(vol);
            if (snd_abracadabra.isPlaying()) snd_abracadabra.stop();
            myGame->playSfx(snd_abracadabra);
        }
        
        int mindex = MenuIndex;
        ib_prepare_graph=true;
        doPrepareMenu(false); // Para que se repinte el menu
        MenuIndex=mindex;
    }
}

void tsc_menu::doPaint(TMABitmap& buff)
{
    buff.clear(0);
    
    switch (ii_Estado)
    {
        case SHOW_SCREEN:

            fBoxes.doFade(buff,*io_screen);
            break;

        case HIDE_MENU:
            buff.draw(io_screen,0,0);
            fLight.doFade(buff,*io_scrollbox);
            if (!fLight.isFading()) gotoState(ii_NextState);
            break;

        case SHOW_MENU:
        case SHOW_MENU_OPTIONS:
        case SHOW_MENU_CONFIRM:

            io_screen->disableAlpha();
            buff.draw(io_screen,0,0);
            if (ib_prepare_graph)
            {
                doPrepareMenu();
            }
            doPaintMenu(buff);
            break;
            
        case SHOW_MENU_KEYS:
            buff.draw(io_screen,0,0);
            doPaintKeys(buff);
            break;
            
        case SHOW_INSTRUCTIONS:
        case SHOW_CREDITS:
            buff.draw(io_screen,0,0);
            doScroll(buff);
            break;
    }
}

void tsc_menu::doPrepareMenu(bool fade)
{
    int ypos=150,w0,w1;
    const char *txt;
    char buff[100];
    
    if (!ib_prepare_graph) return;
    ib_prepare_graph=false;

    // Copiamos el trozo de pantalla original
    //
    io_scrollbox->draw(io_screen,0,ypos,640,250,0,0);

    // Preparamos el efecto de FADE
    //
    switch (ii_Estado)
    {
        case SHOW_MENU_CONFIRM:

            if (fade) fLight.doInitFade1(TMAFader::FADE_IN,640,130,VEL_FADE_LIGHT,0,40,0,ypos+40);

            MenuItems=2;
            MenuIndex=0;

            drawBicolorFont(*io_scrollbox,_("Seguro que desea salir?"),-1,40);

            MenuWidth[0][0]=drawBicolorFont(*io_scrollbox,_("Si, gracias"),-1,80);
            MenuWidth[0][1]=80;

            MenuWidth[1][0]=drawBicolorFont(*io_scrollbox,_("No, por favor"),-1,120);
            MenuWidth[1][1]=120;
            break;

        case SHOW_MENU:

            if (fade) fLight.doInitFade1(TMAFader::FADE_IN,640,210,VEL_FADE_LIGHT,0,20,0,ypos+20);

            MenuItems=5;
            MenuIndex=0;
            
            MenuWidth[0][0]=drawBicolorFont(*io_scrollbox,_("Jugar"),-1,20);
            MenuWidth[0][1]=20;
            
            MenuWidth[1][0]=drawBicolorFont(*io_scrollbox,_("Instrucciones"),-1,60);
            MenuWidth[1][1]=60;

            MenuWidth[2][0]=drawBicolorFont(*io_scrollbox,_("Opciones"),-1,100);
            MenuWidth[2][1]=100;

            MenuWidth[3][0]=drawBicolorFont(*io_scrollbox,_("Creditos"),-1,140);
            MenuWidth[3][1]=140;

            MenuWidth[4][0]=drawBicolorFont(*io_scrollbox,_("Salir"),-1,180);
            MenuWidth[4][1]=180;
            break;

        case SHOW_MENU_OPTIONS:

            if (fade) fLight.doInitFade1(TMAFader::FADE_IN,640,250,VEL_FADE_LIGHT,0,0,0,ypos);

            MenuItems=6;
            MenuIndex=0;

            sprintf(buff,"%s: %s",_("Cambiar Idioma"),myGame->gameConfig.getLanguageName());

            MenuWidth[0][0]=drawBicolorFont(*io_scrollbox,buff,-1,0);
            MenuWidth[0][1]=0;

            MenuWidth[1][0]=drawBicolorFont(*io_scrollbox,_("Redefinir Teclado"),-1,40);
            MenuWidth[1][1]=40;

            if (myGame->gameConfig.isFullScreen())
                MenuWidth[2][0]=drawBicolorFont(*io_scrollbox,_("Modo Ventana"),-1,80);
            else
                MenuWidth[2][0]=drawBicolorFont(*io_scrollbox,_("Modo Pantalla"),-1,80);
            MenuWidth[2][1]=80;

            // Pintamos el indicador del volumen de la musica
            txt=_("Volumen Musica:");
            w0 = myGame->fontMenu.getTextWidth(txt);
            
            #ifdef SMALL_GAME
                w1 = w0 + 10 + io_volumegauge->getWidth()*2;
            #else
                w1 = w0 + 10 + io_volumegauge->getWidth();
            #endif
            
            drawBicolorFont(*io_scrollbox,txt,(640 - w1)/2,120);
            MenuWidth[3][0] = w1;
            MenuWidth[3][1] = 120;
            w1 = (640 - w1)/2 + w0 + 10;
            io_scrollbox->draw(io_volumegauge,w1,130);

            // Calculamos el porcentaje de volumen a mostrar
            w0 = (myGame->gameConfig.getMusicVolume() * io_volumeindicator->getWidth())/DFLT_MUSIC_VOL;            
            #ifdef SMALL_GAME
                io_scrollbox->draw(io_volumeindicator,0,0,w0*2,io_volumeindicator->getHeight()*2,w1+4,134);
            #else
                io_scrollbox->draw(io_volumeindicator,0,0,w0,io_volumeindicator->getHeight(),w1+4,134);
            #endif
            
            // Pintamos el indicador del volumen del sonido
            txt=_("Volumen Sonido:");
            w0 = myGame->fontMenu.getTextWidth(txt);

            #ifdef SMALL_GAME
                w1 = w0 + 10 + io_volumegauge->getWidth()*2;
            #else
                w1 = w0 + 10 + io_volumegauge->getWidth();
            #endif

            drawBicolorFont(*io_scrollbox,txt,(640 - w1)/2,160);
            MenuWidth[4][0] = w1;
            MenuWidth[4][1] = 160;
            w1 = (640 - w1)/2 + w0 + 10;
            io_scrollbox->draw(io_volumegauge,w1,170);

            // Calculamos el porcentaje de volumen a mostrar
            w0 = (myGame->gameConfig.getSfxVolume() * io_volumeindicator->getWidth())/DFLT_SFX_VOL;
            #ifdef SMALL_GAME
                io_scrollbox->draw(io_volumeindicator,0,0,w0*2,io_volumeindicator->getHeight()*2,w1+4,174);
            #else
                io_scrollbox->draw(io_volumeindicator,0,0,w0,io_volumeindicator->getHeight(),w1+4,174);
            #endif

            MenuWidth[5][0]=drawBicolorFont(*io_scrollbox,_("Volver"),-1,200);
            MenuWidth[5][1]=200;
            break;
    }
}

void tsc_menu::doPaintKeys(TMABitmap& buff)
{
    const char *txt=NULL;
    
    if (fLight.isFading())
    {
        fLight.doFade(buff,*io_scrollbox);
        return;
    }

    switch (MenuIndex)
    {
        case 0:
            txt = _("Pulse: Izquierda");
            break;
        case 1:
            txt = _("Pulse: Derecha");
            break;
        case 2:
            txt = _("Pulse: Arriba");
            break;
        case 3:
            txt = _("Pulse: Abajo");
            break;
        case 4:
            txt = _("Pulse: Disparar");
            break;
        case 5:
            txt = _("Pulse: Coger");
            break;
        case 6:
            txt = _("Pulse: Localizar");
            break;
        case 7:
            txt = _("Pulse: Pausa");
            break;
        case 8:
            txt = _("Pulse: Abortar");
            break;
    }
    drawBicolorFont(buff,txt,-1,200);
}

void tsc_menu::doPaintMenu(TMABitmap& buff)
{    
    int ypos=150;
    
    if (fLight.isFading())
    {
        fLight.doFade(buff,*io_scrollbox);
    }
    else
    {
        buff.draw(io_scrollbox,0,ypos);
        // Gestionamos el efecto de seleccion sobre el menu
        //
        for (int i=0;i<MenuItems;i++)
        {
            if (i==MenuIndex)
            {
                #ifdef SMALL_GAME
                    PMABitmap lsel = TMABitmap::create(io_selector,(fgrama/3)*30,0,30,30);
                #else
                    PMABitmap lsel = TMABitmap::create(io_selector,(fgrama/3)*60,0,60,60);
                #endif
                buff.draw(lsel,320 - (MenuWidth[i][0]/2) - 60 - movex,ypos+MenuWidth[i][1]);
                buff.draw(lsel,320 + (MenuWidth[i][0]/2) +10+ movex,ypos+MenuWidth[i][1]);
                delete lsel;
            }
        }
    }
}

void tsc_menu::doScroll(TMABitmap& buff)
{
    int ypos=150;

    io_scrollbox->draw(&buff,0,ypos,640,250,0,0);

    for (int i=0;i<=StrScroll.last_index;i++)
    {
        int cy = (250 - StrScroll.offsety)+(i*40);
        
        // Pintamos el texto si esta en el recuadro
        if ( (i<StrScroll.last_index) && (cy+50>0) && (cy<250) )
        {
            drawBicolorFont(*io_scrollbox,StrScroll.txt[i],-1,cy);
        }
    }
    
    // Hacemos Blend de lineas superiores e inferiores
    // para simular un fundido
    //
    int alpha=1;
    for (int i=0;i<40;i++)
    {
        io_scrollbox->setAlpha(alpha); alpha+=6;
        #ifdef SMALL_GAME
            buff.draw(io_scrollbox,0,i,640,2,0,ypos+i);
            buff.draw(io_scrollbox,0,249-i,640,2,0,ypos+249-i);
        #else
            buff.draw(io_scrollbox,0,i,640,1,0,ypos+i);
            buff.draw(io_scrollbox,0,249-i,640,1,0,ypos+249-i);
        #endif
    }
    io_scrollbox->disableAlpha();
    buff.draw(io_scrollbox,0,40,640,170,0,ypos+40);
}

void tsc_menu::doScroll()
{
    StrScroll.offsety++;
    if (StrScroll.offsety>(StrScroll.last_index+6)*40)
    {
        gotoState(SHOW_MENU);
    }
}

int tsc_menu::drawBicolorFont(TMABitmap& buff,const char *txt,int x,int y)
{
    int width = myGame->fontMenu.getTextWidth(txt);
    #ifdef SMALL_GAME
        int ax = (x==-1)?(buff.getWidth()*2 - width)/2:x;
    #else
        int ax = (x==-1)?(buff.getWidth() - width)/2:x;
    #endif
    int ay = myGame->fontMenu.getFontHeight();    
    
    // Sombras
    myGame->fontMenu.drawText(buff,txt,ax+6,y+6,MAKERGBCOLOR(0,0,0),true);
    myGame->fontMenu.drawText(buff,txt,ax+2,y+2,MAKERGBCOLOR(20,10,0),true);

    // Mitad Inferior
    myGame->fontMenu.drawText(buff,txt,ax,y,MAKERGBCOLOR(200,220,250),true);

    // Mitad Superior
    int rx,ry,rx1,ry1;
    buff.getClipRect(rx,ry,rx1,ry1);
    #ifdef SMALL_GAME
        buff.setClipRect(rx,y/2,rx1,ay/4+1);
    #else
        buff.setClipRect(rx,y,rx1,ay/2+2);
    #endif
    myGame->fontMenu.drawText(buff,txt,ax,y,
        (ii_Estado==SHOW_MENU_KEYS||ii_Estado==SHOW_CREDITS||ii_Estado==SHOW_INSTRUCTIONS)?MAKERGBCOLOR(230,220,100):MAKERGBCOLOR(100,220,250),true);
    buff.setClipRect(rx,ry,rx1,ry1);
    
    return width;
}

void tsc_menu::freeScrollText(void)
{
    for (int i=0;i<MAX_SCROLL_TEXT;i++)
    if (StrScroll.txt[i]!=NULL)
    {
        delete StrScroll.txt[i];
        StrScroll.txt[i]=NULL;
    }
}

void tsc_menu::prepareHistory(void)
{
    int idx=0;
    freeScrollText();
    StrScroll.offsety=0;
    StrScroll.first_index=0;
    StrScroll.txt[idx++]=strdup(_("Corren tiempos dificiles para el gremio"));
    StrScroll.txt[idx++]=strdup(_("de los magos y los eventos acaecidos en"));
    StrScroll.txt[idx++]=strdup(_("los ultimos meses han complicado aun"));
    StrScroll.txt[idx++]=strdup(_("mas la situacion."));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("El Gran Consorcio de los Magos, se"));
    StrScroll.txt[idx++]=strdup(_("encuentra en peligro debido a Aquel Que"));
    StrScroll.txt[idx++]=strdup(_("No Debe Nombrarse, el cual ha regresado"));
    StrScroll.txt[idx++]=strdup(_("tras largos anos de paz y tranquilidad,"));
    StrScroll.txt[idx++]=strdup(_("con el objetivo de reinar sobre el mundo"));
    StrScroll.txt[idx++]=strdup(_("de los magos, para posteriormente"));
    StrScroll.txt[idx++]=strdup(_("conquistar el mundo de los no magos."));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("El Gran Consorcio de los Magos, esta"));
    StrScroll.txt[idx++]=strdup(_("gobernado por una bruja llamada Casandra,"));
    StrScroll.txt[idx++]=strdup(_("la cual ha sido hechizada por Aquel Que"));
    StrScroll.txt[idx++]=strdup(_("No Debe Nombrarse - a pesar del gran"));
    StrScroll.txt[idx++]=strdup(_("poder que ella tiene -, quedando convertida"));
    StrScroll.txt[idx++]=strdup(_("en una pequena e indefensa doncella"));
    StrScroll.txt[idx++]=strdup(_("totalmente desprovista de medios para"));
    StrScroll.txt[idx++]=strdup(_("defender su vida."));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Worf, el leal guardian de Casandra,"));
    StrScroll.txt[idx++]=strdup(_("tambien ha sido hechizado y convertido"));
    StrScroll.txt[idx++]=strdup(_("en una armadura, que sin voluntad propia"));
    StrScroll.txt[idx++]=strdup(_("deambula junto con Casandra, por un"));
    StrScroll.txt[idx++]=strdup(_("castillo donde han sido encerrados y"));
    StrScroll.txt[idx++]=strdup(_("en el que habita una legion de fantasmas,"));
    StrScroll.txt[idx++]=strdup(_("monstruos deformes, diablillos arqueros,"));
    StrScroll.txt[idx++]=strdup(_("hombres lobo y otro tipo de alimanas,"));
    StrScroll.txt[idx++]=strdup(_("seres despreciables e individuos de mal"));
    StrScroll.txt[idx++]=strdup(_("vivir y peor reputacion, que tienen como"));
    StrScroll.txt[idx++]=strdup(_("objetivo acabar con ellos."));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Aquel Que No Debe Nombrarse, se ha"));
    StrScroll.txt[idx++]=strdup(_("transformado en aguila y vuela por el"));
    StrScroll.txt[idx++]=strdup(_("castillo observando los movimientos de sus"));
    StrScroll.txt[idx++]=strdup(_("secuaces y sus torturados prisioneros."));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Los consejeros del Gran Consorcio,"));
    StrScroll.txt[idx++]=strdup(_("reunidos en un Gabinete de Crisis han"));
    StrScroll.txt[idx++]=strdup(_("decidido encargarte la mision de rescatar"));
    StrScroll.txt[idx++]=strdup(_("a Casandra, Worf y restablecer el orden"));
    StrScroll.txt[idx++]=strdup(_("en el mundo de los magos."));
    StrScroll.txt[idx++]=strdup(_("Confian en ti, pues todos conocen las"));
    StrScroll.txt[idx++]=strdup(_("hazanas que se cuentan sobre tu valor,"));
    StrScroll.txt[idx++]=strdup(_("tu inteligencia y tu coraje."));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Tus objetivos dentro del castillo seran"));
    StrScroll.txt[idx++]=strdup(_("los siguientes:"));
    StrScroll.txt[idx++]=strdup(_("En primer lugar debes encontrar el ojo"));
    StrScroll.txt[idx++]=strdup(_("que todo lo ve, es decir, una bola de"));
    StrScroll.txt[idx++]=strdup(_("cristal con la cual podras predecir"));
    StrScroll.txt[idx++]=strdup(_("cuales son los movimientos que hacen"));
    StrScroll.txt[idx++]=strdup(_("tanto tus enemigos, como Casandra y Worf,"));
    StrScroll.txt[idx++]=strdup(_("ademas podras ver en que lugar se"));
    StrScroll.txt[idx++]=strdup(_("encuentran otros elementos que te seran"));
    StrScroll.txt[idx++]=strdup(_("necesarios para completar la mision,"));
    StrScroll.txt[idx++]=strdup(_("como la varita magica, con la cual"));
    StrScroll.txt[idx++]=strdup(_("podras desencantar a Casandra,"));
    StrScroll.txt[idx++]=strdup(_("y el libro de conjuros, con el que"));
    StrScroll.txt[idx++]=strdup(_("podras romper el hechizo de Worf."));
    StrScroll.txt[idx++]=strdup(_("Finalmente con estos dos objetos en tu"));
    StrScroll.txt[idx++]=strdup(_("poder y tras haber liberado a Casandra"));
    StrScroll.txt[idx++]=strdup(_("y Worf, debes encontrar a Aquel Que No"));
    StrScroll.txt[idx++]=strdup(_("Debe Nombrarse para derrotarlo, dando"));
    StrScroll.txt[idx++]=strdup(_("asi por finalizada tu mision y"));
    StrScroll.txt[idx++]=strdup(_("devolviendo la paz al mundo de los magos."));
    StrScroll.last_index=idx;
}

void tsc_menu::prepareCredits(void)
{
    int idx=0;
    freeScrollText();
    StrScroll.offsety=0;
    StrScroll.first_index=0;
    StrScroll.txt[idx++]=strdup(_("Spirits Remake"));
    StrScroll.txt[idx++]=strdup("************************************");
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Desarrollado por:"));
    StrScroll.txt[idx++]=strdup(_("M.A Software 2007"));
    StrScroll.txt[idx++]=strdup(_("www.masoftware.es"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Equipo de desarrollo"));
    StrScroll.txt[idx++]=strdup("************************************");
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Analisis del Juego:"));
    StrScroll.txt[idx++]=strdup(_("Carlos Herrero"));
    StrScroll.txt[idx++]=strdup(_("M.Angel Jimenez"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Musica: Carlos Herrero"));
    StrScroll.txt[idx++]=strdup(_("Sonidos: M.Angel Jimenez"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Grafista Principal:"));
    StrScroll.txt[idx++]=strdup(_("Ricardo Oyon"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Retoques Graficos:"));
    StrScroll.txt[idx++]=strdup(_("Traperic y M.Angel Jimenez"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Programacion Principal:"));
    StrScroll.txt[idx++]=strdup(_("M.Angel Jimenez"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Apoyo de programacion y"));
    StrScroll.txt[idx++]=strdup(_("Portabilidad a Linux y GP2X"));
    StrScroll.txt[idx++]=strdup(_("Traperic"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("*** TRADUCTORES ***"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Italiano:"));
    StrScroll.txt[idx++]=strdup(_("Ingles:"));
    StrScroll.txt[idx++]=strdup(_("Chiquitistani:"));
    //StrScroll.txt[idx++]=strdup(_("Frances:"));
    //StrScroll.txt[idx++]=strdup(_("Aleman:"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Desarrollado originalmente por"));
    StrScroll.txt[idx++]=strdup(_("Topo Software en 1987"));
    StrScroll.txt[idx++]=strdup(_("Para las plataformas:"));
    StrScroll.txt[idx++]=strdup(_("SPECTRUM, AMSTRAD y MSX."));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Equipo de desarrollo original:"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.txt[idx++]=strdup(_("Programa y Musica: Alejandro Andre"));
    StrScroll.txt[idx++]=strdup(_("Graficos: Miguel Blanco Viu"));
    StrScroll.txt[idx++]=strdup(_("Colaboraciones:"));
    StrScroll.txt[idx++]=strdup(_("Fernando Diaz y Javier Cano"));
    StrScroll.txt[idx++]=strdup(_("Version AMSTRAD: Emilio Martinez"));
    StrScroll.txt[idx++]=strdup(_("Version MSX: Carlos Arias"));
    StrScroll.txt[idx++]=strdup("");
    StrScroll.last_index=idx;
}

#endif
