#ifndef _SPIRITS_CPP_
#define _SPIRITS_CPP_

#include <string.h>
#include "spirits.hpp"
#include "scenes/scenes.hpp"

// **************************************
// Definicin de mtodos de clase TMAGame
// **************************************
//
REGISTER_CLASS_ID(TSpiritsGame);
Path    TSpiritsGame::gamePath;
Log     TSpiritsGame::gameLog;
Config  TSpiritsGame::gameConfig;
TMAMsgQueue TSpiritsGame::msgQueue;

// Definicion de conexiones de pantallas
//
mapInfo TSpiritsGame::gameMapInfo[88] =
{
    {-1,-1,-1,-1}, // Pantalla 0: No existe
    {-1,-1, 2,-1}, // Pantalla 1
    { 1,-1, 3,11}, // Pantalla 2
    { 2,-1, 4,-1}, // Pantalla 3
    { 3,-1, 5, 9}, // Pantalla 4
    { 4,-1,-1, 6}, // Pantalla 5
    { 9, 5,-1, 7}, // Pantalla 6
    {23, 6,-1, 8}, // Pantalla 7
    {25, 7,-1,-1}, // Pantalla 8
    {10, 4, 6,-1}, // Pantalla 9
    {11,-1, 9,24}, // Pantalla 10
    {12, 2,10,-1}, // Pantalla 11
    {-1,-1,11,13}, // Pantalla 12
    {-1,12,14,-1}, // Pantalla 13
    {13,-1,24,15}, // Pantalla 14
    {17,14,16,19}, // Pantalla 15
    {15,24,25,18}, // Pantalla 16
    {-1,-1,15,-1}, // Pantalla 17
    {19,16,21,-1}, // Pantalla 18
    {20,15,18,-1}, // Pantalla 19
    {-1,-1,19,78}, // Pantalla 20
    {18,-1,22,-1}, // Pantalla 21
    {21,-1,26,-1}, // Pantalla 22
    {24,-1, 7,-1}, // Pantalla 23
    {14,10,23,16}, // Pantalla 24
    {16,-1, 8,-1}, // Pantalla 25
    {22,-1,27,-1}, // Pantalla 26
    {26,-1,28,-1}, // Pantalla 27
    {27,-1,29,-1}, // Pantalla 28
    {28,-1,30,-1}, // Pantalla 29
    {29,-1,31,37}, // Pantalla 30
    {30,36,32,39}, // Pantalla 31
    {31,35,33,-1}, // Pantalla 32
    {32,34,-1,41}, // Pantalla 33
    {35,81,-1,33}, // Pantalla 34
    {36,-1,34,32}, // Pantalla 35
    {85,83,35,31}, // Pantalla 36
    {38,30,39,-1}, // Pantalla 37
    {52,-1,37,51}, // Pantalla 38
    {37,31,40,48}, // Pantalla 39
    {39,-1,41,-1}, // Pantalla 40
    {40,33,-1,42}, // Pantalla 41
    {49,41,-1,43}, // Pantalla 42
    {44,42,-1,75}, // Pantalla 43
    {45,-1,43,74}, // Pantalla 44
    {46,48,44,73}, // Pantalla 45
    {50,47,45,-1}, // Pantalla 46
    {51,-1,48,46}, // Pantalla 47
    {47,39,49,45}, // Pantalla 48
    {48,-1,42,-1}, // Pantalla 49
    {-1,51,46,67}, // Pantalla 50
    {-1,38,47,50}, // Pantalla 51
    {53,-1,38,-1}, // Pantalla 52
    {54,-1,52,-1}, // Pantalla 53
    {64,-1,53,-1}, // Pantalla 54
    {80,79,61,56}, // Pantalla 55
    {57,55,58,-1}, // Pantalla 56
    {-1,-1,56,-1}, // Pantalla 57
    {56,-1,59,-1}, // Pantalla 58
    {58,-1,60,-1}, // Pantalla 59
    {59,-1,-1,-1}, // Pantalla 60
    {55,66,62,-1}, // Pantalla 61
    {61,-1,63,-1}, // Pantalla 62
    {62,64,-1,-1}, // Pantalla 63
    {65,-1,54,63}, // Pantalla 64
    {66,-1,64,-1}, // Pantalla 65
    {79,-1,65,61}, // Pantalla 66
    {-1,50,68,69}, // Pantalla 67
    {67,-1,73,70}, // Pantalla 68
    {-1,67,70,-1}, // Pantalla 69
    {69,68,71,72}, // Pantalla 70
    {70,73,77,-1}, // Pantalla 71
    {-1,70,-1,-1}, // Pantalla 72
    {68,45,74,71}, // Pantalla 73
    {73,44,75,77}, // Pantalla 74
    {74,43,-1,76}, // Pantalla 75
    {77,75,-1,-1}, // Pantalla 76
    {71,74,76,-1}, // Pantalla 77
    {-1,20,79,80}, // Pantalla 78
    {78,-1,66,55}, // Pantalla 79
    {-1,78,55,-1}, // Pantalla 80
    {82,-1,-1,34}, // Pantalla 81
    {83,-1,81,-1}, // Pantalla 82
    {84,-1,82,36}, // Pantalla 83
    {87,-1,83,85}, // Pantalla 84
    {86,84,36,-1}, // Pantalla 85
    {-1,87,85,-1}, // Pantalla 86
    {-1,-1,84,86}, // Pantalla 87
};

// --------- IMPLEMENTACION DE DESTRUCTOR
//
TSpiritsGame::~TSpiritsGame()
{
    cleanup();
}

void TSpiritsGame::cleanup(void)
{
}

#ifdef TARGET_WII

    #define SENSIBILITY 20 

    void TSpiritsGame::updateInputController()
    {
	  struct expansion_t stick;
	  bool sleft=false,sright=false,sup=false,sdown=false,nun_C=false,nun_Z=false;
	  
	  WPAD_ScanPads();
	  WPAD_Expansion(WPAD_CHAN_0, &stick);
 
	  u16 buttonsHeld = WPAD_ButtonsHeld(0);
	  u16 buttonsDown = WPAD_ButtonsDown(0);
		
	  if(stick.type == WPAD_EXP_NUNCHUK)
	  {
		float diffx = stick.nunchuk.js.pos.x - stick.nunchuk.js.center.x; 
		float diffy = stick.nunchuk.js.pos.y - stick.nunchuk.js.center.y; 
		
        if(fabs(diffx)>SENSIBILITY)  
        { 
		  sright = (stick.nunchuk.js.pos.x > stick.nunchuk.js.center.x);
		  sleft = (stick.nunchuk.js.pos.x < stick.nunchuk.js.center.x);
		} 
    
        if(fabs(diffy)>SENSIBILITY)  
        { 
		  sup = (stick.nunchuk.js.pos.y > stick.nunchuk.js.center.y);
		  sdown = (stick.nunchuk.js.pos.y < stick.nunchuk.js.center.y);
		} 

		nun_C = stick.nunchuk.btns_held & NUNCHUK_BUTTON_C;
		nun_Z = stick.nunchuk.btns_held & NUNCHUK_BUTTON_Z;
		
		inputController.setStatus(K_LEFT, sleft ||  (buttonsHeld & WPAD_BUTTON_LEFT));
		inputController.setStatus(K_KEY_LEFT, sleft ||  (buttonsHeld & WPAD_BUTTON_LEFT));
		
		inputController.setStatus(K_RIGHT,sright || (buttonsHeld & WPAD_BUTTON_RIGHT));
		inputController.setStatus(K_KEY_RIGHT,sright || (buttonsHeld & WPAD_BUTTON_RIGHT));
		
		inputController.setStatus(K_UP,   sup ||    (buttonsHeld & WPAD_BUTTON_UP));
		inputController.setStatus(K_KEY_UP,   sup ||    (buttonsHeld & WPAD_BUTTON_UP));
		
		inputController.setStatus(K_DOWN, sdown ||  (buttonsHeld & WPAD_BUTTON_DOWN));
		inputController.setStatus(K_KEY_DOWN, sdown ||  (buttonsHeld & WPAD_BUTTON_DOWN));
		
		inputController.setStatus(K_SHOOT, nun_C || nun_Z || (buttonsDown & WPAD_BUTTON_B));
	  }
	
      inputController.setStatus(K_TAKE,(buttonsDown & WPAD_BUTTON_A));
	  inputController.setStatus(K_KEY_SELECT0,(buttonsDown & WPAD_BUTTON_A));
	  inputController.setStatus(K_KEY_SELECT1,(buttonsDown & WPAD_BUTTON_A));

      inputController.setStatus(K_ABORT,(buttonsDown & WPAD_BUTTON_HOME));
	  inputController.setStatus(K_KEY_ESCAPE,(buttonsDown & WPAD_BUTTON_HOME));

      inputController.setStatus(K_NEXT,(buttonsDown & WPAD_BUTTON_PLUS));
      inputController.setStatus(K_PAUSE,(buttonsDown & WPAD_BUTTON_MINUS));	  
    }

#elif defined(TARGET_GP2X)

	void TSpiritsGame::updateInputController(SDL_Event& event)
	{        
        bool lb_pressed = false;
        int lbutton = -1;
        
        switch (event.type)
		{
			case SDL_JOYBUTTONDOWN:
                lb_pressed = true;
				lbutton = event.jbutton.button;
				break;
						
			case SDL_JOYBUTTONUP:
				lb_pressed = false;
				lbutton = event.jbutton.button;
				break;
        }
        
		switch (lbutton)
		{
			case GP2X_BUTTON_L:
               	break;
				
			case GP2X_BUTTON_R:
				break;
				
			case GP2X_BUTTON_UPLEFT:
                inputController.setStatus(K_UP,lb_pressed);
                inputController.setStatus(K_LEFT,lb_pressed);
				break;
				
            case GP2X_BUTTON_UPRIGHT:
				inputController.setStatus(K_UP,lb_pressed);
                inputController.setStatus(K_RIGHT,lb_pressed);
				break;				
				
			case GP2X_BUTTON_UP:
			    inputController.setStatus(K_UP,lb_pressed);
			    inputController.setStatus(K_KEY_UP,lb_pressed);
				break;
				
			case GP2X_BUTTON_LEFT:
                inputController.setStatus(K_LEFT,lb_pressed);
                inputController.setStatus(K_KEY_LEFT,lb_pressed);
				break;
				
			case GP2X_BUTTON_RIGHT:
				inputController.setStatus(K_RIGHT,lb_pressed);
				inputController.setStatus(K_KEY_RIGHT,lb_pressed);
                break;
				
			case GP2X_BUTTON_DOWNLEFT:
	            inputController.setStatus(K_DOWN,lb_pressed);
                inputController.setStatus(K_LEFT,lb_pressed);
                break;
                
			case GP2X_BUTTON_DOWNRIGHT:
	            inputController.setStatus(K_DOWN,lb_pressed);
                inputController.setStatus(K_RIGHT,lb_pressed);
                break;
				
			case GP2X_BUTTON_DOWN:
	            inputController.setStatus(K_DOWN,lb_pressed);
	            inputController.setStatus(K_KEY_DOWN,lb_pressed);
                break;
				
			case GP2X_BUTTON_A:
                inputController.setStatus(K_SHOOT,lb_pressed);
				break;

			case GP2X_BUTTON_B:
                inputController.setStatus(K_NEXT,lb_pressed);
				break;

			case GP2X_BUTTON_Y:
                inputController.setStatus(K_TAKE,lb_pressed);
				break;

			case GP2X_BUTTON_X:
                inputController.setStatus(K_PAUSE,lb_pressed);
				break;
								
			case GP2X_BUTTON_VOLDOWN:
				break;
				
			case GP2X_BUTTON_VOLUP:
				break;
				
			case GP2X_BUTTON_SELECT:
                inputController.setStatus(K_KEY_SELECT0,lb_pressed);
                inputController.setStatus(K_KEY_SELECT1,lb_pressed);
				break;
				
			case GP2X_BUTTON_HOME:
                inputController.setStatus(K_ABORT,lb_pressed);
                inputController.setStatus(K_KEY_ESCAPE,lb_pressed);
				break;
		}
	}

#else

    void TSpiritsGame::updateInputController()
    {
        inputController.setStatus(K_LEFT ,KeyState[gameConfig.getGameKey(K_LEFT)]);
        inputController.setStatus(K_RIGHT,KeyState[gameConfig.getGameKey(K_RIGHT)]);
        inputController.setStatus(K_UP   ,KeyState[gameConfig.getGameKey(K_UP)]);
        inputController.setStatus(K_DOWN ,KeyState[gameConfig.getGameKey(K_DOWN)]);
        inputController.setStatus(K_SHOOT,KeyState[gameConfig.getGameKey(K_SHOOT)]);
        inputController.setStatus(K_TAKE ,KeyState[gameConfig.getGameKey(K_TAKE)]);
        inputController.setStatus(K_NEXT ,KeyState[gameConfig.getGameKey(K_NEXT)]);
        inputController.setStatus(K_ABORT,KeyState[gameConfig.getGameKey(K_ABORT)]);
        inputController.setStatus(K_PAUSE,KeyState[gameConfig.getGameKey(K_PAUSE)]);
    
        // --- Teclas de control que no son del juego
        //
        inputController.setStatus(K_KEY_LEFT   ,KeyState[gameConfig.getGameKey(K_KEY_LEFT)]);
        inputController.setStatus(K_KEY_RIGHT  ,KeyState[gameConfig.getGameKey(K_KEY_RIGHT)]);
        inputController.setStatus(K_KEY_UP     ,KeyState[gameConfig.getGameKey(K_KEY_UP)]);
        inputController.setStatus(K_KEY_DOWN   ,KeyState[gameConfig.getGameKey(K_KEY_DOWN)]);
        inputController.setStatus(K_KEY_SELECT0,KeyState[gameConfig.getGameKey(K_KEY_SELECT0)]);
        inputController.setStatus(K_KEY_SELECT1,KeyState[gameConfig.getGameKey(K_KEY_SELECT1)]);
        inputController.setStatus(K_KEY_ESCAPE ,KeyState[gameConfig.getGameKey(K_KEY_ESCAPE)]);
    };

#endif 

// --------- ACTUALIZA MODO GRAFICO
//
bool TSpiritsGame::updateGraphicsMode(bool fullscreen)
{
    #ifdef SMALL_GAME
        if (setGraphicsMode(320,240,16,fullscreen)>0)
    #else
        if (setGraphicsMode(640,480,16,fullscreen)>0)
    #endif
    {
        gameConfig.setFullScreen(fullscreen);
        return true;
    }
    return false;
}

// --------- INICIALIZACION DEL JUEGO
//
bool TSpiritsGame::init(void)
{ 
    if (!TMAGame::init()) return false;

    // Inicializaciones
    //
    lb_init=false; 
    inputController.setNumControls(K_KEY_MAX);
    gameConfig.setConfigDefaults();
    gameConfig.loadConfigFile();
    setupLanguage("spirits",gameConfig.getLanguageID());

    setWindowTitle("Spirits Remake: M.A Software 2007");
    if (!updateGraphicsMode(gameConfig.isFullScreen())) return false;
    setMouseVisible(false);
    setGameFPS(32);
    lb_init = true;
    
    // Inicializamos la semilla de numeros aleatorios con el TICK de reloj.
    //
    TMARandom::init();
    
    // Cargamos los tipos de letra globales
    //
    if (!fontMenu.load(gamePath.makePath(gamePath.getDataPrefix(),"font/myfont.ttf").text(),36)) return false;
    if (!fontGame.load(gamePath.makePath(gamePath.getDataPrefix(),"font/myfont.ttf").text(),21)) return false;
    
    // Limpiamos estado de teclado
    //
    releaseKeys();

    // Establecemos el escenario inicial
    //
    setScene(new tsc_logo());
    
    // --- PARA PROBAR EL FINAL DEL JUEGO
    /*
    if (isSoundAvailable())
    {
        ogg_music.load("data/music/spirits.ogg");
        ogg_music.play(true);
        setScene(new tsc_menu());
    }
    setScene(new tsc_end());
    */
    return true;
}


void TSpiritsGame::doRun(void)
{
    if (inputController.isPressDown(K_KEY_ESCAPE))
    {
        this->endGame();
        return;
    }
}

void TSpiritsGame::doPaint(TMABitmap& buff)
{
    buff.clear(0);
};


int TSpiritsGame::correctSfxVolume(int vol)
{
    int maxvol = gameConfig.getSfxVolume();
    return (vol*maxvol)/DFLT_SFX_VOL;
}

void TSpiritsGame::playSfx(TMASound& s,bool loop,int vol,int pan)
{
    if (gameConfig.isSfxOn())
    {
        int lvol = correctSfxVolume(vol);
        s.play(loop,lvol,pan);
    }
}

int TSpiritsGame::correctMusicVolume(int vol)
{
    int maxvol = gameConfig.getMusicVolume();
    return (vol*maxvol)/DFLT_MUSIC_VOL;
}

void TSpiritsGame::playMusic(TMAOgg& o,bool loop,int vol)
{
    if (gameConfig.isMusicOn())
    {
        int lvol = correctMusicVolume(vol);
        o.play(loop,lvol);
    }
}

void TSpiritsGame::setMusicVolume(TMAOgg& o,int vol)
{
    int lvol = correctMusicVolume(vol);
    o.setVolume(lvol);
}

int MainProc(int argc, char** argv)
{
    TSpiritsGame miGame;
    TSpiritsGame::gamePath.init(argc,argv);
    
    if ((argc>1 && (strcmp(argv[1],"/log")==0)))
    {
        TSpiritsGame::gameLog.setLogFile(
            TSpiritsGame::gamePath.makePath(TSpiritsGame::gamePath.getCurrentDirectory(),"/spirits.log").text()
        );
    }
    else
    {
        TSpiritsGame::gameLog.setEnabled(false);
    }
    
    if (miGame.init())
    {
        miGame.run();
    }
    else
    {
        miGame.showAlert(_("No se pudo inicializar la aplicacion."));
    }

    return -1;    
}

#endif
