#ifndef _SPIRITS_H_
#define _SPIRITS_H_

#include "../masoftware/sdl/ma_classes.hpp"
#include "../masoftware/maps/mapas.hpp"
#include "../masoftware/tad/tmavector.cpp"
#include "../masoftware/util/tmamessage.hpp"
#include "../masoftware/util/tmarandom.hpp"
#include "misc/path.hpp"
#include "misc/log.hpp"
#include "misc/config.hpp"
#include <stdlib.h>

#define MSG_SPHERE_GOT      1
#define MSG_BOOK_GOT        2
#define MSG_MAGIC_GOT       3
#define MSG_CASANDRA_GOT    4
#define MSG_WORF_GOT        5
#define MSG_BAD_GOT         6
#define MSG_PUSH_HANDLE     7

#ifdef TARGET_WII
	
    #include <wiiuse/wpad.h>
	
#elif defined(TARGET_GP2X)
    
    #define GP2X_BUTTON_UP              (0)
	#define GP2X_BUTTON_DOWN            (4)
	#define GP2X_BUTTON_LEFT            (2)
	#define GP2X_BUTTON_RIGHT           (6)
	#define GP2X_BUTTON_UPLEFT          (1)
	#define GP2X_BUTTON_UPRIGHT         (7)
	#define GP2X_BUTTON_DOWNLEFT        (3)
	#define GP2X_BUTTON_DOWNRIGHT       (5)
	#define GP2X_BUTTON_CLICK           (18)
	#define GP2X_BUTTON_A               (12)
	#define GP2X_BUTTON_B               (13)
	#define GP2X_BUTTON_X               (15)
	#define GP2X_BUTTON_Y               (14)
	#define GP2X_BUTTON_L               (11)
	#define GP2X_BUTTON_R               (10)
	#define GP2X_BUTTON_HOME            (8)
	#define GP2X_BUTTON_SELECT          (9)
	#define GP2X_BUTTON_VOLUP           (16)
	#define GP2X_BUTTON_VOLDOWN         (17)
	
#endif

typedef struct
{
    int left,up,right,down;
} mapInfo;

class TSpriteLamp;
class TSpriteDoor;
class TSpriteHandle;
class TSpriteAnim;

typedef struct
{
    bool wall_open;
    bool visited;
    bool visited_eagle;
    bool visited_worf;
    bool visited_casandra;
    bool sound_torch;
    PMAScreenMap scr;
    TMAVector<TSpriteAnim>* lights;
    TMAVector<TSpriteHandle>* handles;
    TSpriteLamp *lamp;
    TSpriteDoor *door;
} roomInfo;

typedef struct
{
    int sphere,book,magical;
} objectInfo;

typedef struct
{
    // Sonidos de Enemigos
    //
    TMASound snd_wolf;      // Lobo
    TMASound snd_ghost;     // Fantasma
    TMASound snd_hump;      // Jorobado
    TMASound snd_cat;       // Gato
    TMASound snd_archier;   // Aparicion del arquero
    TMASound snd_arrow;     // Sonido de la flecha

    // Sonidos de Ambiente
    //
    TMASound snd_handle;    // Palanca
    TMASound snd_cricket;   // Grillo
    TMASound snd_kettle;    // Caldero
    TMASound snd_door;      // Apuertura de Puerta
    TMASound snd_fire;      // Fuego de antorchas
    TMASound snd_rockdown;  // Piedra que cae
    TMASound snd_lampdown;  // Lampara que cae
    TMASound snd_tictac;    // Sonido del Reloj
    TMASound snd_take;      // Sonido al tomar objeto

    // Sonidos del Personaje
    //
    TMASound snd_shoot;     // Disparo
    TMASound snd_jump;      // Salto
    TMASound snd_step;      // Paso
    TMASound snd_energy;    // Muro de Energia
    TMASound snd_die;       // Sonido muerte
    TMASound snd_stairs;    // Sonido en la escalera
    TMASound snd_hurt;      // Sonido de dolor al caer desde alto
    TMASound snd_abra;      // Sonido del AbraCadabra

} gameSound;


class TSpiritsGame:public TMAGame
{
    REGISTER_CLASS protected:

        bool lb_init;
        bool key_array[16];

    public:
        PUBLIC_CLASS_ID;
        
        // Constantes de las teclas
        //
        enum KEY_VALUE {
            K_LEFT      = 0,
            K_RIGHT     = 1,
            K_UP        = 2,
            K_DOWN      = 3,
            K_SHOOT     = 4,
            K_TAKE      = 5,
            K_NEXT      = 6,
            K_PAUSE     = 7,
            K_ABORT     = 8,
            
            K_KEY_LEFT = 9,
            K_KEY_RIGHT = 10,
            K_KEY_UP = 11,
            K_KEY_DOWN = 12,
            K_KEY_SELECT0 = 13,
            K_KEY_SELECT1 = 14,
            K_KEY_ESCAPE = 15,
            
            K_KEY_MAX = 16
        };

        // Rutas, Log y Configuracion
        //
        static Path     gamePath;
        static Log      gameLog;
        static Config   gameConfig;
        static mapInfo  gameMapInfo[88];
        static TMAMsgQueue msgQueue;

        // Tipos de Letra
        //
        TMATTFont fontMenu;
        TMATTFont fontGame;

        // Musica Global
        //
        TMAOgg ogg_music;

        // Destructor 
        //
        ~TSpiritsGame();
        
        // Mtodos
        //
        bool init(void);
        void cleanup(void);
        void doRun(void);
        void doPaint(TMABitmap&);

        // Actualizar Modo Grafico
        //
        bool updateGraphicsMode(bool fullscreen);
                
        // A partir del volumen teorico del sonido, devuelve el volumen
        // correcto segun la configuracin de sonido.
        //
        int correctSfxVolume(int vol);
        void playSfx(TMASound&,bool loop=false,int vol=255,int pan=127);

        int correctMusicVolume(int vol);
        void playMusic(TMAOgg&,bool loop=false,int vol=512);
        void setMusicVolume(TMAOgg&,int vol);
        
        #ifdef TARGET_GP2X
            void updateInputController(SDL_Event&);
        #else
            void updateInputController();
        #endif

};

typedef TSpiritsGame* PSpiritsGame;

#endif
