#ifndef _SPRITES_H_
#define _SPRITES_H_

#include "../spirits.hpp"

// -------------- Mascaras

#define MASK_EMPTY  0       // Nada
#define MASK_WALL   1       // Pared
#define MASK_FLOOR  2       // Suelo
#define MASK_DIE    3       // Muerte
#define MASK_HANDLE 4       // Palanca
#define MASK_STAIRS 5       // Escaleras
#define MASK_CLOSE  6       // Cerrado por palanca

// -------------- Prototipos
//
class TSpriteSpirits;
class TSpriteBrujo;
class TSpriteAmbulante;
class TSpriteEnemigo;
class TSpriteEnemigoAleatorio;
class TSpriteAnim;
class TSpriteDoor;
class TSpriteHandle;
class TSpriteObject;

typedef TSpriteSpirits*          PSpriteSpirits;
typedef TSpriteBrujo*            PSpriteBrujo;
typedef TSpriteAmbulante*        PSpriteAmbulante;
typedef TSpriteEnemigo*          PSpriteEnemigo;
typedef TSpriteEnemigoAleatorio* PSpriteEnemigoAleatorio;
typedef TSpriteAnim*             PSpriteAnim;
typedef TSpriteDoor*             PSpriteDoor;
typedef TSpriteHandle*           PSpriteHandle;
typedef TSpriteObject*           PSpriteObject;

// -------------- TSpriteSpirits
//
class TSpriteSpirits : public TMASprite
{
    REGISTER_CLASS protected:
    
    	static const int _TileAX = 20;
    	static const int _TileAY = 20;

        bool ib_Dying;
        int hotspotX,hotspotY,screenID,ii_flashtime;
        PMAScreenMap io_Screen;
        
        static roomInfo *rooms;
        static gameSound *sounds;
        static PMAImageBank bankShadow;
        static PSpriteBrujo target;
        
        PSpiritsGame myGame;

    public:
        
        PUBLIC_CLASS_ID;
        TSpriteSpirits(PMAImageBank);

        static void setRoomsInfo(roomInfo *rms,PMAImageBank sw) {rooms = rms; bankShadow = sw; }
        static void setTarget(PSpriteBrujo t) { target = t; }
        static void setSounds(gameSound *s) { sounds = s; }

        // Obtiene puntos de comprobacin
        // El HOTSPOT(X,Y), es el punto centrado en la base del sprite.
        //
        virtual int getHotspotMapX(void) { return hotspotX/_TileAX; }
        virtual int getHotspotMapY(void) { return hotspotY/_TileAY; }
        virtual int getHotspotX(void) { return hotspotX; }
        virtual int getHotspotY(void) { return hotspotY; }
        virtual void putAt(int x,int y);
        void setFlashTime(int);
        int  getFlashTime(void) { return ii_flashtime; }

        // --------------------- Comprobaciones para poder moverse
        
        virtual bool evalMask(int mask);
        virtual bool isStairs(int px,int py);
        bool isTop(int px,int py);
        bool isDown(int px,int py,int delta=0);
        bool canWalkRight(int px,int py);
        bool canWalkLeft(int px,int py);
        void correctPositionY(void) { hotspotY = ((getHotspotMapY()+1)*_TileAY)-1; }
        
        // ---------------------- Colisiones

        virtual bool collisionsWith(TSpriteSpirits& spr);
        virtual int *getKillSequence(int& length);
        bool kill(void);

        // ---------------------- Acceso a las pantallas
        
        virtual void setScreenID(int id);
        int  getScreenID(void) { return screenID; }
        
        bool roomChanged(int px,int py);
        void enterLeft(void);
        void enterRight(void);
        void enterUp(void);
        void enterDown(void);
        
        // ---------------------- Dibujo y Animacion del Sprite
        
        virtual void animate(void);
        virtual void drawShadow(TMABitmap&,TMAVector<TSpriteAnim>* lights);
        virtual void drawShadowed(TMABitmap&,int px);
        virtual bool draw(TMABitmap&);
};

// -------------- TSpriteBrujo
//
class TSpriteBrujo : public TSpriteSpirits
{
    REGISTER_CLASS protected:
    
        friend class TSpriteAnim;
        
        enum State { E_WALK,E_JUMP_UP,E_JUMP_DOWN,E_FALLING, E_STAIRS,E_DOWN,E_SHOOT};
        int ii_JumpCount,ii_JumpDir;
        int ii_lives,ii_energy,ii_inmunitytime,ii_falling;
        bool ib_book, ib_magic, ib_casandra, ib_worf, ib_bad;
        
        PSpriteAnim sprShoot;
        
        void setState(State e);

    public:        
        PUBLIC_CLASS_ID;        
        
        static int ia_AnimDying[1];
        static int ia_AnimWalk[6];
        static int ia_AnimJumpUp[1];
        static int ia_AnimJumpDown[1];
        static int ia_AnimGetDown[2];
        static int ia_AnimUpDown[2];
        static int ia_AnimShoot[3];
        static int ia_AnimDown[1];

        TSpriteBrujo(PMAImageBank bank,PMAImageBank bank1);
        ~TSpriteBrujo();

        int *getKillSequence(int& length);

        bool evalMask(int mask);
        bool doControll(void);
        void animate(void);
        
        void setScreenID(int id);
        
        int getLives() { return ii_lives; }
        int getEnerty() { return ii_energy; }
        void setEnergyDown(int how=1);
        void setLiveDown();
        void die(void) { ii_lives=0; ii_energy=0; }
        void moreLives() { ii_lives=3; }
        
        // Control de la colision
        //
        bool collisionsWith(TSpriteSpirits& spr);
        bool collisionMagicWith(TSpriteSpirits& spr);
        void invalidateMagic(void);
        
        bool draw(TMABitmap&);

        // Control de los elementos obtenidos
        //
        void getBook(void) { ib_book=true; }
        bool hasBook(void) { return ib_book; }
        
        void getMagic(void) { ib_magic=true; }
        bool hasMagic(void) { return ib_magic; }

        void getCasandra(void) { ib_casandra=true; }
        bool hasCasandra(void) { return ib_casandra; }

        void getWorf(void) { ib_worf=true; }
        bool hasWorf(void) { return ib_worf; }

        void getBad(void) { ib_bad=true; }
        bool hasBad(void) { return ib_bad; }
};

// -------------- TSpriteDeambulante
// -------------- Sprites Deambulantes: Casandra, Worf y Aguila
//
class TSpriteDeambulante : public TSpriteSpirits
{
    REGISTER_CLASS protected:

        enum State { E_WALK, E_STAIRS_UP, E_STAIRS_DOWN, E_FALLING, E_UP, E_UP1, E_JUMP, E_FLY,
                     E_FLY_UP, E_FLY_DOWN, E_LIBERING, E_LIBERED };

        int itype,posy_eagle,itimeforgot,icounter;
        bool ib_decidido,ib_sube_escalones,ib_decide_saltar,ib_got;
        State last_state;
        
        void setState(State e);
        bool doControllHuman(void);
        bool doControllEagle(void);

    public:
        PUBLIC_CLASS_ID;

        static int ia_AnimCasandra[4];
        static int ia_AnimCasandraWalk[4];
        static int ia_AnimCasandraUpDown[2];

        static int ia_AnimWorf[1];
        static int ia_AnimWorfWalk[4];
        static int ia_AnimWorfUpDown[2];

        static int ia_AnimEagle[4];
        static int ia_AnimHumo[14];

        enum Type { T_WORF, T_CASANDRA, T_EAGLE };

        TSpriteDeambulante(PMAImageBank bank,TSpriteDeambulante::Type t);
        void setScreenID(int id);
        bool canWalkRight(int px,int py);
        bool isDown(int px,int py);
        bool isStairs(int px,int py);

        bool canEagleFlyUp(int px,int py);
        bool canEagleFlyDown(int px,int py);
        bool canStairsUp(int px,int py);
        bool canStairsDown(int px,int py);

        void animate(void);
        
        void setGot(void) { ib_got = true; }
        bool isGot(void) { return ib_got; }
};


// -------------- TSpriteEnemigo
// -------------- Sprites malos: Fantasma, Hombre Lobo, Jorobado
//
class TSpriteEnemigo : public TSpriteSpirits
{
    REGISTER_CLASS protected:

        static int ia_AnimGhost[2];
        static int ia_AnimWolf[4];
        static int ia_AnimHump[4];
        static int ia_AnimHump1[4];

        enum State { E_WALK, E_FOLLOW_LEFT, E_FOLLOW_RIGHT, E_STOP };
        
        PMAImageBank ibank1;
        int screenID1,screenID2;
        int posx1,posx2,velocity,soundwait,soundcount;
        int waiting,ii_ice;
        bool ib_follow;
        State laststate;
        
        void setState(State e);
        bool doControll(void);
        void doSound(bool force);
        
        int iType;

    public:
        PUBLIC_CLASS_ID;

        enum Type { T_WOLF, T_GHOST, T_HUMP , T_HUMP1};

        TSpriteEnemigo(TSpriteEnemigo::Type t,PMAImageBank bank,PMAImageBank bank1);
        void setRoute(int ScreenID1,int px1,int ScreenID2,int px2,int posy,int vel);
        void animate(void);
        bool draw(TMABitmap&);
        void drawShadow(TMABitmap&,TMAVector<TSpriteAnim> *lights);
        bool isStopped(void) { return ii_Estado == E_STOP; }
};


// -------------- TSpriteEnemigoAleatorio
// -------------- Enemigos que aparecen eventualmente (gato y arquero)
//
class TSpriteEnemigoAleatorio : public TSpriteSpirits
{
    REGISTER_CLASS protected:

        enum State { S_NULL,ARC_IN,ARC_SHOOTING,ARC_SHOOT,ARC_OUT };

        static int ia_AnimCat[4];
        static int ia_AnimArc[16];
        static int ia_AnimArcF[16];

        int itype,waitcount,wait;
        int fade;
        PSpriteAnim sprArrow;

        void setState(State e);
        void doControllCat(void);
        void doControllArc(void);

    public:
        PUBLIC_CLASS_ID;

        enum Type { T_CAT, T_ARC };

        TSpriteEnemigoAleatorio(PMAImageBank,TSpriteEnemigoAleatorio::Type t);
        ~TSpriteEnemigoAleatorio();
        
        void init(int scrid);
        void animate(void);
        bool draw(TMABitmap&);
        void drawShadow(TMABitmap&,TMAVector<TSpriteAnim> *lights);
};


// -------------- TSpriteLamp
//
class TSpriteLamp : public TSpriteSpirits
{
    REGISTER_CLASS protected:

        enum State {E_WAITING, E_FALLING, E_FINISH};

        static int ia_AnimLamp[1];
        static int ia_AnimRock[1];
        int initx,inity,inittime,velocity,timewait,itype;

        void setState(State e);
        void doControll(void);

    public:
        PUBLIC_CLASS_ID;

        enum Type { T_LAMP, T_ROCK };

        TSpriteLamp(PMAImageBank,TSpriteLamp::Type t=T_LAMP);

        void reset(void);
        void putAt(int ppx,int ppy);
        void setInit(int ptime,int vel);
        void animate(void);
        bool isOnFloor(void) { return (ii_Estado==E_FINISH); }
        bool isRock(void) { return itype==T_ROCK; }
        bool draw(TMABitmap&);
        void drawShadow(TMABitmap& b,TMAVector<TSpriteAnim>* lights);

};

// -------------- TSpriteAnim
//
class TSpriteAnim : public TMASprite
{
    REGISTER_CLASS protected:

        static int ia_AnimLight[16];
        static int ia_AnimMagic[4];
        static int ia_AnimArrow[4];
        int iType;

    public:
        PUBLIC_CLASS_ID;
        
        enum Type {T_LIGHT,T_MAGIC,T_ARROW,T_OTHER};
        
        TSpriteAnim(PMAImageBank,TSpriteAnim::Type t);
        
        void animate(void);
        bool draw(TMABitmap&);
        void setSpeed(int r) { ii_Retardo=r; }
};

// -------------- TSpriteDoor
//
class TSpriteDoor : public TSpriteSpirits
{
    REGISTER_CLASS protected:

        enum State { S_CLOSE, S_OPENING, S_OPEN };
        int  size;
        bool ib_vertical;

    public:
        PUBLIC_CLASS_ID;
        
        TSpriteDoor(PMAImageBank,int imgindex,bool vertical=true);

        void animate(void);
        bool draw(TMABitmap&);
        bool isOpen(void);
        bool isVertical(void) { return ib_vertical; }
        void open(void);
        void putAt(int x,int y) { TMASprite::putAt(x,y); }
};

// -------------- TSpriteObject
//
class TSpriteObject : public TSpriteSpirits
{
    REGISTER_CLASS protected:

        static int ia_AnimSphere[4];
        static int ia_AnimBook[32];
        static int ia_AnimMagic[4];

        bool ib_got;
        int itype;
        
    public:
        PUBLIC_CLASS_ID;

        enum Type { T_SPHERE, T_BOOK, T_MAGIC };

        TSpriteObject(PMAImageBank,TSpriteObject::Type t);

        void animate(void);
        bool draw(TMABitmap&);
        void setGot(void) { ib_got = true; }
        bool isGot(void) { return ib_got; }
        void drawShadowed(TMABitmap&,int ppx);
        void drawOnPanel(TMABitmap&,int x,int y);
};

// -------------- TSpriteHandle
//
class TSpriteHandle : public TSpriteSpirits
{
    REGISTER_CLASS protected:

        enum State { S_NORMAL, S_MOVE, S_PUSHED };

    public:
        PUBLIC_CLASS_ID;

        TSpriteHandle(PMAImageBank);

        void animate(void);
        bool draw(TMABitmap&);
        bool isPushed(void);
        bool isPushedBy(PSpriteBrujo);
        void push(void);
        void putAt(int x,int y) { TMASprite::putAt(x,y); }
};

#endif
