#ifndef _TSPRITEANIM_CPP_
#define _TSPRITEANIM_CPP_

#include "sprites.hpp"
#include "../spirits.hpp"

#ifdef TARGET_GP2X
    #define ANIM_SPEED 16
#else
    #define ANIM_SPEED 8
#endif

REGISTER_CLASS_ID(TSpriteAnim);

int TSpriteAnim::ia_AnimLight[16]={2,1,0,1,2,1,0,1,2,1,0,1,2,1,0,1};
int TSpriteAnim::ia_AnimMagic[4]={28,29,30,31};
int TSpriteAnim::ia_AnimArrow[4]={24,25,26,27};

TSpriteAnim::TSpriteAnim(PMAImageBank bank,TSpriteAnim::Type t):TMASprite(bank)
{
    ii_Retardo=2;

    switch ((int)t)
    {
        case T_LIGHT:
            setAnimSequence(ia_AnimLight,16);
            ii_Frame = TMARandom::nextInt(16);
            break;

        case T_MAGIC:
            setAnimSequence(ia_AnimMagic,3);
            invalidate();
            break;

        case T_ARROW:
            setAnimSequence(ia_AnimArrow,4);
            invalidate();
            break;
    }
    animate();
    iType = (int)t;
}

void TSpriteAnim::animate(void)
{
    // Programando el comportamiento del rayo magico
    //
    if (iType==T_MAGIC || iType==T_ARROW)
    {
        if (!isValid()) return;
        
        if (ib_flip_x) position.x -= ANIM_SPEED;
        else position.x += ANIM_SPEED;
        
        // Si el disparo se sale de la pantalla, lo invalidamos
        //
        if (position.x < -10 || position.x > 440) invalidate();
    }

	if (ii_RetardoPos<ii_Retardo)
	{
		ii_RetardoPos ++;
	}
	else
	{
		ii_Frame = (ii_Frame<ii_AnimSequenceLength-1)?(ii_Frame+1):0;
		ii_RetardoPos=0;
	}
	TMASprite::animate();
}


bool TSpriteAnim::draw(TMABitmap& g)
{
    PMABitmap bmp;
    bmp = io_images->getBitmap(ii_CurrentFotograma);
    position.width = bmp->getWidth();
    position.height = bmp->getHeight();
    
    #ifdef SMALL_GAME
        position.width<<=1;
        position.height<<=1;
    #endif
    
    switch ((int)iType)
    {
        case T_LIGHT:
        case T_MAGIC:
            g.draw(bmp,position.x,position.y);
            break;
            
        case T_ARROW:
            g.drawSprite(bmp,position.x,position.y,ib_flip_x,ib_flip_y);
            break;
            
        case T_OTHER:
            TMASprite::draw(g);
            break;
    }
    return true;
}


#endif
