#ifndef _TSPRITEBRUJO_CPP_
#define _TSPRITEBRUJO_CPP_

#include "sprites.hpp"
#include "../spirits.hpp"

#ifndef TARGET_GP2X
	#define _DELTA_X    4
	#define _DELTA_Y    4
	#define _MAX_JUMP   15 // _MAX_JUMP = 60/_DELTA_Y
#else
	#define _DELTA_X    10
	#define _DELTA_Y    10
	#define _MAX_JUMP   6 // _MAX_JUMP = 60/_DELTA_Y
#endif

#define GETKEYHOLD(X)   myGame->inputController.isHoldDown(TSpiritsGame::X)
#define GETKEYDOWN(X)   myGame->inputController.isPressDown(TSpiritsGame::X)

REGISTER_CLASS_ID(TSpriteBrujo);

int TSpriteBrujo::ia_AnimDying[1]={0};
int TSpriteBrujo::ia_AnimWalk[6]={1,2,3,4,3,2};
int TSpriteBrujo::ia_AnimJumpUp[1]={6};
int TSpriteBrujo::ia_AnimJumpDown[1]={7};
int TSpriteBrujo::ia_AnimGetDown[2]={6,7};
int TSpriteBrujo::ia_AnimUpDown[2]={9,10};
int TSpriteBrujo::ia_AnimShoot[3]={8,8,8};
int TSpriteBrujo::ia_AnimDown[1]={5};

TSpriteBrujo::TSpriteBrujo(PMAImageBank bank,PMAImageBank bank1):TSpriteSpirits(bank)
{    
    ii_Retardo=3;
    ii_lives = 3;
    ii_energy = 100;
    sprShoot = new TSpriteAnim(bank1,TSpriteAnim::T_MAGIC);
    sprShoot->invalidate();
    ii_falling=0;
    ib_book=false; ib_magic=false; ib_casandra=false; ib_worf=false; ib_bad=false;
    setState(E_WALK);
}

TSpriteBrujo::~TSpriteBrujo()
{
    if (sprShoot) delete sprShoot;
}

void TSpriteBrujo::setState(State e)
{
    sounds->snd_step.stop();
    switch ((int)e)
    {
        case E_SHOOT:
            setAnimSequence(ia_AnimShoot,3);
            sprShoot->setFlipX(ib_flip_x);
            sprShoot->position.y = position.y+20;
            sprShoot->position.x = position.x + ((ib_flip_x)?0:20);
            sprShoot->validate();
            setEnergyDown(2);
            break;

        case E_WALK:
            setAnimSequence(ia_AnimWalk,6);
            ii_CurrentFotograma=0;
            if (ii_falling>30)
            {
                // Decrementamos vida, en funcin de la altura desde la que ha caido.
                setEnergyDown((ii_falling/10)*2);
                myGame->playSfx(sounds->snd_hurt);
                ii_falling=0;
            }
            break;
            
        case E_JUMP_UP:
            myGame->playSfx(sounds->snd_jump);
            ii_JumpCount=0;
            setAnimSequence(ia_AnimJumpUp,1);
            break;

        case E_JUMP_DOWN:
            ii_JumpCount=0;
            setAnimSequence(ia_AnimJumpDown,1);
            break;

        case E_FALLING:
            setAnimSequence(ia_AnimGetDown,2);
            ii_falling=0;
            break;
            
        case E_STAIRS:
            setAnimSequence(ia_AnimUpDown,2);
            break;
            
        case E_DOWN:
            setAnimSequence(ia_AnimDown,1);
            break;
    }
    ii_Estado = e;
}

bool TSpriteBrujo::evalMask(int mask)
{
    // Si se trata de la palanca, le ha tenido que dar con la cabeza y
    // se permite pasar, pero ponemos el flag de que se ha pulsado una palanca.
    //
    if (mask==MASK_HANDLE)
    {
        TSpiritsGame::msgQueue.putMessage(MSG_PUSH_HANDLE,this,(void*)screenID);
        return true;
    }

    switch (ii_Estado)
    {
        case E_JUMP_UP:
        case E_JUMP_DOWN:
            return (!(mask == MASK_DIE || mask == MASK_WALL || (mask == MASK_CLOSE && !rooms[screenID].wall_open) || mask == MASK_FLOOR));

        default:
            return (!(mask == MASK_DIE || mask == MASK_WALL || (mask == MASK_CLOSE && !rooms[screenID].wall_open)));
    }
}

bool TSpriteBrujo::doControll(void)
{
    // Calculamos el HOTSPOTX,Y
    //
    int px = getHotspotMapX();
    int py = getHotspotMapY();
    
    if (roomChanged(px,py))
    {
        sprShoot->invalidate();
        return false;
    }

    // Si el disparo esta activo, lo movemos.
    //
    if (sprShoot->isValid()) sprShoot->animate();

    // Si hay una lampara en el suelo, no podemos pasar
    //
    if (rooms[screenID].lamp)
    {
        TSpriteLamp *l = rooms[screenID].lamp;
        
        if (l->isOnFloor())
        {
            // Si el brujo esta a la izquierda del centro de la lampara
            if (hotspotX < l->getHotspotX())
            {
                if (hotspotX >= l->getHotspotX()-40 ) hotspotX -= _DELTA_X;
            }
            else
            {
                if (hotspotX < l->getHotspotX()+40 ) hotspotX += _DELTA_X;
            }
        }
    }

    // Si se esta saltando, el comportamiento cambia
    //
    switch (ii_Estado)
    {
    
        case E_JUMP_UP:

            if (ii_JumpCount>=_MAX_JUMP)
            {
                setState(E_JUMP_DOWN);
                return false;
            }
            else if (!isTop(px,py))
            {
                hotspotY -= _DELTA_Y;
            }
            ii_JumpCount++;

            px = getHotspotMapX();
            py = getHotspotMapY();

            if (ii_JumpDir<0)
            {
                if (canWalkLeft(px,py)) hotspotX -= _DELTA_X;
            }
            else if (ii_JumpDir>0)
            {
                if (canWalkRight(px,py)) hotspotX += _DELTA_X;
            }
            return true;

        case E_JUMP_DOWN:
    
            if (ii_JumpCount>_MAX_JUMP || isDown(px,py,_DELTA_Y) )
            {
                correctPositionY();
                setState(E_WALK);
                return true;
            }
            else if (!isDown(px,py,_DELTA_Y))
            {
                hotspotY += _DELTA_Y;
            }
            ii_JumpCount++;

            px = getHotspotMapX();
            py = getHotspotMapY();

            if (ii_JumpDir<0)
            {
                if (canWalkLeft(px,py)) hotspotX -= _DELTA_X;
            }
            else if (ii_JumpDir>0)
            {
                if (canWalkRight(px,py)) hotspotX += _DELTA_X;
            }
            return true;

        case E_FALLING:

            if (!isDown(px,py,_DELTA_Y))
            {
                hotspotY += _DELTA_Y;
                ii_falling++;
            }
            else
            {
                correctPositionY();
                setState(E_WALK);
            }
            return true;

        case E_STAIRS:

            if (! (isStairs(px,py) || isStairs(px,py-2)) )
            {
                setState(E_WALK);
                return false;
            }
            else
            if (GETKEYHOLD(K_UP))
            {
                if (!isTop(px,py)) hotspotY -= _DELTA_Y;
                py = getHotspotMapY();
                if (!isStairs(px,py-2)) hotspotY += _DELTA_Y;
                if (!sounds->snd_stairs.isPlaying()) myGame->playSfx(sounds->snd_stairs);
                return true;
            }
            else
            if (GETKEYHOLD(K_DOWN))
            {
                if (isStairs(px,py-2)) hotspotY += _DELTA_Y;
                py = getHotspotMapY();
                if (!isStairs(px,py)) hotspotY -= _DELTA_Y;
                if (!sounds->snd_stairs.isPlaying()) myGame->playSfx(sounds->snd_stairs);
                return true;
            }
            else
            if (GETKEYHOLD(K_LEFT))
            {
                if (canWalkLeft(px,py)) hotspotX -= _DELTA_X;
                return true;
            }
            else
            if (GETKEYHOLD(K_RIGHT))
            {
                if (canWalkRight(px,py)) hotspotX += _DELTA_X;
                return true;
            }
            else
            {
                sounds->snd_stairs.stop();
            }
            return false;
            
        case E_DOWN:

            if (!GETKEYHOLD(K_DOWN))
            {
                setState(E_WALK);
                return false;
            }
            return true;

        case E_SHOOT:
            if (ii_Frame==2) setState(E_WALK);
            return true;

        case E_WALK:
        
            // Caso que estemos sobre muerte
            //
            if (io_Screen->getMask(px,py+1)==MASK_DIE)
            {
                setEnergyDown(2);
            }

            // Comprobamos si hay hay suelo, si no hay, caemos.
            //
            if (!isDown(px,py,_DELTA_Y))
            {
                setState(E_FALLING);
                return false;
            }

            if (isStairs(px,py))
            {
                setState(E_STAIRS);
                return false;
            }

            // Disparar
            //
            if (GETKEYDOWN(K_SHOOT))
            {
                if (!sprShoot->isValid())
                {
                    setState(E_SHOOT);
                    myGame->playSfx(sounds->snd_shoot);
                }
                return false;
            }
            else if (GETKEYHOLD(K_LEFT))
            {
                // Intento de andar a la izquierda
                // Lo primero, poner hacia la izquierda
                //
                if (!ib_flip_x)
                {
                    ib_flip_x = true;
                    return true;
                }

                // Miramos si se pulsa salto
                //
                if (GETKEYHOLD(K_UP))
                {
                    ii_JumpDir=-1;
                    setState(E_JUMP_UP);
                    return false;
                }

                if (canWalkLeft(px,py)) hotspotX-= _DELTA_X;
                if (!sounds->snd_step.isPlaying()) myGame->playSfx(sounds->snd_step);
                return true;
            }
            else if (GETKEYHOLD(K_RIGHT))
            {
                // Lo primero, poner hacia la izquierda
                //
                if (ib_flip_x)
                {
                    ib_flip_x = false;
                    return true;
                }

                // Miramos si se pulsa salto
                //
                if (GETKEYHOLD(K_UP))
                {
                    ii_JumpDir=1;
                    setState(E_JUMP_UP);
                    return false;
                }

                if (canWalkRight(px,py)) hotspotX+= _DELTA_X;
                if (!sounds->snd_step.isPlaying()) myGame->playSfx(sounds->snd_step);
                return true;
            }
            else if (GETKEYHOLD(K_UP))
            {
                ii_JumpDir=0;
                setState(E_JUMP_UP);
                return false;
            }
            else if (GETKEYHOLD(K_DOWN))
            {
                setState(E_DOWN);
                return false;
            }
            else
            {
                // EL brujo esta parado.
                //
                ii_CurrentFotograma=0;
                sounds->snd_step.stop();
                return false;
            }
        }

    return false;
}

void TSpriteBrujo::animate(void)
{
    if (ii_flashtime>0)
    {
        ii_flashtime--;
    }

    if (!doControll()) return;

	if (ii_RetardoPos<ii_Retardo)
	{
		ii_RetardoPos ++;
	}
	else
	{
		ii_Frame = (ii_Frame<ii_AnimSequenceLength-1)?(ii_Frame+1):0;
		ii_RetardoPos=0;
	}
	
	TSpriteSpirits::animate();
}

int *TSpriteBrujo::getKillSequence(int& length)
{
    length=1;
    return ia_AnimDying;
}

// Detecta una colision con otro Sprite
//
bool TSpriteBrujo::collisionsWith(TSpriteSpirits& spr)
{
    // Mientras estemos inmune, no pueden hacernos dao.
    //
    if (ii_flashtime>0) return false;
    if (spr.getScreenID()!=screenID) return false;
    
    // Si estamos agachados, corregimos el area de sprite
    //
    if (ii_Estado==E_DOWN)
    {
        TMARectangle r;
        r.setBounds(position);
        r.y+=20;
        r.height-=20;
        return r.intersects(spr.position);
    }
    return position.intersects(spr.position);
}

bool TSpriteBrujo::collisionMagicWith(TSpriteSpirits& spr)
{
    if (!sprShoot->isValid()) return false;
    if (spr.getScreenID()!=screenID) return false;
    return sprShoot->isCollisioningWith(spr);
}

void TSpriteBrujo::setLiveDown(void)
{
    if (ii_lives>=0)
    {
        ii_lives--;
        ii_energy=100;
        setFlashTime(200);
    }
}

void TSpriteBrujo::setEnergyDown(int how)
{
    static int count=0;
    
    for (int i=0;i<how;i++)
    {
        count++;
        if (count>2)
        {
            count=0;
            if (ii_energy>0) ii_energy--;
        }
    }
}

bool TSpriteBrujo::draw(TMABitmap& b)
{
    TSpriteSpirits::draw(b);
    if (sprShoot->isValid()) sprShoot->draw(b);
    return true;
}

void TSpriteBrujo::setScreenID(int id)
{
    TSpriteSpirits::setScreenID(id);
    rooms[id].visited = true;
}

void TSpriteBrujo::invalidateMagic(void)
{
    sprShoot->invalidate();
}

#undef _DELTA_X
#undef _DELTA_Y
#undef _MAX_JUMP
#undef GETKEY

#endif
