#ifndef _TSPRITEDEAMBULANTE_CPP_
#define _TSPRITEDEAMBULANTE_CPP_

#include "sprites.hpp"
#include "../spirits.hpp"

#ifndef TARGET_GP2X
	#define _DELTA_X    2
	#define _DELTA_Y    2
	#define _SALTO_DX   4
	#define _SALTO_DY   4
	#define _SALTO_Y    16
	#define _EAGLE_Y    8
#else
	#define _DELTA_X    4
	#define _DELTA_Y    4
	#define _SALTO_DX   4
	#define _SALTO_DY   4
	#define _SALTO_Y    16
	#define _EAGLE_Y    8
#endif

REGISTER_CLASS_ID(TSpriteDeambulante);

int TSpriteDeambulante::ia_AnimCasandra[4]={74,75,74,74};
int TSpriteDeambulante::ia_AnimCasandraWalk[4]={38,39,38,40};
int TSpriteDeambulante::ia_AnimCasandraUpDown[2]={41,42};
int TSpriteDeambulante::ia_AnimWorf[1]={77};
int TSpriteDeambulante::ia_AnimWorfWalk[4]={33,34,33,35};
int TSpriteDeambulante::ia_AnimWorfUpDown[2]={36,37};
int TSpriteDeambulante::ia_AnimEagle[4]={28,29,30,29};
int TSpriteDeambulante::ia_AnimHumo[14] = {109,110,111,112,111,110,111,112,111,110,111,112,111,113 };


TSpriteDeambulante::TSpriteDeambulante(PMAImageBank bank,TSpriteDeambulante::Type t):TSpriteSpirits(bank)
{
    ii_Retardo=6;
    itype=t;
    setFlipX(TMARandom::nextInt(100)<50);
    ib_decidido=false;
    ib_sube_escalones=false;
    ib_decide_saltar=false;
    itimeforgot=0;
    ib_got=0;
    
    switch (t)
    {
        case T_CASANDRA:
            setScreenID(81);
            putAt(17,9);
            last_state = E_WALK;
            setState(E_WALK);
            break;
            
        case T_WORF:
            setScreenID(14);
            putAt(3,9);
            last_state = E_WALK;
            setState(E_WALK);
            break;
            
        case T_EAGLE:
            setScreenID(71);
            putAt(11,_EAGLE_Y);
            last_state = E_FLY;
            setState(E_FLY);
            posy_eagle = hotspotY;
            break;
    }
    invalidate();
}

void TSpriteDeambulante::setState(State e)
{
    last_state = (State)ii_Estado;
    switch ((int)e)
    {
        case E_UP:
            ib_sube_escalones=true;
            break;
            
        case E_WALK:
        case E_FALLING:
            setAnimSequence((itype==T_CASANDRA)?ia_AnimCasandraWalk:ia_AnimWorfWalk,4);
            break;
            
        case E_STAIRS_UP:
        case E_STAIRS_DOWN:
            setAnimSequence((itype==T_CASANDRA)?ia_AnimCasandraUpDown:ia_AnimWorfUpDown,2);
            break;
            
        case E_FLY:
        case E_FLY_UP:
        case E_FLY_DOWN:
            setAnimSequence(ia_AnimEagle,4);
            break;
            
        case E_LIBERING:
            setAnimSequence(ia_AnimHumo,14);
            myGame->playSfx(sounds->snd_abra);
            break;
            
        case E_LIBERED:

            switch (itype)
            {
                case T_CASANDRA:
                    setAnimSequence(ia_AnimCasandra,4);
                    break;
                    
                case T_WORF:
                    setAnimSequence(ia_AnimWorf,1);
            }
            break;

    }
    ii_Estado = e;
    icounter=0;
}

bool TSpriteDeambulante::canWalkRight(int px,int py)
{
    if (px<0) return true;

    if (itype==T_EAGLE)
    {
        if (px>=22) return true;
    }
    else
    {
        if (px>=23) return true;
    }

    int lMask,i;
    int mid = position.width/2/_TileAX;
    int ay = position.height/_TileAY;

    for (i=0;i<ay;i++)
    {
        lMask=io_Screen->getMask(px+mid,py-i);
        if (!evalMask(lMask)) return false;
    }
	return true;
}

bool TSpriteDeambulante::isDown(int px,int py)
{
    bool lb_resp=false;
    
    // PX,PY son el HOTSPOT del Sprite, situado en el centro
    // de la base del sprite.
    //
    int mid = position.width/2/_TileAX;
    int li_rest = (hotspotY%_TileAY);
    
    for (int i=1-mid;i<mid;i++)
	{
        int val = io_Screen->getMask(px+i,py+1);
		if ( val==MASK_DIE || val == MASK_WALL || (itype!=T_EAGLE && (val == MASK_FLOOR || val == MASK_STAIRS) ) ||
            (val == MASK_CLOSE && !rooms[screenID].wall_open) )
        {
            lb_resp=true;
            break;
        }
	}
	
	// Si debajo hay suelo, pero aun no estamos cuadrados sobre la posicin Y
	// devolvemos false.
	//
    if (lb_resp && (li_rest+_DELTA_Y <= _TileAY)) return false;
    
	return lb_resp;
}


bool TSpriteDeambulante::doControllHuman(void)
{
    int prob_up,prob_down;
    
    // Calculamos el HOTSPOTX,Y
    //
    int px = getHotspotMapX();
    int py = getHotspotMapY();

    if (roomChanged(px,py))
    {
        px = getHotspotMapX();
        py = getHotspotMapY();
        ib_decidido=false;
        ib_sube_escalones=false;
        ib_decide_saltar=false;
        itimeforgot=0;
        if (isDown(px,py))
        {
        	  correctPositionY();
            if (ii_Estado==E_UP) setState(E_WALK);
        }
        return false;
    }


    itimeforgot++;
    if (itimeforgot>50)
    {
        ib_decidido=false;
        itimeforgot=0;
    }

    switch (ii_Estado)
    {

        case E_LIBERED:

            if (!isDown(px,py))
            {
                hotspotY += _DELTA_Y;
            }
            
            if (target && target->getScreenID()==getScreenID())
            {
                setFlipX(getHotspotMapX()>target->getHotspotMapX());
            }

            return true;
            
        case E_LIBERING:

            if (ii_Frame==13) setState(E_LIBERED);
            return true;

        case E_STAIRS_UP:

            if (!isStairs(px,py))
            {
                ib_decidido=true;
                // Hay un 30% de probabilidades, de cambiar de sentido al volver a caminar
                if (TMARandom::nextInt(100)<30) ib_flip_x = !ib_flip_x;
                setState(E_WALK);
                return false;
            }
            hotspotY -= _DELTA_Y;
            return true;

        case E_STAIRS_DOWN:

            if (io_Screen->getMask(px,py)==MASK_WALL)
            {
                ib_decidido=true;
                hotspotY -= _DELTA_Y;
                // Hay un 30% de probabilidades, de cambiar de sentido al volver a caminar
                if (TMARandom::nextInt(100)<30) ib_flip_x = !ib_flip_x;
                setState(E_WALK);
                return false;
            }
            hotspotY += _DELTA_Y;
            return true;

        case E_FALLING:

            if (!isDown(px,py))
            {
                hotspotY += _DELTA_Y;
            }
            else
            {
            		correctPositionY();
                setState(E_WALK);
                return false;
            }
            return true;

        case E_UP: // Subiendo una escalera
        
            hotspotY -= _DELTA_Y;
            py = getHotspotMapY();
            
            if (isDown(px,py))
            {
            		correctPositionY();
                setState(E_WALK);
                return false;
            }
            return true;

        case E_UP1: // Subiendo unico escalon

            if (icounter<12)
            {
                hotspotY -= 2;
                hotspotX += ((ib_flip_x)?-2:2);
            }
            else
            {
                putAt(px,py);
                setState(E_WALK);
            }

            icounter++;
            return true;
            
        case E_JUMP:

            if (ib_flip_x)
            {
                if (canWalkLeft(px,py)) hotspotX -= _SALTO_DX;
            }
            else
            {
                if (canWalkRight(px,py)) hotspotX += _SALTO_DX;
            }
            
            if (icounter<_SALTO_Y)
            {
                if (!isTop(px,py)) hotspotY -= _SALTO_DY;
            }
            else
            {
                if (!isDown(px,py)) hotspotY += _SALTO_DY;
            }
            
            icounter++;
            if (icounter>_SALTO_Y*2-1)
            {
                putAt(px,py);
                setState(E_WALK);
                ib_decide_saltar=false;
            }
            return true;

        case E_WALK:


            // Evitamos que se entre en algunas pantallas que podran bloquear
            // al sprite
            //
            switch (screenID)
            {
                case 78:
                    if (px<12 && ib_flip_x && !rooms[80].wall_open) ib_flip_x=false;
                    break;
            }

            // Determinamos si merece la pena pegar un salto.
            //
            if (!ib_decide_saltar && (py>=8) && !ib_sube_escalones)
            {
                // Hay ms probabilidades de realizar el salto, si la pantalla de abajo
                // ha sido visitada.
                //
                if (itype==T_CASANDRA)
                {
                    prob_down = (rooms[TSpiritsGame::gameMapInfo[screenID].down].visited_casandra)?80:60;
                }
                else
                {
                    prob_down = (rooms[TSpiritsGame::gameMapInfo[screenID].down].visited_worf)?80:60;
                }
                
                if (ib_flip_x)
                {
                    if (!isDown(px-1,py) && !isDown(px-1,py+1))
                    {
                        ib_decide_saltar=true;
                        if (TMARandom::nextInt(100)<prob_down)
                        {
                            setState(E_JUMP);
                            return false;
                        }
                    }
                }
                else
                {
                    if (!isDown(px+1,py) && !isDown(px+1,py+1))
                    {
                        ib_decide_saltar=true;
                        if (TMARandom::nextInt(100)<prob_down)
                        {
                            setState(E_JUMP);
                            return false;
                        }
                    }
                }
            }

            if (ib_sube_escalones && (io_Screen->getMask(px,py-1)==MASK_FLOOR || io_Screen->getMask(px,py)==MASK_FLOOR))
            {
                setState(E_UP);
                return false;
            }
            else if (!ib_decidido && (io_Screen->getMask(px,py-1)==MASK_FLOOR || io_Screen->getMask(px,py)==MASK_FLOOR))
            {
                ib_decidido=true;
                if (TMARandom::nextInt(100)<55)
                {
                    setState(E_UP);
                    return false;
                }
            }

            // Comprobamos si hay hay suelo, si no hay, caemos.
            //
            if (!isDown(px,py))
            {
                setState(E_FALLING);
                return false;
            }
            
            if (!ib_decidido && isStairs(px,py))
            {
                ib_decidido=true;
                
                // Primero decidimos si queremos subir o bajar por la escalera.
                // Con una probabilidad del 80%. Adems, no se debe bajar a la pantalla 72 (ojos).
                //
                if (TSpiritsGame::gameMapInfo[screenID].down!=72)
                {
                    if (itype==T_CASANDRA)
                    {
                        prob_up = (rooms[TSpiritsGame::gameMapInfo[screenID].up].visited_casandra)?40:80;
                        prob_down = (rooms[TSpiritsGame::gameMapInfo[screenID].down].visited_casandra)?40:80;
                    }
                    else
                    {
                        prob_up = (rooms[TSpiritsGame::gameMapInfo[screenID].up].visited_worf)?40:80;
                        prob_down = (rooms[TSpiritsGame::gameMapInfo[screenID].down].visited_worf)?40:80;
                    }
                    
                    // Ahora miramos si queremos subir o bajar.
                    //
                    if (canStairsUp(px,py) && !canStairsDown(px,py) && TMARandom::nextInt(100)<prob_up) setState(E_STAIRS_UP);
                    else if (!canStairsUp(px,py) && canStairsDown(px,py) && TMARandom::nextInt(100)<prob_down) setState(E_STAIRS_DOWN);
                    else if (canStairsUp(px,py) && TMARandom::nextInt(100)<prob_up) setState(E_STAIRS_UP);
                    else if (canStairsDown(px,py) && TMARandom::nextInt(100)<prob_down) setState(E_STAIRS_DOWN);
                }
                return true;
            }
            
            if (ib_flip_x)
            {
                if (canWalkLeft(px,py)) hotspotX-= _DELTA_X;
                else if (canWalkLeft(px,py-1))
                {
                    setState(E_UP1);
                    return false;
                }
                else ib_flip_x = false;
            }
            else
            {
                if (canWalkRight(px,py)) hotspotX+= _DELTA_X;
                else if (canWalkRight(px,py-1))
                {
                    setState(E_UP1);
                    return false;
                }
                else ib_flip_x = true;
            }
            return true;
    }

    return false;
}

bool TSpriteDeambulante::doControllEagle(void)
{
    int prob=0;
    
    // Calculamos el HOTSPOTX,Y
    //
    int px = getHotspotMapX();
    int py = getHotspotMapY();

    if (roomChanged(px,py))
    {
        ib_decidido=false;
        itimeforgot=0;
        return false;
    }

    itimeforgot++;
    if (itimeforgot>50)
    {
        ib_decidido=false;
        itimeforgot=0;
    }

    if (target && target->collisionsWith(*this))
    {
        target->setEnergyDown(2);
    }

    switch (ii_Estado)
    {
        case E_FLY_UP:

            hotspotY-= _DELTA_Y;
            if (hotspotY==posy_eagle)
            {
                if (!canEagleFlyUp(px,py) || TMARandom::nextInt(100)<10 )
                {
                    setFlipX(!ib_flip_x);
                    setState(E_FLY);
                    return false;
                }
            }
            return true;

        case E_FLY_DOWN:

            hotspotY+= _DELTA_Y;
            if (hotspotY==posy_eagle) 
            {
                if (!canEagleFlyDown(px,py) || TMARandom::nextInt(100)<10)
                {
                    setFlipX(!ib_flip_x);
                    setState(E_FLY);
                    return false;
                }
            }
            return true;

        case E_FLY:

            // Si se puede ir a la pantalla superior, decidimos si se hace o no.
            // Si ya ha estado, la probabilidad de volver es del 40% sino del 80%.
            //
            if (!ib_decidido && canEagleFlyUp(px,py))
            {
                ib_decidido=true;
                prob = (rooms[TSpiritsGame::gameMapInfo[screenID].up].visited_eagle)?40:80;
                
                if (TMARandom::nextInt(100)<=prob)
                {
                    // El aguila no puede ir a una pantalla inexistente
                    //
                    if (TSpiritsGame::gameMapInfo[screenID].up>0)
                    {
                        setState(E_FLY_UP);
                        return false;
                    }
                }
            }

            // Si se puede ir a la pantalla inferior, decidimos si se hace o no.
            // Si ya ha estado, la probabilidad de volver es del 40% sino del 80%.
            //
            if (!ib_decidido && canEagleFlyDown(px,py))
            {
                ib_decidido=true;

                // Solo bajaremos, si ya se han visitado las pantallas de ambos lados.
                //
                if (TSpiritsGame::gameMapInfo[screenID].left>0 && !rooms[TSpiritsGame::gameMapInfo[screenID].left].visited_eagle) prob=10;
                else if (TSpiritsGame::gameMapInfo[screenID].right>0 && !rooms[TSpiritsGame::gameMapInfo[screenID].right].visited_eagle) prob=10;
                else prob = (rooms[TSpiritsGame::gameMapInfo[screenID].down].visited_eagle)?40:80;
                
                if (TMARandom::nextInt(100)<prob)
                {
                    // El aguila no puede ir a la pantalla 72 y 74
                    //
                    if (TSpiritsGame::gameMapInfo[screenID].down!=72 &&
                        TSpiritsGame::gameMapInfo[screenID].down!=74)
                    {
                        setState(E_FLY_DOWN);
                        return false;
                    }
                }
            }
                
            if (ib_flip_x)
            {
                if (canWalkLeft(px,py)) hotspotX-= _DELTA_X;
                else ib_flip_x = false;
            }
            else
            {
                if (canWalkRight(px,py)) hotspotX+= _DELTA_X;
                else ib_flip_x = true;
            }
            return true;
    }

    return false;
}

void TSpriteDeambulante::animate(void)
{
    if (!isGot() && target && target->collisionMagicWith(*this))
    {
        switch (itype)
        {
            case T_CASANDRA:

                // Si el mago dispara a casandra, y tiene en su poder la barita
                // la podra desencantar
                //
                if (target->hasMagic())
                {
                    TSpiritsGame::msgQueue.putMessage(MSG_CASANDRA_GOT);
                    target->invalidateMagic();
                    setGot();
                    setState(E_LIBERING);
                }
                break;

            case T_WORF:

                // Si el mago dispara a worf, y tiene en su poder el libro
                // lo podra desencantar
                //
                if (target->hasBook())
                {
                    TSpiritsGame::msgQueue.putMessage(MSG_WORF_GOT);
                    target->invalidateMagic();
                    setGot();
                    setState(E_LIBERING);
                }
                break;

            case T_EAGLE:

                // Si el mago dispara al aguila y ya ha liberado a casandra y worf
                // podra vencerla
                //
                if (target->hasCasandra() && target->hasWorf())
                {
                    TSpiritsGame::msgQueue.putMessage(MSG_BAD_GOT);
                    target->invalidateMagic();
                    setGot();
                    setState(E_LIBERED);
                }
                break;
        }
    }

    switch (itype)
    {
        case T_CASANDRA:
        case T_WORF:
            if (!doControllHuman()) return;
            break;

        case T_EAGLE:
            if (!doControllEagle()) return;
            break;
    }

	if (ii_RetardoPos<ii_Retardo)
	{
		ii_RetardoPos ++;
	}
	else
	{
		ii_Frame = (ii_Frame<ii_AnimSequenceLength-1)?(ii_Frame+1):0;
		ii_RetardoPos=0;
	}

	TSpriteSpirits::animate();
}

bool TSpriteDeambulante::canEagleFlyUp(int px,int py)
{
    int off = (ib_flip_x)?1:-1;

    if (
        io_Screen->getMask(px+off,0)==MASK_WALL || io_Screen->getMask(px,0)==MASK_WALL ||
        (!rooms[screenID].wall_open && (io_Screen->getMask(px,0)==MASK_CLOSE || io_Screen->getMask(px+off,0)==MASK_CLOSE))
    ) return false;
       
    return true;
}

bool TSpriteDeambulante::canEagleFlyDown(int px,int py)
{
    int off = (ib_flip_x)?1:-1;

    if (
        io_Screen->getMask(px+off,11)==MASK_WALL || io_Screen->getMask(px,11)==MASK_WALL ||
        (!rooms[screenID].wall_open && (io_Screen->getMask(px,11)==MASK_CLOSE || io_Screen->getMask(px+off,11)==MASK_CLOSE))
    ) return false;

    return true;
}

bool TSpriteDeambulante::canStairsUp(int px,int py)
{
    return (io_Screen->getMask(px,0)==MASK_STAIRS);
}

bool TSpriteDeambulante::canStairsDown(int px,int py)
{
    return (io_Screen->getMask(px,11)==MASK_STAIRS);
}

bool TSpriteDeambulante::isStairs(int px,int py)
{
    if (ib_flip_x) return (io_Screen->getMask(px+1,py-1)==MASK_STAIRS);
    else return (io_Screen->getMask(px-1,py-1)==MASK_STAIRS);
}

void TSpriteDeambulante::setScreenID(int id)
{
    TSpriteSpirits::setScreenID(id);
    switch (itype)
    {
        case T_EAGLE:
            rooms[id].visited_eagle = true;
            break;
        case T_WORF:
            rooms[id].visited_worf = true;
            break;
        case T_CASANDRA:
            rooms[id].visited_casandra = true;
            break;
    }
}

#undef _DELTA_X
#undef _DELTA_Y
#undef _EAGLE_Y

#endif
