#ifndef _TSPRITEDOOR_CPP_
#define _TSPRITEDOOR_CPP_

#include "sprites.hpp"
#include "../spirits.hpp"

REGISTER_CLASS_ID(TSpriteDoor);

TSpriteDoor::TSpriteDoor(PMAImageBank bank,int imgindex,bool vertical):TSpriteSpirits(bank)
{
    ii_Retardo=0;
    ii_CurrentFotograma = imgindex;
    ii_Estado = S_CLOSE;
    ib_vertical = vertical;
    
    // Obtenemos la altura inicial
    //
    PMABitmap bmp = io_images->getBitmap(ii_CurrentFotograma);
    size = (ib_vertical)?bmp->getHeight():bmp->getWidth();
}

bool TSpriteDoor::isOpen(void)
{
    return (ii_Estado == S_OPEN);
}

void TSpriteDoor::open(void)
{
    if (ii_Estado==S_CLOSE) ii_Estado = S_OPENING;
}

void TSpriteDoor::animate(void)
{
    switch (ii_Estado)
    {
        case S_OPENING:

            #ifdef SMALL_GAME
            if (size>5)
            #else
            if (size>10)
            #endif
            {
            	if (ii_RetardoPos<ii_Retardo)
            	{
            		ii_RetardoPos ++;
            	}
            	else
            	{
            		ii_RetardoPos=0;
            		size--;
            	}
            }
            else
            {
                ii_Estado = S_OPEN;
            }
            break;
    }
}


bool TSpriteDoor::draw(TMABitmap& g)
{
    PMABitmap bmp = io_images->getBitmap(ii_CurrentFotograma);
    int lsize = (ib_vertical)?(bmp->getHeight() - size):(bmp->getWidth() - size);
    position.width = (ib_vertical)?bmp->getWidth():size;
    position.height = (ib_vertical)?size:bmp->getHeight();
    
    #ifdef SMALL_GAME
        position.width<<=1;
        position.height<<=1;
    #endif

    PMABitmap img;
    if (ib_vertical)
        img = TMABitmap::create(bmp,0,lsize,bmp->getWidth(),size);
    else
        img = TMABitmap::create(bmp,lsize,0,size,bmp->getHeight());

    g.draw(img,position.x,position.y);
    delete img;

    #ifdef SMALL_GAME
        lsize<<=1;
    #endif

    if (ib_vertical)
    {
        if (!rooms[screenID].wall_open && lsize>=target->position.height)
    	{
            rooms[screenID].wall_open=true;
        }
    }
    else
    {
        if (!rooms[screenID].wall_open && lsize>=target->position.width)
    	{
            rooms[screenID].wall_open=true;
        }
    }
    
    return true;
}


#endif
