#ifndef _TSPRITEENEMIGO_CPP_
#define _TSPRITEENEMIGO_CPP_

#include "sprites.hpp"
#include "../spirits.hpp"

#ifndef TARGET_GP2X
	#define _DELTA_X   4
#else
	#define _DELTA_X   10
#endif

#define _MAX_WAITING 200

REGISTER_CLASS_ID(TSpriteEnemigo);

int TSpriteEnemigo::ia_AnimWolf[4]={11,12,11,13};
int TSpriteEnemigo::ia_AnimHump[4]={14,15,14,16};
int TSpriteEnemigo::ia_AnimHump1[4]={19,31,19,32};
int TSpriteEnemigo::ia_AnimGhost[2]={32,33};

TSpriteEnemigo::TSpriteEnemigo(TSpriteEnemigo::Type t,PMAImageBank bank,PMAImageBank bank1):TSpriteSpirits(bank)
{
    ii_Retardo=3;
    velocity = _DELTA_X;
    soundwait=100+TMARandom::nextInt(100);
    ii_ice=47;
    ibank1=bank1;
    
    switch (t)
    {
        case T_WOLF:
            setAnimSequence(ia_AnimWolf,4);
            break;
            
        case T_GHOST:
            ii_ice=68;
            setAnimSequence(ia_AnimGhost,2);
            break;

        case T_HUMP:
            setAnimSequence(ia_AnimHump,4);
            break;

        case T_HUMP1:
            setAnimSequence(ia_AnimHump1,4);
            break;
    }

    ib_follow=false;
    soundcount=soundwait;
    iType = (int)t;
    ii_Estado=E_WALK;
    setState(E_WALK);
}

void TSpriteEnemigo::setState(State e)
{
    laststate = (State)ii_Estado;
    ii_Estado = e;
    
    if (e==E_STOP)
    {
        waiting=0;
        if (iType!=T_GHOST)
        {
            ii_ice++;
            if (ii_ice>49) ii_ice=47;
        }
    }
}

void TSpriteEnemigo::doSound(bool force)
{
    // Controlamos el sonido.
    //
    soundcount++;
    if (force || (soundcount>=soundwait))
    {
        int pan,volume=0;
        soundcount=0;

        // El sonido varia si el enemigo esta en la misma pantalla que el
        // sprite del mago, si esta una pantalla a la izquierda o una pantalla
        // a la derecha
        //
        if (screenID == target->getScreenID())
        {
            pan=127,volume=127;
        }
        else if (screenID == TSpiritsGame::gameMapInfo[target->getScreenID()].left)
        {
            pan=0,volume=20;
        }
        else if (screenID == TSpiritsGame::gameMapInfo[target->getScreenID()].right)
        {
            pan=255,volume=20;
        }

        if (volume>0)
        {
            switch (iType)
            {
                case T_WOLF:
                    myGame->playSfx(sounds->snd_wolf,false,volume,pan);
                    break;

                case T_GHOST:
                    myGame->playSfx(sounds->snd_ghost,false,volume,pan);
                    break;

                case T_HUMP:
                case T_HUMP1:
                    myGame->playSfx(sounds->snd_hump,false,volume,pan);
                    break;
            }
        }
    }
}

bool TSpriteEnemigo::doControll(void)
{
    // Calculamos el HOTSPOTX,Y
    //
    int px = getHotspotMapX();
    int py = getHotspotMapY();
    
    if (target && target->collisionMagicWith(*this) && (ii_Estado!=E_STOP))
    {
        setState(E_STOP);
        target->invalidateMagic();
        return false;
    }

    if (target && target->collisionsWith(*this))
    {
        // Decrementamos energia, en funcin del tipo de enemigo que sea
        //
        switch (iType)
        {
            case T_GHOST:
                target->setEnergyDown(2);
                break;
                
            case T_HUMP:
            case T_HUMP1:
                target->setEnergyDown(4);
                break;
                
            case T_WOLF:
                target->setEnergyDown(6);
                break;
        }

        switch (ii_Estado)
        {
            case E_WALK:
                setState(E_FOLLOW_LEFT);
                break;

            case E_FOLLOW_LEFT:

                if (px<target->getHotspotMapX()-2)
                {
                    ib_flip_x = false;
                    velocity = abs(velocity);
                    setState(E_FOLLOW_RIGHT);
                }
                break;

            case E_FOLLOW_RIGHT:

                if (px>target->getHotspotMapX()+1)
                {
                    ib_flip_x = true;
                    velocity = -abs(velocity);
                    setState(E_FOLLOW_LEFT);
                }
                break;
        }
    }
    else
    {
        setState(E_WALK);
        if (roomChanged(px,py))
        {
            ib_follow=false;
            // Si el enemigo entra en la misma pantalla que el brujo
            // debe emitir su sonido
            //
            if (screenID == target->getScreenID()) doSound(true);
            return false;
        }
    }

    doSound(false);

    // Si estamos en la misma pantalla que el brujo, lo perseguimos
    //
    if (screenID == target->getScreenID())
    {
        if (!ib_follow)
        {
            ib_follow=true;

            if (px<target->getHotspotMapX())
            {
                ib_flip_x = false;
                velocity = abs(velocity);
            }
            else if (px>target->getHotspotMapX())
            {
                ib_flip_x = true;
                velocity = -abs(velocity);
            }
        }
    }
    else ib_follow=false;


    // Si ha llegado a su limite izquierdo
    //
    if (screenID == screenID1 && px <= posx1) 
    {
        // Hacemos que ande hacie la derecha
        //
        ib_flip_x = false;
        velocity = abs(velocity);
    }
    //
    else
    //
    // Si ha llegado a su limite derecho
    //
    if (screenID == screenID2 && px >= posx2)
    {
        // Hacemos que ande hacie la derecha
        //
        ib_flip_x = true;
        velocity = -abs(velocity);
    }
    
    if (velocity>0)
    {
        if (!canWalkRight(px,py))
        {
            velocity = - abs(velocity);
            ib_flip_x = true;
        }
    }
    else
    {
        if (!canWalkLeft(px,py))
        {
            velocity = abs(velocity);
            ib_flip_x = false;
        }
    }
    
    // Desplazamos al personaje
    //
    hotspotX += velocity;

    return true;
}

void TSpriteEnemigo::animate(void)
{
    if (ii_Estado==E_STOP)
    {
        waiting++;
        if (waiting>_MAX_WAITING) setState(laststate);
        return;
    }

    if (!doControll()) return;

	if (ii_RetardoPos<ii_Retardo)
	{
		ii_RetardoPos ++;
	}
	else
	{
		ii_Frame = (ii_Frame<ii_AnimSequenceLength-1)?(ii_Frame+1):0;
		ii_RetardoPos=0;
	}

	TSpriteSpirits::animate();
}

void TSpriteEnemigo::setRoute(int ScreenID1,int px1,int ScreenID2,int px2,int posy,int vel)
{
    screenID1 = ScreenID1;
    screenID2 = ScreenID2;
    posx1 = px1;
    posx2 = px2;
    hotspotY = posy;
    
    #ifndef TARGET_GP2X
	    	velocity = abs(vel);
		#else
			velocity=abs((vel*3)/2);
		#endif

    // Establecemos una posicin horizontal aleatoria de inicio
    // en el camino definido.
    // Lo primero es detectar cuantas pantallas hay en horizontal desde
    // la pantalla de origen a la de fin
    //
    int count = 0, scr=screenID1;
    
    while (scr!=screenID2 && scr!=-1)
    {
        scr = TSpiritsGame::gameMapInfo[scr].right;
        count++;
    }
    
    // Caso que haya algun error, porque no se puede llegar
    // desde screenID1 en horizontal a screenID2, partimos
    // de screenID1
    //
    if (scr==-1) scr=screenID1;
    else
    {
        count = TMARandom::nextInt(count);
        scr = screenID1;
        while (count>0)
        {
            scr = TSpiritsGame::gameMapInfo[scr].right;
            count--;
        }
    }
    
    // Ahora determinamos una posicin aleatoria dentro de la pantalla
    //
    if (scr == screenID1)
    {
        hotspotX = posx1 + TMARandom::nextInt(22-posx1);
        if (hotspotX == posx1) hotspotX = posx1+1;
    }
    else if (scr == screenID2)
    {
        hotspotX = 1 + TMARandom::nextInt(posx2-2);
        if (hotspotX == posx2) hotspotX = posx2-1;
    }
    else
    {
        hotspotX = 4 + TMARandom::nextInt(20);
    }
    
    // Posicionamos el sprite y establecemos la pantalla
    // a la que pertenecera inicialmente
    //
    putAt(hotspotX,hotspotY);
    setScreenID(scr);
}

bool TSpriteEnemigo::draw(TMABitmap& g)
{    
    g.drawSprite(io_images->getBitmap(ii_CurrentFotograma),position.x,position.y,ib_flip_x,ib_flip_y);
    
    if (ii_Estado==E_STOP)
    {
        PMABitmap bmp1 = ibank1->getBitmap(ii_ice);
        int lw = bmp1->getWidth();
        int lh = bmp1->getHeight();
        
        #ifdef SMALL_GAME
            lw<<=1; lh<<=1;
        #endif
        
        int px = position.x + (position.width - lw)/2;
        int py = (iType!=T_GHOST)?(position.y + position.height - lh):position.y;

        int fade = (waiting>_MAX_WAITING-100)?(_MAX_WAITING-waiting):100;
        
        bmp1->setAlpha(fade);
        g.drawSprite(bmp1,px,py,false,false);
    }
    return true;
}

void TSpriteEnemigo::drawShadow(TMABitmap& b,TMAVector<TSpriteAnim>* lights)
{
    switch (iType)
    {
        case T_GHOST:
            break;
            
        default:
            TSpriteSpirits::drawShadow(b,lights);
    }
}


#undef _DELTA_X
#undef _MAX_WAITING

#endif
