#ifndef _TSPRITENEEMIGOALEATORIO_CPP_
#define _TSPRITENEEMIGOALEATORIO_CPP_

#include "sprites.hpp"
#include "../spirits.hpp"

#ifndef TARGET_GP2X
    #define _DELTA_X    6
    #define _DELTA_Y    4
#else
    #define _DELTA_X    12
    #define _DELTA_Y    8
#endif

REGISTER_CLASS_ID(TSpriteEnemigoAleatorio);

int TSpriteEnemigoAleatorio::ia_AnimCat[4]={20,21,22,21};
int TSpriteEnemigoAleatorio::ia_AnimArc[16]={44,44,44,44, 45,45,45,45, 46,46,46,46, 46,46,46,46};
int TSpriteEnemigoAleatorio::ia_AnimArcF[16]={46,46,46,46, 45,45,45,45, 44,44,44,44, 44,44,44,44};

TSpriteEnemigoAleatorio::TSpriteEnemigoAleatorio(PMAImageBank bank,TSpriteEnemigoAleatorio::Type t):TSpriteSpirits(bank)
{
    invalidate();
    itype = t;
    fade=0;
    setState(S_NULL);
    sprArrow=NULL;

    switch ((int)t)
    {
        case T_CAT:
            ii_Retardo=2;
            setAnimSequence(ia_AnimCat,4);
            break;

        case T_ARC:
            ii_Retardo=1;
            sprArrow = new TSpriteAnim(bank,TSpriteAnim::T_ARROW);
            break;
    }
}

TSpriteEnemigoAleatorio::~TSpriteEnemigoAleatorio()
{
    if (sprArrow) delete sprArrow;
}

void TSpriteEnemigoAleatorio::init(int scrid)
{
    int px,py;
    
    invalidate();

    // En determinadas pantallas no debe aparecer este tipo de enemigo
    //
    if (scrid==26 || scrid==27 || scrid==28 || scrid==72 ) return;

    // En el resto si..
    //
    setScreenID(scrid);

    // Lo primero es determinar de forma aleatoria con una probabilidad
    // de que el personaje aparezca.
    //
    switch ((int)itype)
    {
        case T_CAT: // Prob, 5%
            if (TMARandom::nextInt(100)<95) return;
            break;
            
        case T_ARC: // Prob, 10%
            if (TMARandom::nextInt(100)<90) return;
            break;
    }
    
    // Ahora determinamos cuantos ciclos ha de estar esperando antes de
    // entrar en accin, entre 20 y 40 ciclos.
    //
    waitcount=0;
    wait=20+TMARandom::nextInt(20);
    
    // El enemigo aparecera por el lado opuesto al que se encuentre
    // el brujo
    //
    setFlipX(target->getHotspotMapX()<12);
    
    // Determinamos la localizacin
    //
    switch ((int)itype)
    {
        case T_CAT:

            if (ib_flip_x) px=22; else px=1;

            for (py=10;py>1;py--)
            {
                if (io_Screen->getMask(px,py) != MASK_WALL) break;
            }
            
            // Caso que no se haya podido localizar una ubicacin valida
            if (py<2)
            {
                invalidate();
                return;
            }
            
            if (ib_flip_x && !canWalkLeft(px,py))
            {
                invalidate();
                return;
            }

            if (!canWalkRight(px,py))
            {
                invalidate();
                return;
            }
            putAt(px,py);
            break;

        case T_ARC:

            if (ib_flip_x)
            {
                setAnimSequence(ia_AnimArcF,16);
                px=21;
            }
            else
            {
                setAnimSequence(ia_AnimArc,16);
                px=2;
            }
            py=target->getHotspotMapY()-1;

            // Si el brujo no esta a una altura determinada, no aparece.
            //
            if (py<6)
            {
                invalidate();
                return;
            }

            putAt(px,py);
            sprArrow->position.x = position.x;
            sprArrow->position.y = position.y+30;
            sprArrow->setFlipX(ib_flip_x);
            setState(ARC_IN);
            break;
    }
    
    // Activamos el sprite
    //
    validate();
}

void TSpriteEnemigoAleatorio::setState(State e)
{
    ii_Estado = e;

    switch (ii_Estado)
    {
        case ARC_IN:
            fade=0;
            break;
            
        case ARC_SHOOTING:
            ii_Frame=0;
            break;
            
        case ARC_SHOOT:
            sprArrow->validate();
            myGame->playSfx(sounds->snd_arrow);
            break;
    }
}

void TSpriteEnemigoAleatorio::doControllCat(void)
{
    // Calculamos el HOTSPOTX,Y
    //
    int px = getHotspotMapX();
    int py = getHotspotMapY();

    // Si se sale de la pantalla, hemos terminado
    //
    if (roomChanged(px,py))
    {
        invalidate();
        return;
    }

    if (target && target->collisionsWith(*this))
    {
        target->setEnergyDown(1);
    }

    if (ib_flip_x)
    {
        // Si no se puede mover a la izquierda, hemos terminado.
        //
        if (!canWalkLeft(px,py))
        {
            invalidate();
            return;
        }
        
        // Mover a la izquierda
        //
        hotspotX -= _DELTA_X;
    }
    else
    {
        // Si no se puede mover a la derecha, hemos terminado.
        //
        if (!canWalkRight(px,py))
        {
            invalidate();
            return;
        }

        // Mover a la derecha
        //
        hotspotX += _DELTA_X;
    }
    
    // Si no hay suelo, que vaya cayendo.
    //
    if (!isDown(px,py))
    {
        hotspotY += _DELTA_Y;
    }
    else correctPositionY();
}

void TSpriteEnemigoAleatorio::doControllArc(void)
{
    // Mientras esta apareciendo...
    //
    switch (ii_Estado)
    {
        case ARC_IN:
					#ifndef TARGET_GP2X
						if (fade+8<255)
					#else
						if (fade+16<255)
					#endif
						{
						#ifndef TARGET_GP2X
							fade+=8;
						#else
							fade+=16;
						#endif
              return;
            }
            fade=255;
            setState(ARC_SHOOTING);
            break;
            
        case ARC_SHOOTING:

            if (ii_Frame>11)
            {
                setState(ARC_SHOOT);
            }
            break;

        case ARC_SHOOT:

            sprArrow->animate();
            if (!sprArrow->isValid()) setState(ARC_OUT);
            else
            {
                if (target->isCollisioningWith(*sprArrow))
                {
                    target->setEnergyDown(2);
                }
            }
            break;

        case ARC_OUT:

						#ifndef TARGET_GP2X
							if (fade-8>0)
							{
							    fade-=8;
							    return;
							}
						#else
							if (fade-16>0)
							{
							    fade-=16;
							    return;
							}
						#endif

            fade=0;
            invalidate();
            break;
    }
}

void TSpriteEnemigoAleatorio::drawShadow(TMABitmap& b,TMAVector<TSpriteAnim>* lights)
{
    if (itype!=T_CAT) return;
    
    // Si el sprite esta invalidado, no se pinta.
    //
    if (!isValid()) return;

    // Si aun esta esperando, no hacemos nada
    //
    if (waitcount<wait) return;

    TSpriteSpirits::drawShadow(b,lights);
}

bool TSpriteEnemigoAleatorio::draw(TMABitmap& g)
{
    // Si el sprite esta invalidado, no se pinta.
    //
    if (!isValid()) return false;

    // Si aun esta esperando, no hacemos nada
    //
    if (waitcount<wait) return false;

    switch ((int)itype)
    {
        case T_CAT:
            g.drawSprite(io_images->getBitmap(ii_CurrentFotograma),position.x,position.y,ib_flip_x,ib_flip_y);
            break;
            
        case T_ARC:
            
            io_images->getBitmap(ii_CurrentFotograma)->setAlpha(fade);
            g.drawSprite(io_images->getBitmap(ii_CurrentFotograma),position.x,position.y,ib_flip_x,ib_flip_y);

            switch (ii_Estado)
            {
                case ARC_IN:
                case ARC_SHOOTING:
                    io_images->getBitmap(23)->setAlpha(fade);
                    if (ib_flip_x)
                        g.drawSprite(io_images->getBitmap(23),position.x-30,position.y+30,ib_flip_x,ib_flip_y);
                    else
                        g.drawSprite(io_images->getBitmap(23),position.x+50,position.y+30,ib_flip_x,ib_flip_y);
                    break;
                    
                case ARC_SHOOT:
                    sprArrow->draw(g);
                    break;
            }
            
            break;
    }
    return true;
}

void TSpriteEnemigoAleatorio::animate(void)
{
    // Si el sprite esta invalidado, no se anima.
    //
    if (!isValid()) return;
    
    // Si aun esta esperando, incrementamos contador, pero no hacemos nada
    //
    if (waitcount<wait)
    {
        waitcount++;
        return;
    }
    else
    {
        if (waitcount==wait)
        {
            switch ((int)itype)
            {
                case T_CAT:
                    myGame->playSfx(sounds->snd_cat);
                    break;

                case T_ARC:
                    myGame->playSfx(sounds->snd_archier);
                    break;
            }
            waitcount++; // Para que solo se haga una vez.
        }
    }

    switch ((int)itype)
    {
        case T_CAT:
            doControllCat();
            break;
            
        case T_ARC:
            doControllArc();
            break;
    }

    if (itype==T_CAT || (itype==T_ARC && ii_Estado==ARC_SHOOTING) )
    {
    	if (ii_RetardoPos<ii_Retardo)
    	{
    		ii_RetardoPos ++;
    	}
    	else
    	{
    		ii_Frame = (ii_Frame<ii_AnimSequenceLength-1)?(ii_Frame+1):0;
    		ii_RetardoPos=0;
    	}
        TSpriteSpirits::animate();
    }
}

#endif
