#ifndef _TSPRITELAMP_CPP_
#define _TSPRITELAMP_CPP_

#include "sprites.hpp"
#include "../spirits.hpp"

REGISTER_CLASS_ID(TSpriteLamp);

int TSpriteLamp::ia_AnimLamp[1]={62};
int TSpriteLamp::ia_AnimRock[1]={23};

TSpriteLamp::TSpriteLamp(PMAImageBank bank,TSpriteLamp::Type t):TSpriteSpirits(bank)
{
    ii_Retardo=2;
    itype = t;
    if (t==T_LAMP)
        setAnimSequence(ia_AnimLamp,1);
    else
        setAnimSequence(ia_AnimRock,1);
        
    setState(E_WAITING);
}

void TSpriteLamp::setState(State e)
{
    ii_Estado = e;
}

void TSpriteLamp::setInit(int ptime,int vel)
{
    timewait = ptime; inittime=ptime; velocity = vel;
}

void TSpriteLamp::reset(void)
{
    putAt(initx,inity);
    timewait = inittime;
    setState(E_WAITING);
}

void TSpriteLamp::doControll(void)
{
    int px,py;
    bool lbtrue;
    
    switch (ii_Estado)
    {
        case E_WAITING:
            // La lampara solo cae cuando esta en la misma habitacin
            // que el sprite target, o ya ha empezado a caer.
            //
            if ((!target) || (target->getScreenID()!=screenID)) return;
            timewait--;
            if (timewait<=0)
            {
                setState(E_FALLING);
                if (itype==T_ROCK) myGame->playSfx(sounds->snd_rockdown);
            }
            break;
            
        case E_FALLING:

            if (target && target->collisionsWith(*this)) target->setEnergyDown(10);

            // Calculamos el HOTSPOTX,Y
            //
            px = getHotspotMapX();
            py = getHotspotMapY();
            
            lbtrue = ((itype == T_ROCK) && (py<4));
            if (lbtrue || (!isDown(px,py,velocity)) )
            {
                hotspotY += velocity;
            }
            else
            {
            		correctPositionY();
                setState(E_FINISH);
                if ((itype==T_LAMP) && (target->getScreenID()==screenID)) myGame->playSfx(sounds->snd_lampdown);
            }
            break;
            
        case E_FINISH:

            //if (target && target->collisionsWith(*this)) target->setEnergyDown();
            break;
    }
}

bool TSpriteLamp::draw(TMABitmap& g)
{
    switch (itype)
    {
        case T_ROCK:
            g.draw(io_images->getBitmap(ii_CurrentFotograma),position.x,position.y);
            break;

        default:
            g.drawSprite(io_images->getBitmap(ii_CurrentFotograma),position.x,position.y,ib_flip_x,ib_flip_y);
    }
    return true;
}

void TSpriteLamp::drawShadow(TMABitmap& b,TMAVector<TSpriteAnim>* lights)
{
    switch (itype)
    {
        case T_ROCK:
            break;

        default:
            TSpriteSpirits::drawShadow(b,lights);
    }
}

void TSpriteLamp::putAt(int ppx,int ppy)
{
    initx = ppx; inity = ppy;
    
    int px = ppx * _TileAX;
    int py = ppy * _TileAY;
    
    // Obtenemos la imagen del fotograma actual y a partir de ah
    // con su tamao calculamos la X e Y reales.
    //
    PMABitmap bmp = io_images->getBitmap(ii_CurrentFotograma);
    if (bmp)
    {
        position.width = bmp->getWidth();
        position.height = bmp->getHeight();
        #ifdef SMALL_GAME
            position.width<<=1;
            position.height<<=1;
        #endif
        position.x = px;
        position.y = py;

        hotspotX = px + position.width / 2;
        hotspotY = py + position.height - 1;
    }
}

void TSpriteLamp::animate(void)
{
    doControll();
    
	if (ii_RetardoPos<ii_Retardo)
	{
		ii_RetardoPos ++;
	}
	else
	{
		ii_Frame = (ii_Frame<ii_AnimSequenceLength-1)?(ii_Frame+1):0;
		ii_RetardoPos=0;
	}
	TSpriteSpirits::animate();
}

#endif
