#ifndef _TSPRITESPIRITS_CPP_
#define _TSPRITESPIRITS_CPP_

#include "sprites.hpp"
#include "../spirits.hpp"
//#include <math.h>

REGISTER_CLASS_ID(TSpriteSpirits);

PSpriteBrujo TSpriteSpirits::target=NULL;
PMAImageBank TSpriteSpirits::bankShadow=NULL;
roomInfo *TSpriteSpirits::rooms=NULL;
gameSound *TSpriteSpirits::sounds=NULL;

TSpriteSpirits::TSpriteSpirits(PMAImageBank bank):TMASprite(bank)
{
    ib_Dying=false;
    screenID=0;
    io_Screen=NULL;
    myGame = (PSpiritsGame)TMAGame::getCurrentGame();
    setFlashTime(0);
}


void TSpriteSpirits::setScreenID(int id)
{
    screenID = id;
    io_Screen = rooms[id].scr;
}

// Indica si hay techo por encima del HOTSPOT
//
bool TSpriteSpirits::isTop(int px,int py)
{
    // PX,PY son el HOTSPOT del Sprite, situado en el centro
    // de la base del sprite.
    //
    int mid = position.width/2/_TileAX;
    int ay = position.height/_TileAY;
    
	for (int i=1-mid;i<mid;i++)
	{
        if (py-ay>=0)
        {
            int mask = io_Screen->getMask(px+i,py-ay);
            evalMask(mask); // Solo tiene sentido en la TSpriteBrujo
            if (mask == MASK_WALL) return true;
        }
	}

	return false;
}

// Indica si hay suelo por debajo del HOTSPOT
//
bool TSpriteSpirits::isDown(int px,int py,int delta)
{
    bool lb_resp=false;
    
    // PX,PY son el HOTSPOT del Sprite, situado en el centro
    // de la base del sprite.
    //
    int mid = position.width/2/_TileAX;
    int li_rest = (hotspotY%_TileAY);
    
    for (int i=1-mid;i<mid;i++)
	{
        int val = io_Screen->getMask(px+i,py+1);
		if (val == MASK_WALL || val == MASK_FLOOR || val == MASK_STAIRS || val == MASK_DIE ||
            (val == MASK_CLOSE && !rooms[screenID].wall_open) )
        {
            lb_resp=true;
            break;
        }
	}
	
	// Si debajo hay suelo, pero aun no estamos cuadrados sobre la posicin Y
	// devolvemos false.
	//
	if (delta>0)
	{
        if (lb_resp && (li_rest+delta <= _TileAY)) return false;
    }
	
	return lb_resp;
}

// Indica si estamos sobre una escalera
//
bool TSpriteSpirits::isStairs(int px,int py)
{
    if (py>11) return true;
    
    // PX,PY son el HOTSPOT del Sprite, situado en el centro
    // de la base del sprite.
    //
    int mid = position.width/2/_TileAX;

	for (int i=1-mid;i<mid;i++)
	{
        int val = io_Screen->getMask(px+i,py);
		if (val == MASK_STAIRS) return true;
	}
	return false;
}


// Por defecto se puede caminar a si no chocamos
// con un muro o con una pared cerrada por una palanca
//
bool TSpriteSpirits::evalMask(int mask)
{
    if (mask==MASK_WALL) return false;
    if (mask==MASK_CLOSE && !rooms[screenID].wall_open) return false;
    return true;
}

// Indica si es posible caminar a la derecha a partir del HOTSPOT
// situado en la centro de la parte inferior.
//
bool TSpriteSpirits::canWalkRight(int px,int py)
{
    // La comprobacin con px=23, solo para el brujo.
    if (this->isInstanceOf(CLASS_ID(TSpriteBrujo)))
    {
        if (px<0 || px>=23) return true;
    }
    else if (px<0 || px>=22) return true;

    int lMask,i;
    int mid = position.width/2/_TileAX;
    int ay = position.height/_TileAY;

    for (i=0;i<ay;i++)
    {
        lMask=io_Screen->getMask(px+mid,py-i);
        if (!evalMask(lMask)) return false;
    }
	return true;
}

// Indica si es posible caminar a la izquierda a partir del HOTSPOT
// situado en la centro de la parte inferior.
//
bool TSpriteSpirits::canWalkLeft(int px,int py)
{		
    if (px<0 || px>=23) return true;

    int lMask,i;
    int mid = position.width/2/_TileAX;
    int ay = position.height/_TileAY;

    for (i=0;i<ay;i++)
	{
        lMask=io_Screen->getMask(px-mid,py-i);
        if (!evalMask(lMask)) return false;
	}
	return true;
}

int *TSpriteSpirits::getKillSequence(int& length)
{
    length = 0;
    return NULL;
}

bool TSpriteSpirits::kill(void)
{
	if (!ib_Dying)
	{
		ii_Frame=0;
		ib_Dying = true;
		ii_Retardo=3;
		ia_AnimSequence = getKillSequence(ii_AnimSequenceLength);
		ib_flip_x = false;
	}
 else
	{
        if ((ii_Frame+1)==ii_AnimSequenceLength)
		{
			ii_Retardo=2;
			invalidate();
            return false;
		}
	}
    return true;
}

// Detecta una colision con otro Sprite
//
bool TSpriteSpirits::collisionsWith(TSpriteSpirits& spr)
{
    if (ib_Dying) return false;
    return position.intersects(spr.position);
}

// Asumiendo X,Y como coordenadas dentro de la reticula de una pantalla
// calcula a partir de estas coordenadas el correcto HOTSPOT (centro de la base)
// y situa ah el sprite
//
void TSpriteSpirits::putAt(int x,int y)
{
    // Convertimos X e Y en coordenadas reales en lugar de reticulas
    //
    x =x *_TileAX + (_TileAX/2);
    y =y *_TileAY + (_TileAY-1);

    // Obtenemos la imagen del fotograma actual y a partir de ah
    // con su tamao calculamos la X e Y reales.
    //
    PMABitmap bmp = io_images->getBitmap(ii_CurrentFotograma);
    if (bmp)
    {
        #ifdef SMALL_GAME
            position.width = bmp->getWidth()*2;
            position.height = bmp->getHeight()*2;
        #else
            position.width = bmp->getWidth();
            position.height = bmp->getHeight();
        #endif
        position.x = x - position.width/2;
        position.y = y - position.height;
    }
    
    hotspotX = x;
    hotspotY = y;
}

void TSpriteSpirits::animate(void)
{
    TMASprite::animate();

    // Calculamos la posicion del rectangulo de dibujo en funcin del fotograma
    //
    PMABitmap bmp = io_images->getBitmap(ii_CurrentFotograma);
    if (bmp)
    {
        #ifdef SMALL_GAME
            position.width = bmp->getWidth()*2;
            position.height = bmp->getHeight()*2;
        #else
            position.width = bmp->getWidth();
            position.height = bmp->getHeight();
        #endif
            
        position.x = hotspotX - position.width/2;
        position.y = hotspotY - position.height;
    }
}

bool TSpriteSpirits::draw(TMABitmap& b)
{
    if (ii_flashtime>0)
    {
        if (!(ii_flashtime % 2)) return true;
    }
    return TMASprite::draw(b);
}

void TSpriteSpirits::setFlashTime(int t)
{
    ii_flashtime = (t>0)?t:0;
}

void TSpriteSpirits::drawShadow(TMABitmap& g,TMAVector<TSpriteAnim>* lights)
{
    // En las 3 pantallas exteriores, no debe proyectarse la sombra
    //
    if (screenID>=26 && screenID<=28) return;

    int px = 0;
    if (lights!=NULL)
    {
        // Calculamos la distancia y la posicin con respecto a cada luz
        // y luego calculamos la media.
        //
        for (int i=0;i<lights->size();i++)
        {
            PSpriteAnim sa = (PSpriteAnim)lights->elementAt(i);
            int ldist = abs((sa->position.x + sa->position.width/2) - (position.x + position.width/2));

            if (position.x + position.width/2 < sa->position.x+sa->position.width/2)
                px -= (ldist/8);
            else
                px += (ldist/8);
        }
        px/=lights->size();
        px += position.x;
    }
    else px = position.x-15;

    drawShadowed(g,px);
}

void TSpriteSpirits::drawShadowed(TMABitmap& g,int px)
{
    g.drawSprite(bankShadow->getBitmap(ii_CurrentFotograma),px,position.y,ib_flip_x,ib_flip_y);
}

bool TSpriteSpirits::roomChanged(int px,int py)
{
    // Controlamos el paso de pantalla
    //
    if (px<1)
    {
        enterLeft();
        return true;
    }
    if (py<3)
    {
        enterUp();
        return true;
    }
    if (px>22)
    {
        enterRight();
        return true;
    }
    if (py>12)
    {
        enterDown();
        return true;
    }
    return false;
}


void TSpriteSpirits::enterLeft(void)
{
    if (TSpiritsGame::gameMapInfo[screenID].left>0)
    setScreenID(TSpiritsGame::gameMapInfo[screenID].left);
    putAt(22,getHotspotMapY());
}

void TSpriteSpirits::enterRight(void)
{
    if (TSpiritsGame::gameMapInfo[screenID].right>0)
    setScreenID(TSpiritsGame::gameMapInfo[screenID].right);
    putAt(1,getHotspotMapY());
}

void TSpriteSpirits::enterUp(void)
{
    // Aqui ajustamos que en ciertas pantallas, al subir hacia arriba
    // hemos de aparecer un poco ms alto, para poder estar sobre una plataforma
    //
    bool lb_upMore = true; //(screenID==80 || screenID==18);
    if (TSpiritsGame::gameMapInfo[screenID].up>0)
    setScreenID(TSpiritsGame::gameMapInfo[screenID].up);
    putAt(getHotspotMapX(),(lb_upMore)?10:12);
}

void TSpriteSpirits::enterDown(void)
{
    if (TSpiritsGame::gameMapInfo[screenID].down>0)
    setScreenID(TSpiritsGame::gameMapInfo[screenID].down);
    putAt(getHotspotMapX(),3);
}


#endif
