#ifndef _TMAGAME_CPP_
#define _TMAGAME_CPP_

#ifdef WIN32
    #include <windows.h>
#endif

#include "ma_classes.hpp"
#include "../../tinyxml/tinyxml.h"
#include "../tad/tmavector.cpp"
#include <ctype.h>
#include <string.h>
#include <unistd.h>

// **************************************
// Definicin de mtodos de clase TMAGame
// **************************************
//
REGISTER_CLASS_ID(TMAGame);

PMAGame TMAGame::oc_MAGame=NULL;
bool TMAGame::KeyState[TMAGame::KEY_MAX];
bool TMAGame::ci_endgame=false;
bool TMAGame::ib_soundAvailable=false;
bool TMAGame::ib_joystickAvailable=false;
long TMAGame::il_time_millis=0;
int  TMAGame::ii_last_ascii_key=0;
int  TMAGame::ARGC=0;
char **TMAGame::ARGV=NULL;
SDL_Surface *TMAGame::ScreenSurf=NULL;
SDL_Surface *TMAGame::BackScreenSurf=NULL;
PMABitmap TMAGame::Screen=NULL;
PMABitmap TMAGame::BackScreen=NULL;
TMAInputController TMAGame::inputController;
TMAStringDictionary TMAGame::langTable;

TMAGame::TMAGame(void)
{
    if (oc_MAGame!=NULL)
    {
        exit(0);
    }
    oc_MAGame = this;
    Screen = NULL;    
    ScreenSurf = NULL;
    BackScreen = NULL;
    BackScreenSurf = NULL;
    io_current_scene=NULL;
    io_next_scene=NULL;
    ib_remove_last_scene=false;
    releaseKeys();
    setGameFPS(0);
}

TMAGame::~TMAGame()
{
    cleanup();
}

bool TMAGame::init(void)
{
    setMouseVisible(false);
    return true;
}

void TMAGame::setMouseVisible(bool v)
{
    SDL_ShowCursor((v)?SDL_ENABLE:SDL_DISABLE);
}

bool TMAGame::isMouseVisible(void)
{
    return SDL_ShowCursor(SDL_QUERY);
}

const char *TMAGame::getTranslationFor(const char *str)
{	
    TMAStringDictionaryEntry strEntry;
    strEntry.key = str;
    
    long index = langTable.findElementIndex(&strEntry);
    if (index>0)
    {
        return langTable.elementAt(index)->value.text();
    }
    
    // En caso de no encontrar el texto, devolvemos la misma cadena
    //
    return str;
}

void TMAGame::setupLanguage(const char *domain,const char *locale)
{
   // Obtenemos el directorio del repositorio de idioma
   //
    char buff[1024],filelang[1024];
    memset(buff,0,sizeof(buff));

    getcwd(buff,1024);
    sprintf(filelang,"%s/data/lang/%s/%s.xml",buff,locale,domain);
    for (int i=0;*(filelang+i)!=0;i++) if (*(filelang+i)=='\\') *(filelang+i)='/';
   
    // Ahora cargamos el archivo de idioma
    //
    langTable.clear();
  
    TiXmlDocument doc( filelang );

    if (doc.LoadFile())
    {
        // Obtenemos el nivel "translations"
        //
        TiXmlElement* entry = doc.FirstChildElement("translations");
        if (entry) entry = entry->FirstChildElement("entry");
        
        while (entry)
        {
            TiXmlElement* e_id = entry->FirstChildElement("msgid");
            TiXmlElement* e_str = entry->FirstChildElement("msgstr");
            
            TMAStringDictionaryEntry *strEntry = new TMAStringDictionaryEntry();
            if (strEntry)
            {
                strEntry->key = e_id->GetText();
                strEntry->value = e_str->GetText();
                langTable.addElement(strEntry);
            }
            
            entry = entry->NextSiblingElement( "entry" );
        }
    }
	
}

void TMAGame::releaseKeys(void)
{
    memset(KeyState,0,sizeof(KeyState));
    releaseLastAsciiKeyPressed();
    inputController.clearStatus();
}

int TMAGame::isAnyKeyPressed(void)
{
    for (unsigned int i=0;i<sizeof(KeyState);i++)
    {
        if (KeyState[i]) return i;
    }
    return 0;
}

void TMAGame::setGameFPS(int fps)
{
    ii_game_speed = (fps<=0)?0:(1000/fps);
}

PMAGame TMAGame::getCurrentGame(void)
{
    return oc_MAGame;
}

long TMAGame::getCPUTime(void)
{
    return SDL_GetTicks();
}

void TMAGame::sleep(int ms)
{
   SDL_Delay(ms);
}

void TMAGame::showAlert(const char *txt)
{
    #ifdef WIN32
        MessageBox(NULL,txt,"Alert",0);
    #endif
}

void TMAGame::showLastAlert()
{
    #ifdef WIN32
        MessageBox(NULL,"LastError","<undefined>",0);
    #endif
}

void TMAGame::showFatalError(const char *txt)
{
    #ifdef WIN32
        MessageBox(NULL,txt,"Alert",0);
    #endif
    endGame();
}

void TMAGame::setWindowTitle(const char *txt)
{
    #if !defined(TARGET_GP2X) && !defined(TARGET_WII)
        SDL_WM_SetCaption(txt,NULL);
    #endif
}

void TMAGame::getScreenSize(int& width,int& height)
{
    SDL_Surface *lscr = (ScreenSurf!=NULL)?ScreenSurf:SDL_GetVideoSurface();

    width = lscr->w;
    height = lscr->h;
}

int TMAGame::getScreenWidth(void)
{
    SDL_Surface *lscr = (ScreenSurf!=NULL)?ScreenSurf:SDL_GetVideoSurface();
    return lscr->w;
}

int TMAGame::getScreenHeight(void)
{
    SDL_Surface *lscr = (ScreenSurf!=NULL)?ScreenSurf:SDL_GetVideoSurface();
    return lscr->h;
}



void TMAGame::flipScreen(void)
{
    Screen->draw(BackScreen,0,0);
    SDL_UpdateRect(ScreenSurf, 0, 0, 0, 0);
}
 
bool TMAGame::setGraphicsMode(int width,int height,int colordepth,bool fullscreen)
{    
    Uint32 flags =  SDL_HWSURFACE;

    #if defined(TARGET_WII) || defined(TARGET_GP2X)
        flags|=SDL_FULLSCREEN;
    #else
        if (fullscreen) flags|=SDL_FULLSCREEN;
    #endif
    
    if (Screen!=NULL)
    {
        delete Screen;
        Screen=NULL;
        if (ScreenSurf!=NULL) 
        {
            SDL_FreeSurface(ScreenSurf);
            ScreenSurf=NULL;
        }
    }

    if (BackScreen!=NULL)
    {
        delete BackScreen;
        BackScreen=NULL;
        if (BackScreenSurf!=NULL)
        {
            SDL_FreeSurface(BackScreenSurf);
            BackScreenSurf=NULL;
        }
    }
    
    ScreenSurf = SDL_SetVideoMode(width, height, colordepth, flags);
    BackScreenSurf = SDL_CreateRGBSurface(SDL_HWSURFACE, width, height, colordepth,0,0,0,0);
    
    if (ScreenSurf!=NULL && BackScreenSurf!=NULL)
    {
        Screen = new TMABitmap(ScreenSurf,true);
        BackScreen = new TMABitmap(BackScreenSurf,true);
    }
    else
    {
        if (ScreenSurf!=NULL) SDL_FreeSurface(ScreenSurf);
        if (BackScreen!=NULL) SDL_FreeSurface(BackScreenSurf);
        ScreenSurf=NULL; BackScreenSurf=NULL;
        return false;
    }
    
    return true;
}

int TMAGame::getScreenDepth(void)  { return ScreenSurf->format->BitsPerPixel; }
        
bool TMAGame::isFullScreen()
{
    if (Screen==NULL) return false;
    return (ScreenSurf->flags & SDL_FULLSCREEN);
}

bool TMAGame::setFullScreen(bool fs)
{
    return setGraphicsMode(getScreenWidth(),getScreenHeight(),getScreenDepth(),fs);
}
        
void TMAGame::cleanup(void)
{    
	if (Screen != NULL) 
    {
        delete Screen;
        Screen = NULL;
    }
    
    if (BackScreen != NULL)
    {
        delete BackScreen;
        BackScreen = NULL;
    }
}

void TMAGame::endGame(void)
{
    ci_endgame = true;
}



int TMAGame::getLastAsciiKeyPressed(void)
{
    return ii_last_ascii_key;
}

void TMAGame::releaseLastAsciiKeyPressed(void)
{
    ii_last_ascii_key=0;
}

void TMAGame::setVolume(int v)
{
    Mix_Volume(-1,v);
    Mix_VolumeMusic(v);
}

void TMAGame::run(void)
{    
    bool lbRun=true;
    long startTime=0;
    SDL_Event event;
    SDL_ShowCursor(SDL_DISABLE);
    
    // ---- CICLO PRINCIPAL DEL JUEGO
    //
    while (!ci_endgame)
    {
        lbRun=false;

        // Invocamos el RUN
        //
        if (ii_game_speed>0) 
        {
            if (getCPUTime()>=startTime)
            {
                startTime = getCPUTime() + ii_game_speed;
                lbRun = true;
            }     
        }
        else lbRun=true;
        
        if (lbRun)
        {        
            updateInputController();
            			
			while (SDL_PollEvent (&event))
			{
				switch (event.type)
				{
					#if !defined (TARGET_GP2X) && !defined (TARGET_WII)
						case SDL_KEYDOWN:
							KeyState[event.key.keysym.sym]=true;
							ii_last_ascii_key = event.key.keysym.unicode;
							break;                        
						case SDL_KEYUP:
							KeyState[event.key.keysym.sym]=false;
							releaseLastAsciiKeyPressed();
							break;
					#endif
					
					case SDL_JOYBUTTONDOWN:
						updateInputController(event);
						break;
						
					case SDL_JOYBUTTONUP:
						updateInputController(event);
						break;
					
					case SDL_QUIT:
						endGame();
						break;
				}
			}			

            if ( (io_current_scene==NULL) || (!io_current_scene->doRun()) ) 
            {
                doRun();
            }
            
            if (io_next_scene==NULL)
			{
				    // Invocamos el Paint
				    //
				if (!io_current_scene) doPaint(*BackScreen);
				else io_current_scene->doPaint(*BackScreen);
				flipScreen();
			}
			else
			{
				// Establecemos la nueva escena, si se cambio.
				//
				if (ib_remove_last_scene && io_current_scene!=NULL) delete io_current_scene;
				io_current_scene = io_next_scene;
				io_next_scene=NULL;
			    continue;
			}       
		}
    }
    //
    // ----------- FIN DE CICLO PRINCIPAL 
}

bool TMAGame::setScene(PMAScene esc,const PMAObject params,bool remove_last)
{
    if (io_next_scene!=NULL)
    {
        delete io_next_scene;
        io_next_scene=NULL;
    }
    
    if (!esc->isInitialized())
    {
        if (!esc->doInit(params)) return false;
        io_next_scene = esc;
        ib_remove_last_scene=remove_last;
    }
    
    return true;
}

// ***************************************
// Definicin de mtodos de clase TMAScene
// ***************************************
//
REGISTER_CLASS_ID(TMAScene);

TMAScene::TMAScene()
{
    ib_initialized=false;
    ii_Estado=0;
}

bool TMAScene::doInit(const PMAObject params)
{
    ib_initialized=true;
    return true;
}

// ------------------------ METODO PRINCIPAL

// Debe implementarse est funcin como punto de entrada
// del programa
//
extern int MainProc(int argc, char** argv);

// Punto de entrada principal, que se redireccionara al metodo MainProc
//
int main(int argc,char **argv)
{
    int ret=-1;
    TMAGame::ARGC = argc;
    TMAGame::ARGV = argv;

    if((SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER)==-1))
    {
        printf("Could not initialize SDL: %s.\n", SDL_GetError());
        goto lab_end;
    }
    TMAGame::setSoundAvailable(true);
    if (SDL_JoystickOpen(0)) TMAGame::setJoystickAvailable(true);

 	#if defined (TARGET_GP2X)
		Mix_OpenAudio(32000, AUDIO_S16SYS, MIX_DEFAULT_CHANNELS, 128); // Initialize SDL_mixer for GP2X, buffer is set lower than PC
	#elif defined (TARGET_WII)
		// Inicializamos el sistema FAT
		//
		if (!fatInitDefault()) goto lab_end;
		WPAD_Init();
		SDL_EnableUNICODE(1);
		Mix_OpenAudio(32000, AUDIO_S16MSB , MIX_DEFAULT_CHANNELS, 2048); 
	#else
		SDL_EnableUNICODE(1);
		Mix_OpenAudio(32000, AUDIO_S16SYS, MIX_DEFAULT_CHANNELS, 2048); // Initialize SDL_mixer for PC or CONSOLE, buffer is set higher
	#endif

	if (TTF_Init()) goto lab_end;

    ret = MainProc(argc,argv);

lab_end:
	SDL_Delay(100);
	Mix_CloseAudio();

	if (TTF_WasInit()) TTF_Quit();

	#ifndef TARGET_GP2X
		SDL_Quit();
	#else
		chdir("/usr/gp2x");
		execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
	#endif

	return ret;
}


#endif
