#ifndef _TMATTFONT_CPP_
#define _TMATTFONT_CPP_

#include "ma_classes.hpp"
#include <string.h>

REGISTER_CLASS_ID(TMATTFont);

TMATTFont::TMATTFont()
{
    font=NULL;
    fontname=NULL;
}

TMATTFont::~TMATTFont()
{
    cleanup();
}

void TMATTFont::cleanup()
{
    if (font!=NULL)
    {
        TTF_CloseFont(font);
        font=NULL;
    }
    
    if (fontname!=NULL)
    {
        delete fontname;
        fontname=NULL;
    }
}

// Carga un tipo de letra.
//
bool TMATTFont::load(const char *fontFile,int size)
{
    #ifdef SMALL_GAME
        size>>=1;
    #endif
    
    cleanup();
    font = TTF_OpenFont(fontFile,size);
    fontname = strdup(fontFile);
    return (font!=NULL);
}

// Establece el tamao del tipo de letra
//
void TMATTFont::setSize(int size)
{
    if (font!=NULL)
    {
        char *fname = strdup(fontname);
        load(fname,size);
        delete fname;
    }
}

// Devuelve la anchura de un texto
//
int TMATTFont::getTextWidth(const char *txt)
{
    int w=0,h;
    if(!TTF_SizeText(font,txt,&w,&h)) 
    {
        #ifdef SMALL_GAME
            w<<=1;
        #endif
        return w;
    }
    return -1;
}

// Devuelve la altura de un texto
//
int TMATTFont::getTextSize(const char *txt,int& w,int& h)
{
    int ret = TTF_SizeText(font,txt,&w,&h);
    #ifdef SMALL_GAME
        w<<=1; h<<=1;
    #endif
    return ret;
}

int TMATTFont::getFontHeight(void)
{
    #ifdef SMALL_GAME
        return (TTF_FontHeight(font)<<1);
    #else
        return TTF_FontHeight(font);
    #endif
}

int TMATTFont::getFontAscent(void)
{
    #ifdef SMALL_GAME
        return (TTF_FontAscent(font)<<1);
    #else
        return TTF_FontAscent(font);
    #endif
}

int TMATTFont::getFontDescent(void)
{
    #ifdef SMALL_GAME
        return (TTF_FontDescent(font)<<1);
    #else
        return TTF_FontDescent(font);
    #endif
}

int TMATTFont::getStyle(void)
{
    return TTF_GetFontStyle(font);
}

void TMATTFont::setStyle(int s)
{
    TTF_SetFontStyle(font,s);
}

PMABitmap TMATTFont::getRenderedText(const char *text,Uint32 color,bool antialias)
{
    SDL_Color lcolor = { GET_REDVALUE(color), GET_GREENVALUE(color), GET_BLUEVALUE(color) };
    SDL_Surface *s = (antialias)?TTF_RenderText_Blended(font, text, lcolor):TTF_RenderText_Solid(font, text, lcolor);
    
    if (!s) return NULL;
    
    PMABitmap bmp = new TMABitmap(s,false);
    if (!bmp) 
    {
        SDL_FreeSurface(s);
        return NULL;
    }
    
    // Si no es antialias, como la SURFACE que crea SDL es de color indexado
    // copiamos el color 1 en el resto de elementos de la paleta, para evitar
    // el efecto de color 0 al aplicar el estilo BOLD.
    //
    if (!antialias)
    {
        for (int i=2;i<s->format->palette->ncolors;i++)
        {
            s->format->palette->colors[i] = s->format->palette->colors[1];
        }
    }
    
    return bmp;
}

void TMATTFont::drawText(TMABitmap& bmp,const char *text,int x, int y,Uint32 color,bool antialias)
{
    PMABitmap bmp1 = getRenderedText(text,color,antialias);
    if (!bmp1) return;
    
    bmp.draw(bmp1,x,y);
    delete bmp1;
}

void TMATTFont::drawTextRight(TMABitmap& bmp,const char *text,int x, int y,Uint32 color,bool antialias)
{
    PMABitmap bmp1 = getRenderedText(text,color,antialias);
    if (!bmp1) return;
    
    bmp.draw(bmp1,x - bmp1->getWidth(), y);
    delete bmp1;
}

void TMATTFont::drawTextCenterOnX(TMABitmap& bmp,const char *text,int x, int y,Uint32 color,bool antialias)
{
    PMABitmap bmp1 = getRenderedText(text,color,antialias);
    if (!bmp1) return;
    
    bmp.draw(bmp1,x - (bmp1->getWidth()/2), y);
    delete bmp1;
}

void TMATTFont::drawTextCenterOnY(TMABitmap& bmp,const char *text,int x, int y,Uint32 color,bool antialias)
{
    PMABitmap bmp1 = getRenderedText(text,color,antialias);
    if (!bmp1) return;
    
    bmp.draw(bmp1,x, y - (bmp1->getHeight()/2));
    delete bmp1;
}

void TMATTFont::drawTextCenterOnXY(TMABitmap& bmp,const char *text,int x, int y,Uint32 color,bool antialias)
{
    PMABitmap bmp1 = getRenderedText(text,color,antialias);
    if (!bmp1) return;
    
    bmp.draw(bmp1,x - (bmp1->getWidth()/2), y - (bmp1->getHeight()/2));
    delete bmp1;
}

#endif
