#ifndef _TMAVECTOR_CPP_
#define _TMAVECTOR_CPP_

#include "tmavector.hpp"
#include "string.h"

template <typename ITYPE>REGISTER_CLASS_ID(TMAVectorBase<ITYPE>);
template <typename ITYPE>REGISTER_CLASS_ID(TMAVector<ITYPE>);
template <typename ITYPE>REGISTER_CLASS_ID(TMAItemComparator<ITYPE>);
template <typename ITYPE>REGISTER_CLASS_ID(TMAVectorOrdered<ITYPE>);

// *******************
// Clase TMAVectorBase
// *******************
//
template <typename ITYPE> TMAVectorBase<ITYPE>::TMAVectorBase(int elems,int grow,bool delete_items)
{
    il_initsize = elems;
    il_vsize = elems;
    ii_ngrow = grow;
    CItems = NULL;
    ib_delete_items=delete_items;
    clear();
}

template <typename ITYPE>TMAVectorBase<ITYPE>::~TMAVectorBase()
{
    if (CItems!=NULL)
    {
        clear();
        delete CItems;
        CItems=NULL;
    }
}

template <typename ITYPE> void TMAVectorBase<ITYPE>::delete_objects(void)
{
    // Si el vector no requiere que se eliminen los items... salimos
    //
    if (!ib_delete_items || CItems==NULL) return;
    
    // Eliminamos cada item
    //
    for(int i =  0; i < il_count; i++ )
    {
        if (CItems[i]!=NULL)
        {
            delete (CItems[i]);
            CItems[i]=NULL;
        }
    }
}

template <typename ITYPE> void TMAVectorBase<ITYPE>::swaps_element(long index1,long index2)
{
    ITYPE *T;
    if (index1>=0 && index2>=0 && index1<il_count && index2<il_count && index1!=index2)
    {
        T=CItems[index1];
        CItems[index1]=CItems[index2];
        CItems[index2]=T;
    }        
}

template <typename ITYPE> void TMAVectorBase<ITYPE>::clear(void)
{
    delete_objects();
    if (CItems) delete CItems;
    CItems = new ITYPE*[il_initsize];
    for (int i=0;i<il_initsize;i++) CItems[i]=NULL;
    il_vsize=il_initsize;
    il_count=0;
}

template <typename ITYPE>void TMAVectorBase<ITYPE>::grow_buffer(bool grow)
{
    long i;
    ITYPE **aItems;
    
    if (grow) il_vsize += ii_ngrow;
    aItems = new ITYPE*[il_vsize];
    for(i=0;i<il_vsize;i++) aItems[i]=NULL;
    
    if( il_count !=  0 )
    {
        memcpy( &aItems[0], &CItems[0], il_count*sizeof(ITYPE*) );
    }
    delete CItems;
    CItems = aItems;
}

template <typename ITYPE>void TMAVectorBase<ITYPE>::redim(long initsize)
{
    il_initsize = initsize;
    pack();
}

template <typename ITYPE>void TMAVectorBase<ITYPE>::pack(void)
{
    il_vsize = (il_count / ii_ngrow);
    if ((il_count % ii_ngrow)>0) il_vsize++;
    il_vsize*=ii_ngrow;
    if (il_vsize<il_initsize) il_vsize=il_initsize;
    grow_buffer(false);
}

template <typename ITYPE>bool TMAVectorBase<ITYPE>::exists(ITYPE *obj)
{
    return (findElementIndex(obj)>=0);
}

template <typename ITYPE>bool TMAVectorBase<ITYPE>::removeElement(ITYPE *obj,bool delobject)
{
    long index = findElementIndex(obj);
    if (index>=0)
    {
        return removeElementAt(index,delobject);
    }
    return false;
}

template <typename ITYPE>bool TMAVectorBase<ITYPE>::removeElementAt(long index,bool delobject)
{
    if( index < 0 || index >= il_count ) return false;

    ITYPE *obj = CItems[index];    
    memmove( &CItems[index], &CItems[index+1], (il_count-index-1)*sizeof(ITYPE*) );
    CItems[--il_count]=NULL;
    
    if (ib_delete_items && delobject) delete obj;
    return true;
}

template <typename ITYPE>long TMAVectorBase<ITYPE>::add_element(ITYPE *item,long index)
{
    if ( index < 0 ) return -1; // Error.
    if ( il_count == il_vsize ) grow_buffer();

    memmove( &CItems[index+1], &CItems[index], (il_count-index)*sizeof(ITYPE*) );
    il_count++;

    CItems[index] = item;
    return index;
}

template <typename ITYPE>ITYPE*TMAVectorBase<ITYPE>::elementAt(long index)
{
    if( index < 0 || index >= il_count ) return NULL;
    return CItems[index];
}

template <typename ITYPE>long TMAVectorBase<ITYPE>::size()
{
    return this->il_count; 
}

// ***************
// Clase TMAVector
// ***************
//
template <typename ITYPE>void TMAVector<ITYPE>::fillEmpty(long numelems)
{
    while (numelems>0)
    {
        add_element(NULL,this->il_count);
        numelems--;
    }
}

template <typename ITYPE>long TMAVector<ITYPE>::findElementIndex(ITYPE* obj)
{
    for (int i=0;i<this->il_count;i++)
    {
        if (this->CItems[i]==obj)
        {
            return i;
        }
    }
    return -1;
}

template <typename ITYPE>long TMAVector<ITYPE>::addElement(ITYPE *obj)
{
    return add_element(obj,this->size());
}

template <typename ITYPE>long TMAVector<ITYPE>::addElementAt(ITYPE *obj,long index)
{
    return add_element(obj,index);
}

template <typename ITYPE>bool TMAVector<ITYPE>::replaceElementAt(ITYPE *obj,long index)
{
    if (index>=0 && index<this->il_count)
    {
        if (this->ib_delete_items && this->CItems[index]!=NULL) delete this->CItems[index];
        this->CItems[index]=obj;
        return true;
    }
    return false;
}


// **********************
// Clase TMAVectorOrdered
// **********************
//
template <typename ITYPE> TMAVectorOrdered<ITYPE>::~TMAVectorOrdered()
{
    if (io_comp!=NULL) delete io_comp;
}        

template <typename ITYPE> TMAVectorOrdered<ITYPE>::TMAVectorOrdered(TMAItemComparator<ITYPE>* icomp,int elems,int grow,bool delete_items):
    TMAVectorBase<ITYPE>(elems,grow,delete_items)
{
    io_comp=NULL;
    setComparator(icomp);
}

template <typename ITYPE> void TMAVectorOrdered<ITYPE>::setComparator(TMAItemComparator<ITYPE>* icomp)
{
    if (io_comp) delete io_comp;
    io_comp = icomp;
}

template <typename ITYPE> long TMAVectorOrdered<ITYPE>::findElementIndex(ITYPE* obj)
{
    // Si no hay elementos, pues indice invalido.
    //
    if (this->size()==0) return -1;

    // En otro caso buscamos.
    //
    long index_izq=0;
	long index_der=this->il_count-1;
	long index=0;

    while (index_izq<=index_der)
    {
        index = (index_izq + index_der)/2;
        ITYPE* obj1 = this->CItems[index];
        int ret =io_comp->compare(*obj,*obj1);
		if (ret<0)
        {
            index_der=index-1;
        }
		else if (ret==0)
		{
            return index;
        }
		else
		{
            index_izq=index+1;
        }        
    }

    // No existe el elemento.
    //
    return -1;
}

template <typename ITYPE> long TMAVectorOrdered<ITYPE>::addElement(ITYPE* obj)
{
    // Si no hay elementos, insertamos directamente.
    //
    if (this->size()==0) return add_element(obj,0);
    
    // En otro caso ordenamos.
    //
    long index_izq=0;
	long index_der=this->il_count-1;
	long index=0;
    
    while (index_izq<=index_der)
    {
        index = (index_izq + index_der)/2;
        ITYPE* obj1 = this->CItems[index];
        int ret =io_comp->compare(*obj,*obj1);
		if (ret<0)
        {
            index_der=index-1;
        }
		else if (ret==0)
		{
            index_izq=index+1;
        }
		else
		{
            index_izq=index+1;
            index++;
        }        
    }        
    if (index>this->il_count) index=this->il_count;
    if (index<0) index=0;
    
    return add_element(obj,index);
}

#endif
