#ifndef _TAD_H_
#define _TAD_H_

#include "../ma_object.hpp"

// **************************************************
// Clase que implementa un vector dinamico de objetos
// **************************************************
//
template <typename ITYPE> class TMAVectorBase: public TMAObject
{
    REGISTER_CLASS protected:
                
        ITYPE **CItems;
        int ii_ngrow;
        long il_initsize,il_vsize,il_count;
        bool ib_delete_items;
        
        void grow_buffer(bool grow=true);
        void delete_objects();
        void swaps_element(long index1,long index2);
        long add_element(ITYPE*,long index);   
        
    public:
        PUBLIC_CLASS_ID;
        
        // Crea un vector con una determinada capacidad inicial
        // y con un factor de crecimimento.
        //
        TMAVectorBase(int elems=10,int grow=10,bool delete_items=true);
        virtual ~TMAVectorBase();
        
        // Mtodos de manejo de elementos del vector
        //
        void clear(void);
        void pack(void);
        void redim(long initsize);
        bool removeElement(ITYPE*,bool delobject=true);
        bool removeElementAt(long index,bool delobject=true);

        ITYPE* elementAt(long index);
        long size(void);
        bool exists(ITYPE*);
        
        virtual long findElementIndex(ITYPE*)=0;
        virtual long addElement(ITYPE*)=0;
};

// ***************************************
// Clase que implementa un vector generico
// ***************************************
//
template <typename ITYPE> class TMAVector : public TMAVectorBase<ITYPE>
{
    REGISTER_CLASS protected:

    public:
        PUBLIC_CLASS_ID;

        TMAVector(int elems=10,int grow=10,bool delete_items=true):
            TMAVectorBase<ITYPE>(elems,grow,delete_items){};

        // Aade n-elementos vacios tras el ultimo elemento existente.
        //
        void fillEmpty(long numelems);

        // Incluye o Reemplaza elementos
        //
        long addElement(ITYPE*);
        long addElementAt(ITYPE*,long index);
        bool replaceElementAt(ITYPE*,long index);
        
        // Encuentra la posicin de un determinado elemento
        //
        long findElementIndex(ITYPE*);
};

// **************************************************************
// Clase que implementa un interfaz para comparacin de elementos
// **************************************************************
//
template <typename ITYPE> class TMAItemComparator : public TMAObject
{
    REGISTER_CLASS protected:

    public:
        PUBLIC_CLASS_ID;

        // Comparacin de dos objetos.
        // Retorna:
        //      -1 -> o1<o2
        //       0 -> o1=o2
        //       1 -> o1>o2
        //
        virtual signed int compare(ITYPE& o1,ITYPE& o2)=0;
};

// ***************************************
// Clase que implementa un vector ordenado
// ***************************************
//
template <typename ITYPE> class TMAVectorOrdered : public TMAVectorBase<ITYPE> 
{
    REGISTER_CLASS protected:

        TMAItemComparator<ITYPE>* io_comp;
                
    public:
        PUBLIC_CLASS_ID;
                
        TMAVectorOrdered(TMAItemComparator<ITYPE>* icomp=0L,int elems=10,int grow=10,bool delete_items=true);
        ~TMAVectorOrdered();
        
        // Establece el objeto comparador
        //
        void setComparator(TMAItemComparator<ITYPE>*);
        
        // Inserta un elemento en el vector de forma ordenada
        //
        long addElement(ITYPE*);
        
        // Encuentra la posicin de un determinado elemento
        //
        long findElementIndex(ITYPE*);
};

#endif
