#ifndef _TMASTRING_CPP_
#define _TMASTRING_CPP_

#include "tmastring.hpp"
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

REGISTER_CLASS_ID(TMAString);

char *TMAString::NULLSTR="";

void TMAString::init(void)
{
    c_str=NULLSTR;
    l_size=0;
}

void TMAString::free(void)
{
    if (c_str!=NULLSTR)
    {
        delete c_str;
        c_str=NULLSTR;
    }
    l_size=0;
}

void TMAString::copyFrom(const char *src)
{
    if (c_str == src) return;
    
    free();
    if (src!=NULL && src!=NULLSTR)
    {
        c_str = strdup(src);
        l_size = strlen(c_str);
    }
}

void TMAString::assignChars(char *src)
{
    free();
    if (src!=NULL && src!=NULLSTR)
    {
        c_str = src;
        l_size = strlen(c_str);
    }
}
TMAString::TMAString()
{
    init();
}

TMAString::TMAString(const TMAString& str)
{
    init();
    copyFrom(str.c_str);
}

TMAString::TMAString(const char *str)
{
    init();
    copyFrom(str);
};


TMAString::~TMAString()
{
    free();
}

void TMAString::replaceChar(char origen,char destino)
{
    for (long i=0;i<l_size;i++)
    {
        if (c_str[i]==origen) c_str[i]=destino;
    }
}    

long TMAString::indexOf(const char *substr,long fromindex)
{
    const char *pinit = &c_str[fromindex];
    const char *pindex = strstr(pinit,substr);
    if (pindex==NULL) return -1;
    return (pindex - pinit);
}

long TMAString::indexOf(const TMAString& substr,long fromindex)
{
    return indexOf(substr.c_str,fromindex);
}

TMAString TMAString::substring(long index,long nchars)
{
    TMAString aux;
    if (index<0 || index>l_size || nchars<=0)
    {
        return aux;
    }
    
    char *c_tmp=new char[nchars+1];
    if (c_tmp!=NULL)
    {
        memcpy(c_tmp,&c_str[index],nchars);
        c_tmp[nchars]=0;
        aux.assignChars(c_tmp);
    }
    return aux;
}

TMAString TMAString::substring(long index)
{
    TMAString aux;
    if (index<0 || index>l_size)
    {
        return aux;
    }

    char *c_tmp=&c_str[index];
    aux.assignChars(c_tmp);
    return aux;
}

TMAString TMAString::trim(void)
{
    int begin,end;
    begin = 0;
    end = l_size-1;
    while (c_str[begin]==' ' && begin<end) begin++;
    while (c_str[end]==' ' && end>begin) end--;
    return substring(begin,(end-begin+1));
}

TMAString TMAString::toUpperCase(void)
{
    TMAString aux(c_str);
    char *S=aux.c_str;
    while (*S!=0)
    {
        *S=toupper(*S); S++;
    }
    return aux;
}

TMAString TMAString::toLowerCase(void)
{
    TMAString aux(c_str);
    char *S=aux.c_str;
    while (*S!=0)
    {
        *S=tolower(*S); S++;
    }
    return aux;
}

TMAString TMAString::valueOf(int num)
{
    char buff[40];
    sprintf(buff,"%d",num);
    return TMAString(buff);
}

TMAString TMAString::valueOf(long num)
{
    char buff[40];
    sprintf(buff,"%ld",num);
    return TMAString(buff);
}

TMAString TMAString::valueOf(float num)
{
    char buff[40];
    sprintf(buff,"%f",num);
    return TMAString(buff);
}


TMAString TMAString::valueOf(double num)
{
    char buff[40];
    sprintf(buff,"%f",num);
    return TMAString(buff);
}

const int TMAString::intValue(void)
{
    return atoi(c_str);
}

const long TMAString::longValue(void)
{
    return atol(c_str);
}

const float TMAString::floatValue(void)
{
    return atof(c_str);
}

const double TMAString::doubleValue(void)
{
    char *end;
    return strtod(c_str,&end);
}

int TMAString::compare(const TMAString& str)
{
    return strcmp(c_str,str.c_str);
}

bool TMAString::isEqual(const TMAString& str)
{
    return strcmp(c_str,str.c_str)==0;
}

int TMAString::compare(const char *str)
{
    return strcmp(c_str,str);
}

bool TMAString::isEqual(const char *str)
{
    return strcmp(c_str,str)==0;
}


TMAString TMAString::operator + (const char *str)
{
    if (!str) return *this;
    
    char *buff;
    if (c_str!=NULL)
    {
        buff = new char [l_size + strlen(str) + 1];
        buff[0]=0;
        strcat(buff,c_str);
        strcat(buff,str);
    }
    else
    {
        buff = strdup(str);
    }
    TMAString ret;
    ret.assignChars(buff);
    return ret;
}

TMAString TMAString::operator + (const TMAString& str)
{
    return (*this + str.c_str);
}

TMAString& TMAString::operator = (char *str)
{
    if (c_str != str) copyFrom(str);
    return *this;
}
 
TMAString& TMAString::operator = (TMAString str)
{
    if (this!=&str) copyFrom(str.c_str);
    return *this;
}   
    
#endif
